@@ -4,7 +4,7 @@
 #  ninix-install.py - a command-line installer for ninix
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -16,7 +16,7 @@
 #
 
 # env.var: NINIX_HOME, NINIX_USER, NINIX_ARCHIVE, TMPDIR
-# command: cp, mv, rm, mkdir, lha, zip, find, chmod
+# command: cp, mv, rm, mkdir, zip, find, chmod
 
 
 import getopt
@@ -195,8 +195,6 @@ def install(filename, filetype, target, homedir):
     if filename.lower().endswith('.nar') or \
        filename.lower().endswith('.zip'):
         archiver = 'unzip -o %(f)s -d %(d)s'
-    elif filename.lower().endswith('.lzh'):
-        archiver = 'lha xfw=%(d)s %(f)s'
     else:
         fatal('unknown archive format')
     # extract files from the archive
@@ -330,8 +328,6 @@ def download(url, basedir):
     if filename.lower().endswith('.nar') or \
        filename.lower().endswith('.zip'):
         archiver = 'unzip -t'
-    elif filename.lower().endswith('.lzh'):
-        archiver = 'lha t'
     else:
         fatal('unknown archive format')
     if os.system('%s %s >/dev/null 2>&1' % (archiver, escape(filename))):
@@ -462,7 +458,7 @@ def install_redo_ghost(archive, tmpdir, homedir):
     path = os.path.join(ghost_src, 'alias.txt')
     if os.path.exists(path):
         filelist.append((path, os.path.join(ghost_dst, 'alias.txt')))
-        aliases = ninix.alias.open(path)
+        aliases = ninix.alias.create_from_file(path)
     else:
         aliases = {}
     # find pseudo AI
@@ -500,7 +496,7 @@ def install_redo_ghost(archive, tmpdir, homedir):
         inst_dst = ninix.home.read_install_txt(prefix)
         if not inst_dst or inst_dst.get('name') != inst.get('name'):
             mode |= INTERACTIVE
-        if inst.getint('refresh', 0):
+        if inst.get_with_type('refresh', int, 0):
             # uninstall older versions of the ghost
             if confirm_removal(prefix):
                 mask = inst.get('refreshundeletemask', '').lower().split(':')
@@ -552,7 +548,7 @@ def install_redo_shell(archive, tmpdir, homedir):
         inst_dst = ninix.home.read_install_txt(prefix)
         if not inst_dst or inst_dst.get('name') != inst.get('name'):
             mode |= INTERACTIVE
-        if inst.getint('refresh', 0):
+        if inst.get_with_type('refresh', int, 0):
             # uninstall older versions of the ghost
             if confirm_removal(prefix):
                 mask = inst.get('refreshundeletemask', '').lower().split(':')
@@ -737,7 +733,7 @@ def install_inverse_ghost(archive, tmpdir, homedir):
         inst_dst = ninix.home.read_install_txt(prefix)
         if not inst_dst or inst_dst.get('name') != inst.get('name'):
             mode |= INTERACTIVE
-        if inst.getint('refresh', 0):
+        if inst.get_with_type('refresh', int, 0):
             # uninstall older versions of the ghost
             if confirm_removal(prefix):
                 mask = inst.get('refreshundeletemask', '').lower().split(':')
@@ -787,7 +783,7 @@ def install_inverse_shell(archive, tmpdir, homedir):
         inst_dst = ninix.home.read_install_txt(prefix)
         if not inst_dst or inst_dst.get('name') != inst.get('name'):
             mode |= INTERACTIVE
-        if inst.getint('refresh', 0):
+        if inst.get_with_type('refresh', int, 0):
             # uninstall older versions of the shell
             if confirm_removal(prefix):
                 mask = inst.get('refreshundeletemask', '').lower().split(':')
@@ -821,7 +817,7 @@ def find_inverse_shell(srcdir, dstdir):
     path = os.path.join(srcdir, 'surface.txt')
     if os.path.exists(path):
         filelist.append((path, os.path.join(dstdir, 'surface.txt')))
-        config = ninix.config.open(path)
+        config = ninix.config.create_from_file(path)
         for name, subdir in config.items():
             dirlist.append((name, subdir.lower()))
     else:
@@ -866,7 +862,7 @@ def install_balloon(archive, srcdir, homedir):
         inst_dst = ninix.home.read_install_txt(dstdir)
         if not inst_dst or inst_dst.get('name') != inst.get('name'):
             mode |= INTERACTIVE
-        if inst.getint('refresh', 0):
+        if inst.get_with_type('refresh', int, 0):
             # uninstall older versions of the balloon
             if confirm_removal(dstdir):
                 mask = inst.get('refreshundeletemask', '').lower().split(':')
@@ -1023,7 +1019,7 @@ def find_surface_set(srcdir, dstdir):
     filename_alias = {}
     path = os.path.join(srcdir, 'alias.txt')
     if os.path.exists(path):
-        dic = ninix.alias.open(path)
+        dic = ninix.alias.create_from_file(path)
         for basename, alias in dic.iteritems():
             if basename.startswith('surface'):
                 filename_alias[alias] = basename
