@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -20,29 +20,41 @@ if 'DISPLAY' in os.environ:
     import gobject
     import pango
     import cairo
+    import glib
 
 import ninix.pix
 
 
 class Balloon:
 
-    def __init__(self, sakura, prefs, debug=0):
-        self.__sakura = sakura
-        self.prefs = prefs
+    def __init__(self, callback, debug=0):
+        self.callback = callback
         self.debug = debug
         self.synchronized = []
         self.__scalling = 0
         self.__scale = 100 # %
-        self.__use_pna = 0
+        self.__use_pna = False
         self.__alpha_channel = None
         self.user_interaction = False
         self.window = []
         # create communicatebox
-        self.communicatebox = CommunicateBox(self, sakura, debug)
+        communicatebox_callback = {
+            'reset_user_interaction': self.reset_user_interaction,
+            'notify_event':  self.callback['notify_event'],
+            }
+        self.communicatebox = CommunicateBox(communicatebox_callback, debug)
         # create teachbox
-        self.teachbox = TeachBox(self, sakura, debug)
+        teachbox_callback = {
+            'reset_user_interaction': self.reset_user_interaction,
+            'notify_user_teach':  self.callback['notify_user_teach'],
+            }
+        self.teachbox = TeachBox(teachbox_callback, debug)
         # create inputbox
-        self.inputbox = InputBox(self, sakura, debug)
+        inputbox_callback = {
+            'reset_user_interaction': self.reset_user_interaction,
+            'notify_event': self.callback['notify_event'],
+            }
+        self.inputbox = InputBox(inputbox_callback, debug)
 
     def reset_user_interaction(self):
         self.user_interaction = False
@@ -71,10 +83,7 @@ class Balloon:
             self.window[side].reset_text_count()
 
     def set_use_pna(self, flag):
-        if flag:
-            self.__use_pna = 1
-        else:
-            self.__use_pna = 0
+        self.__use_pna = bool(flag)
         for balloon_window in self.window:
             balloon_window.set_use_pna(flag)
             balloon_window.reset_balloon()
@@ -107,7 +116,7 @@ class Balloon:
         window.set_resizable(False)
         window.set_skip_pager_hint(False)
         window.set_skip_taskbar_hint(True)
-        window.font_name = self.prefs.get('balloon_fonts')
+        window.font_name = self.callback['get_preference']('balloon_fonts')
         window.connect('delete_event', self.delete)
         window.realize()
         return window
@@ -119,8 +128,7 @@ class Balloon:
         return False
 
     def delete(self, window, event):
-        self.__sakura.finalize()
-        return False
+        return True
 
     def finalize(self):
         for balloon_window in self.window:
@@ -156,21 +164,14 @@ class Balloon:
                 communicate2 = value # teach box
             elif key == 'c3':
                 communicate3 = value # input box
-        self.balloon1 = balloon1 ## FIXME
+        self.balloon0 = balloon0
+        self.balloon1 = balloon1
         # create balloon windows
         for balloon_window in self.window:
             balloon_window.destroy()
         self.window = []
-        for name, side, id_format, balloon in [('sakura', 0, 's%d', balloon0),
-                                               ('kero', 1, 'k%d', balloon1)]:
-            gtk_window = self.create_gtk_window(''.join(('balloon.', name)))
-            balloon_window = BalloonWindow(
-                gtk_window, side, self.__sakura, desc, balloon,
-                id_format, self.__use_pna, self.__alpha_channel, self.debug)
-            self.window.append(balloon_window)
-        for balloon_window in self.window:
-            balloon_window.set_scalling(self.__scalling)
-            balloon_window.set_scale(self.__scale)
+        self.add_window(0)
+        self.add_window(1)
         # configure communicatebox
         self.communicatebox.new(desc, communicate1)
         # configure teachbox
@@ -179,19 +180,32 @@ class Balloon:
         self.inputbox.new(desc, communicate3)
 
     def add_window(self, side):
-        assert side >= 2 and len(self.window) == side ## FIXME
-        gtk_window = self.create_gtk_window('balloon.char%d' % side)
-        id_format = 'k%d'
-        balloon = self.balloon1 ## FIXME
+        assert len(self.window) == side
+        if side == 0:
+            name = 'balloon.sakura'
+            id_format = 's%d'
+            balloon = self.balloon0
+        elif side == 1:
+            name = 'balloon.kero'
+            id_format = 'k%d'
+            balloon = self.balloon1
+        else:
+            name = 'balloon.char%d' % side
+            id_format = 'k%d'
+            balloon = self.balloon1
+        gtk_window = self.create_gtk_window(name)
+        balloon_window_callback = self.callback.copy() ## FIXME
         balloon_window = BalloonWindow(
-            gtk_window, side, self.__sakura, self.desc, balloon,
+            gtk_window, side, balloon_window_callback, self.desc, balloon,
             id_format, self.__use_pna, self.__alpha_channel, self.debug)
         self.window.append(balloon_window)
+        balloon_window.set_scalling(self.__scalling)
         balloon_window.set_scale(self.__scale)
-        balloon_window.set_balloon(0) ## FIXME
+        if side > 1:
+            balloon_window.set_balloon(0) ## FIXME
 
     def reset_fonts(self):
-        font_name = self.prefs.get('balloon_fonts')
+        font_name = self.callback['get_preference']('balloon_fonts')
         for window in self.window:
             window.window.font_name = font_name
             window.update_gc()
@@ -207,7 +221,7 @@ class Balloon:
             return (0, 0)
 
     def reset_balloon(self):
-        scale = scale # %
+        scale = self.__scale # %
         use_pna = 0
         for balloon_window in self.window:
             balloon_window.set_use_pna(use_pna)
@@ -242,10 +256,7 @@ class Balloon:
 
     def is_shown(self, side):
         if len(self.window) > side:
-            if self.window[side].is_shown():
-                return 1
-            else:
-                return 0
+            return 1 if self.window[side].is_shown() else 0
         else:
             return 0
 
@@ -368,7 +379,7 @@ class Balloon:
     def open_teachbox(self):
         if not self.user_interaction:
             self.user_interaction = True
-            self.__sakura.notify_event('OnTeachStart')
+            self.callback['notify_event']('OnTeachStart')
             self.teachbox.show()
 
     def open_inputbox(self, symbol, limittime=-1, default=None):
@@ -380,11 +391,11 @@ class Balloon:
 
 class BalloonWindow:
 
-    def __init__(self, window, side, sakura, desc, balloon,
+    def __init__(self, window, side, callback, desc, balloon,
                  id_format, use_pna, alpha, debug):
         self.window = window
         self.side = side
-        self.__sakura = sakura
+        self.callback = callback
         self.desc = desc
         self.balloon = balloon
         self.balloon_id = None
@@ -421,24 +432,23 @@ class BalloonWindow:
                               gtk.gdk.POINTER_MOTION_MASK|
                               gtk.gdk.POINTER_MOTION_HINT_MASK|
                               gtk.gdk.SCROLL_MASK)
-        self.callbacks = []
         for signal, func in [('expose_event',        self.redraw),
                              ('button_press_event',  self.button_press),
                              ('motion_notify_event', self.motion_notify),
                              ('scroll_event',        self.scroll)]:
-            self.callbacks.append(self.darea.connect(signal, func))
+            self.darea.connect(signal, func)
         self.window.add(self.darea)
         self.darea.realize()
         self.darea.window.set_back_pixmap(None, False)
-        mask_r = desc.getint('maskcolor.r', 128)
-        mask_g = desc.getint('maskcolor.g', 128)
-        mask_b = desc.getint('maskcolor.b', 128)
+        mask_r = desc.get_with_type('maskcolor.r', int, 128)
+        mask_g = desc.get_with_type('maskcolor.g', int, 128)
+        mask_b = desc.get_with_type('maskcolor.b', int, 128)
         self.cursor_color = '#%02x%02x%02x' % (mask_r, mask_g, mask_b)
-        text_r = desc.getint(['font.color.r', 'fontcolor.r'], 0)
-        text_g = desc.getint(['font.color.g', 'fontcolor.g'], 0)
-        text_b = desc.getint(['font.color.b', 'fontcolor.b'], 0)
+        text_r = desc.get_with_type(['font.color.r', 'fontcolor.r'], int, 0)
+        text_g = desc.get_with_type(['font.color.g', 'fontcolor.g'], int, 0)
+        text_b = desc.get_with_type(['font.color.b', 'fontcolor.b'], int, 0)
         self.text_normal_color = '#%02x%02x%02x' % (text_r, text_g, text_b)
-        if desc.getint('maskmethod') == 1:
+        if desc.get_with_type('maskmethod', int) == 1:
             text_r = 255 - text_r
             text_g = 255 - text_g
             text_b = 255 - text_b
@@ -461,10 +471,7 @@ class BalloonWindow:
                                 (pixbuf.get_width(), pixbuf.get_height()))
 
     def set_use_pna(self, flag):
-        if flag:
-            self.__use_pna = 1
-        else:
-            self.__use_pna = 0
+        self.__use_pna = bool(flag)
         self.reset_pixbuf_cache()
 
     def set_alpha_channel(self, value):
@@ -474,10 +481,7 @@ class BalloonWindow:
         self.redraw()
 
     def get_scale(self):
-        if self.__scalling:
-            return self.__scale
-        else:
-            return 100 # %
+        return self.__scale if self.__scalling else 100 # %
 
     def set_scale(self, scale):
         self.__scale = scale # %
@@ -519,8 +523,8 @@ class BalloonWindow:
         x, y = self.arrow[0]
         pixbuf, (w, h) = self.pixbuf['arrow0']
         scale = self.get_scale()
-        w = max(1, int(w * scale / 100))
-        h = max(1, int(h * scale / 100))
+        w = max(8, int(w * scale / 100))
+        h = max(8, int(h * scale / 100))
         mask = gtk.gdk.Pixmap(None, w, h, 1)
         pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
         pixbuf.render_threshold_alpha(mask, 0, 0, 0, 0, w, h, 255)
@@ -528,8 +532,8 @@ class BalloonWindow:
         self.arrow0_gc = self.new_mask_gc(mask, x, y)
         x, y = self.arrow[1]
         pixbuf, (w, h) = self.pixbuf['arrow1']
-        w = max(1, int(w * scale / 100))
-        h = max(1, int(h * scale / 100))
+        w = max(8, int(w * scale / 100))
+        h = max(8, int(h * scale / 100))
         mask = gtk.gdk.Pixmap(None, w, h, 1)
         pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
         pixbuf.render_threshold_alpha(mask, 0, 0, 0, 0, w, h, 255)
@@ -539,8 +543,8 @@ class BalloonWindow:
         if self.side == 0 and 'sstp' in self.pixbuf:
             x, y = self.sstp[0]
             pixbuf, (w, h) = self.pixbuf['sstp']
-            w = max(1, int(w * scale / 100))
-            h = max(1, int(h * scale / 100))
+            w = max(8, int(w * scale / 100))
+            h = max(8, int(h * scale / 100))
             mask = gtk.gdk.Pixmap(None, w, h, 1)
             pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
             pixbuf.render_threshold_alpha(mask, 0, 0, 0, 0, w, h, 255)
@@ -548,7 +552,7 @@ class BalloonWindow:
             self.sstp_gc = self.new_mask_gc(mask, x, y)
         # font
         default_size = 12 # for Windows environment
-        size = self.desc.getint(['font.height', 'font.size'], default_size)
+        size = self.desc.get_with_type(['font.height', 'font.size'], int, default_size)
         self.layout = pango.Layout(self.darea.get_pango_context())
         self.font_desc = pango.FontDescription(self.window.font_name)
         pango_size = self.font_desc.get_size()
@@ -565,7 +569,7 @@ class BalloonWindow:
         # font for sstp message
         if self.side == 0:
             default_size = 10 # for Windows environment
-            size = self.desc.getint('sstpmessage.font.height', default_size)
+            size = self.desc.get_with_type('sstpmessage.font.height', int, default_size)
             self.sstp_layout = pango.Layout(self.darea.get_pango_context())
             self.sstp_font_desc = pango.FontDescription(self.window.font_name)
             pango_size = self.sstp_font_desc.get_size()
@@ -580,24 +584,26 @@ class BalloonWindow:
         else:
             sstp_font_height = 0
         # font metrics
-        origin_x = self.config_getint(
-            'origin.x',
-            self.config_getint('zeropoint.x',
-                               self.config_getint('validrect.left', 14)))
-        origin_y = self.config_getint(
-            'origin.y',
-            self.config_getint('zeropoint.y',
-                               self.config_getint('validrect.top', 14)))
-        wpx = self.config_getint(
-            'wordwrappoint.x',
-            self.config_getint('validrect.right', -14))
+        origin_x = self.__get_with_scaling(
+            'origin.x', int,
+            self.__get_with_scaling(
+                'zeropoint.x', int,
+                self.__get_with_scaling('validrect.left', int, 14)))
+        origin_y = self.__get_with_scaling(
+            'origin.y', int,
+            self.__get_with_scaling(
+                'zeropoint.y', int,
+                self.__get_with_scaling('validrect.top', int, 14)))
+        wpx = self.__get_with_scaling(
+            'wordwrappoint.x', int, 
+            self.__get_with_scaling('validrect.right', int, -14))
         if wpx > 0:
             line_width = wpx - origin_x
         elif wpx < 0:
             line_width = self.width - origin_x + wpx
         else:
             line_width = self.width - origin_x * 2
-        wpy = self.config_getint('validrect.bottom', -14)
+        wpy = self.__get_with_scaling('validrect.bottom', int, -14)
         if wpy > 0:
             text_height = min(wpy, self.height) - origin_y
         elif wpy < 0:
@@ -608,7 +614,7 @@ class BalloonWindow:
         self.lines = text_height / line_height
         self.line_regions = []
         y = origin_y
-        for i in range(self.lines + 1):
+        for _ in range(self.lines + 1):
             self.line_regions.append((origin_x, y, line_width, line_height))
             y = y + line_height
         self.line_width = line_width
@@ -620,11 +626,12 @@ class BalloonWindow:
             self.sstp_region = (x, y, w, h)
 
     def update_line_regions(self, offset, new_y):
-        origin_y = self.config_getint(
-            'origin.y',
-            self.config_getint('zeropoint.y',
-                               self.config_getint('validrect.top', 14)))
-        wpy = self.config_getint('validrect.bottom', -14)
+        origin_y = self.__get_with_scaling(
+            'origin.y', int, 
+            self.__get_with_scaling(
+                'zeropoint.y', int,
+                self.__get_with_scaling('validrect.top', int, 14)))
+        wpy = self.__get_with_scaling('validrect.bottom', int, -14)
         if wpy > 0:
             text_height = min(wpy, self.height) - origin_y
         elif wpy < 0:
@@ -664,8 +671,8 @@ class BalloonWindow:
         x, y = self.position
         pixbuf, (w, h) = self.pixbuf[balloon_id]
         scale = self.get_scale()
-        self.width = max(1, int(w * scale / 100))
-        self.height = max(1, int(h * scale / 100))
+        self.width = max(8, int(w * scale / 100))
+        self.height = max(8, int(h * scale / 100))
         self.darea.set_size_request(self.width, self.height)
         self.balloon_pixbuf = pixbuf.scale_simple(
             self.width, self.height, gtk.gdk.INTERP_BILINEAR)
@@ -679,7 +686,7 @@ class BalloonWindow:
         if self.__shown:
             self.update_gc()
         if reset_position:
-            self.__sakura.position_balloons()
+            self.callback['position_balloons']()
 
     def set_direction(self, dir):
         if self.direction != dir:
@@ -688,24 +695,23 @@ class BalloonWindow:
 
     def config_adjust(self, name, base, default_value):
         path, config = self.balloon[self.balloon_id]
-        value = config.adjust(name, base)
+        value = config.get_with_type(name, int)
         if value is None:
-            value = self.desc.adjust(name, base)
-            if value is None:
-                if default_value < 0:
-                    value = base + default_value
-                else:
-                    value = default_value
+            value = self.desc.get_with_type(name, int)
+        if value is None:
+            value = default_value
+        if value < 0:
+            value = base + value
         return int(value * self.get_scale() / 100)
 
-    def config_getint(self, name, default_value):
+    def __get_with_scaling(self, name, conv, default_value):
         path, config = self.balloon[self.balloon_id]
-        value = config.getint(name)
+        value = config.get_with_type(name, conv)
         if value is None:
-            value = self.desc.getint(name)
+            value = self.desc.get_with_type(name, conv)
             if value is None:
                 value = default_value
-        return int(value * self.get_scale() / 100)
+        return conv(value * self.get_scale() / 100)
 
     def __move(self):
         x, y = self.get_position()
@@ -720,32 +726,29 @@ class BalloonWindow:
         return self.position
 
     def destroy(self, finalize=0):
-        for tag in self.callbacks:
-            self.darea.disconnect(tag)
         self.window.remove(self.darea)
         self.darea.destroy()
         self.window.destroy()
 
     def is_shown(self):
-        if self.__shown:
-            return 1
-        else:
-            return 0
+        return 1 if self.__shown else 0
 
     def show(self):
-        if not self.__shown:
-            self.window.show()
-            self.__shown = 1
-            # make sure window is in its position
-            self.__move()
-            self.update_gc()
-            self.raise_()
+        if self.__shown:
+            return
+        self.window.show()
+        self.__shown = 1
+        # make sure window is in its position
+        self.__move()
+        self.update_gc()
+        self.raise_()
 
     def hide(self):
-        if self.__shown:
-            self.window.hide()
-            self.__shown = 0
-            self.images = []
+        if not self.__shown:
+            return
+        self.window.hide()
+        self.__shown = 0
+        self.images = []
 
     def raise_(self):
         if self.__shown:
@@ -814,7 +817,7 @@ class BalloonWindow:
         x, y = self.arrow[1]
         w, h = self.arrow1_pixbuf[2]
         if self.lineno + self.lines < len(self.text_buffer) or \
-           self.__sakura.is_paused():
+           self.callback['is_paused']():
             self.darea.window.draw_pixbuf(
                 self.arrow1_gc, self.arrow1_pixbuf[0], 0, 0, x, y, w, h)
         elif clear:
@@ -827,7 +830,7 @@ class BalloonWindow:
         cr.set_source_pixbuf(self.balloon_pixbuf, 0, 0)
         cr.paint_with_alpha(self.__alpha_channel)
 
-    def set_markup(self, index, text): # FIXME: escape & -> &amp;
+    def set_markup(self, index, text):
         tags_ = ('sup', 'sub', 's', 'u')
         count_ = {}
         for tag_ in tags_:
@@ -836,10 +839,15 @@ class BalloonWindow:
         for sl, sn, tag in self.meta_buffer:
             if sl == index:
                 markup_list.append((sn, tag))
+        if not markup_list:
+            return glib.markup_escape_text(text)
         markup_list.sort()
         markup_list.reverse()
+        pn = len(text)
         for sn, tag in markup_list:
-            text = ''.join((text[:sn], tag, text[sn:]))
+            text = ''.join((text[:sn], tag,
+                            glib.markup_escape_text(text[sn:pn]), text[pn:]))
+            pn = sn
             if tag[1] == '/':
                 tag_ = tag[2:-1]
                 assert tag_ in tags_
@@ -872,8 +880,8 @@ class BalloonWindow:
         for i in range(len(self.images)):
             pixbuf, (w, h), (x, y) = self.images[i]
             scale = self.get_scale()
-            w = max(1, int(w * scale / 100))
-            h = max(1, int(h * scale / 100))
+            w = max(8, int(w * scale / 100))
+            h = max(8, int(h * scale / 100))
             if x == 'centerx':
                 bw, bh = self.get_balloon_size()
                 x = (bw - w) / 2
@@ -1034,11 +1042,11 @@ class BalloonWindow:
             if self.selection != new_selection:
                 sl, sn, el, en, link_id, raw_text, text = \
                     self.link_buffer[new_selection]
-                self.__sakura.notify_event(
+                self.callback['notify_event'](
                     'OnChoiceEnter', raw_text, link_id, self.selection)
         else:
             if self.selection is not None:
-                self.__sakura.notify_event('OnChoiceEnter')
+                self.callback['notify_event']('OnChoiceEnter')
         if new_selection == self.selection:
             return 0
         else:
@@ -1078,13 +1086,10 @@ class BalloonWindow:
         return True
 
     def button_press(self, darea, event):
-        self.__sakura.reset_idle_time()
-        if event.type == gtk.gdk.BUTTON_PRESS:
-            click = 1
-        else:
-            click = 2
-        if self.__sakura.is_paused():
-            self.__sakura.notify_balloon_click(event.button, click, self.side)
+        self.callback['reset_idle_time']()
+        click = 1 if event.type == gtk.gdk.BUTTON_PRESS else 2
+        if self.callback['is_paused']():
+            self.callback['notify_balloon_click'](event.button, click, self.side)
             return True
         # arrows
         px = int(event.x)
@@ -1110,11 +1115,11 @@ class BalloonWindow:
         if self.selection is not None:
             sl, sn, el, en, link_id, raw_text, text = \
                 self.link_buffer[self.selection]
-            self.__sakura.notify_link_selection(
+            self.callback['notify_link_selection'](
                 link_id, raw_text, self.selection)
             return True
         # balloon's background
-        self.__sakura.notify_balloon_click(event.button, click, self.side)
+        self.callback['notify_balloon_click'](event.button, click, self.side)
         return True
 
     def clear_text(self):
@@ -1257,7 +1262,6 @@ class BalloonWindow:
         w = pixbuf.get_width()
         h = pixbuf.get_height()
         self.images.append((pixbuf, (w, h), (x, y)))
-        del pixbuf
         self.redraw()
 
     def draw_last_line(self, column=0):
@@ -1294,12 +1298,10 @@ class CommunicateWindow:
     NAME = ''
     ENTRY = ''
 
-    def __init__(self, balloon, sakura, debug):
-        self.balloon = balloon
-        self.sakura = sakura
+    def __init__(self, callback, debug):
+        self.callback = callback
         self.debug = debug
         self.window = None
-        self.dragged = 0
 
     def new(self, desc, balloon):
         if self.window:
@@ -1315,8 +1317,8 @@ class CommunicateWindow:
         self.window.set_modal(True)
         self.window.set_position(gtk.WIN_POS_CENTER)
         self.window.realize()
-        w = desc.getint('communicatebox.width', 250)
-        h = desc.getint('communicatebox.height', -1)
+        w = desc.get_with_type('communicatebox.width', int, 250)
+        h = desc.get_with_type('communicatebox.height', int, -1)
         self.entry = gtk.Entry()
         self.entry.connect('activate', self.activate)
         self.entry.set_size_request(w, h)
@@ -1335,8 +1337,8 @@ class CommunicateWindow:
             gtk_image.set_padding(0, 0)
             gtk_image.set_from_pixmap(image, mask)
             gtk_image.show()
-            x = desc.getint('communicatebox.x', 10)
-            y = desc.getint('communicatebox.y', 20)
+            x = desc.get_with_type('communicatebox.x', int, 10)
+            y = desc.get_with_type('communicatebox.y', int, 20)
             fixed = gtk.Fixed()
             fixed.put(gtk_image, 0, 0)
             fixed.put(self.entry, x, y)
@@ -1405,14 +1407,14 @@ class CommunicateBox(CommunicateWindow):
     def delete(self, widget, event):
         self.window.hide()
         self.cancel()
-        self.balloon.reset_user_interaction()
+        self.callback['reset_user_interaction']()
         return True
 
     def key_press(self, widget, event):
         if event.keyval == gtk.keysyms.Escape:
             self.window.hide()
             self.cancel()
-            self.balloon.reset_user_interaction()
+            self.callback['reset_user_interaction']()
             return True
         return False
 
@@ -1431,7 +1433,7 @@ class CommunicateBox(CommunicateWindow):
         if data:
             data = unicode(data, 'utf-8')
         if data is not None:
-            self.sakura.notify_event('OnCommunicate', 'user', data)
+            self.callback['notify_event']('OnCommunicate', 'user', data)
 
 class TeachBox(CommunicateWindow):
 
@@ -1447,8 +1449,8 @@ class TeachBox(CommunicateWindow):
     def send(self, data):
         if data:
             data = unicode(data, 'utf-8')
-        self.sakura.notify_user_teach(data)
-        self.balloon.reset_user_interaction()
+        self.callback['notify_user_teach'](data)
+        self.callback['reset_user_interaction']()
 
 class InputBox(CommunicateWindow):
 
@@ -1503,13 +1505,13 @@ class InputBox(CommunicateWindow):
             data = ''
         ## CHECK: symbol
         if self.symbol == 'OnUserInput' and \
-           self.sakura.notify_event('OnUserInput', data):
+           self.callback['notify_event']('OnUserInput', data):
             pass
-        elif self.sakura.notify_event('OnUserInput', self.symbol, data):
+        elif self.callback['notify_event']('OnUserInput', self.symbol, data):
             pass
-        elif self.sakura.notify_event(self.symbol, data):
+        elif self.callback['notify_event'](self.symbol, data):
             pass
-        self.balloon.reset_user_interaction()
+        self.callback['reset_user_interaction']()
 
 
 def test():
