@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
-#  Copyright (C) 2003-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -11,52 +11,27 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
-import __builtin__
 import codecs
 
 class Config(dict):
 
+    def get_with_type(self, name, conv, default=None):
+        value = self.get(name)
+        if value is None:
+            return default
+        ##assert conv is not None
+        try:
+            return conv(value)
+        except ValueError:
+            return default # XXX
+
     def get(self, name, default=None):
-        if isinstance(name, list):
-            keylist = name
-        else:
-            keylist = [name]
+        keylist = name if isinstance(name, list) else [name]
         for key in keylist:
             if key in self:
                 return self[key]
         return default
 
-    def getint(self, name, default=None):
-        value = self.get(name)
-        if value:
-            try:
-                return int(value)
-            except ValueError:
-                pass
-        return default
-
-    def getfloat(self, name, default=None):
-        value = self.get(name)
-        if value:
-            try:
-                return float(value)
-            except ValueError:
-                pass
-        return default
-
-    def adjust(self, name, base, default=None):
-        value = self.get(name)
-        if value:
-            try:
-                if value.startswith('--'):
-                    return int(value[1:])
-                value = int(value)
-            except (TypeError, ValueError):
-                value = default
-        if value is not None and value < 0:
-            value = base + value
-        return value
-
     def __str__(self):
         buf = []
         for item in self.iteritems():
@@ -64,17 +39,17 @@ class Config(dict):
         return ''.join(buf)
 
 
-def open(path):
-    f = __builtin__.open(path)
+def create_from_file(path):
+    f = open(path)
     buf = []
     for line in f:
         if not line.strip():
             continue
         buf.append(line)
     f.close()
-    return new_config(buf)
+    return create_from_buffer(buf)
 
-def new_config(buf):
+def create_from_buffer(buf):
     dic = Config()
     charset = 'Shift_JIS' # default
     for line in buf:
