@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  dll.py - a pseudo DLL (SHIORI/SAORI API support) module for ninix
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -21,6 +21,11 @@ import codecs
 
 class SAORI:
 
+    RESPONSE = {204: 'SAORI/1.0 204 No Content\r\n\r\n',
+                400: 'SAORI/1.0 400 Bad Request\r\n\r\n',
+                500: 'SAORI/1.0 500 Internal Server Error\r\n\r\n',
+                }
+
     def __init__(self):
         self.loaded = 0
 
@@ -56,17 +61,14 @@ class SAORI:
     def request(self, req):
         req_type, argument = self.evaluate_request(req)
         if not req_type:
-            return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+            return self.RESPONSE[400]
         elif req_type == 'GET Version':
-            return 'SAORI/1.0 204 No Content\r\n\r\n'
+            return self.RESPONSE[204]
         elif req_type == 'EXECUTE':
             result = self.execute(argument)
-            if result is None:
-                return 'SAORI/1.0 204 No Content\r\n\r\n'
-            else:
-                return result
+            return self.RESPONSE[204] if result is None else result
         else:
-            return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+            return self.RESPONSE[400]
 
     def execute(self, args):
         return None
@@ -134,7 +136,6 @@ class Library:
             if self.__type == 'saori':
                 if getattr(module, 'Saori', None):
                     saori = module.Saori()
-                    saori.prefs = self.__sakura.prefs
                     if getattr(saori, 'need_ghost_backdoor', None):
                         saori.need_ghost_backdoor(self.__sakura)
                 else:
