@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  bln.py - a easyballoon compatible Saori module for ninix
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -13,7 +13,6 @@
 
 # TODO:
 # - font.face
-# - \l
 
 import os
 import random
@@ -50,17 +49,11 @@ class Saori(SAORI):
         self.__sakura = sakura
 
     def check_import(self):
-        if self.__sakura and gtk:
-            return 1
-        else:
-            return 0
+        return 1 if self.__sakura and gtk else 0
 
     def setup(self):
         self.blns = self.read_bln_txt(self.dir)
-        if self.blns:
-            return 1
-        else:
-            return 0
+        return 1 if self.blns else 0
 
     def read_bln_txt(self, dir):
         blns = {}
@@ -105,12 +98,9 @@ class Saori(SAORI):
 
     def execute(self, argument):
         if not argument:
-            return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+            return self.RESPONSE[400]
         name = argument[0]
-        if len(argument) >= 2:
-            text = argument[1]
-        else:
-            text = ''
+        text = argument[1] if len(argument) >= 2 else ''
         if len(argument) >= 3:
             try:
                 offset_x = int(argument[2])
@@ -125,10 +115,7 @@ class Saori(SAORI):
                 offset_y = 0
         else:
             offset_y = 0
-        if len(argument) >= 5:
-            bln_id = argument[4]
-        else:
-            bln_id = ''
+        bln_id = argument[4] if len(argument) >= 5 else ''
         if len(argument) >= 6 and argument[5] in ['1', '2']:
             update = int(argument[5])
         else:
@@ -140,7 +127,7 @@ class Saori(SAORI):
                 del bln[bln_id]
             if text:
                 if update == 0 or bln_id not in bln:
-                    bln[bln_id] = Balloon(self.__sakura, self.prefs, self.dir,
+                    bln[bln_id] = Balloon(self.__sakura, self.dir,
                                           data, text,
                                           offset_x, offset_y, name, bln_id)
                 else:
@@ -158,158 +145,65 @@ class Saori(SAORI):
 
 class Balloon:
 
-    def __init__(self, sakura, prefs, dir, data,
+    def __init__(self, sakura, dir, data,
                  text, offset_x, offset_y, name, bln_id):
         self.dir = dir
         self.__sakura = sakura
-        self.prefs = prefs
-        self.dragged = 0
         self.name = name
         self.id = bln_id
-        if 'position' not in data:
-            self.position = 'sakura'
-        else:
-            self.position = data['position']
-        self.window = gtk.Window(type=gtk.WINDOW_POPUP)
-        self.window.connect('delete_event', self.delete)
-        if 'clickerase' in data and data['clickerase'] == 'off':
-            self.clickerase = 'off'
-        else:
-            self.clickerase = 'on'
-        if 'dragmove.horizontal' in data and \
-           data['dragmove.horizontal'] == 'on':
-            self.dragmove_horizontal = 1
-        else:
-            self.dragmove_horizontal = 0
-        if 'dragmove.vertical' in data and \
-           data['dragmove.vertical'] == 'on':
-            self.dragmove_vertical = 1
-        else:
-            self.dragmove_vertical = 0
-        self.window.connect('button_press_event', self.button_press)
-        self.window.connect('button_release_event', self.button_release)
-        self.window.connect('motion_notify_event', self.motion_notify)
-        self.window.connect('leave_notify_event', self.leave_notify)
-        self.window.set_events(gtk.gdk.BUTTON_PRESS_MASK|
-                               gtk.gdk.BUTTON_RELEASE_MASK|
-                               gtk.gdk.POINTER_MOTION_MASK|
-                               gtk.gdk.LEAVE_NOTIFY_MASK)
+        self.data = data # XXX
+        self.window = self.create_window()
+        self.position = data.get('position', 'sakura')
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
-        s0_x, s0_y, s0_w, s0_h = self.get_sakura_status('SurfaceSakura')
-        s1_x, s1_y, s1_w, s1_h = self.get_sakura_status('SurfaceKero')
-        b0_x, b0_y, b0_w, b0_h = self.get_sakura_status('BalloonSakura')
-        b1_x, b1_y, b1_w, b1_h = self.get_sakura_status('BalloonKero')
-        if s0_x + s0_w / 2 > left + scrn_w / 2:
-            self.direction0 = 0 # left
+        # -1: left, 1: right
+        if self.position == 'sakura':
+            s0_x, s0_y, s0_w, s0_h = self.get_sakura_status('SurfaceSakura')
+            self.direction = -1 if s0_x + s0_w / 2 > left + scrn_w / 2 else 1
+        elif self.position == 'kero':
+            s1_x, s1_y, s1_w, s1_h = self.get_sakura_status('SurfaceKero')
+            self.direction = -1 if s1_x + s1_w / 2 > left + scrn_w / 2 else 1
         else:
-            self.direction0 = 1 # right
-        if s1_x + s1_w / 2 > left + scrn_w / 2:
-            self.direction1 = 0 # left
-        else:
-            self.direction1 = 1 # right  
-        if ((self.position == 'sakura' and self.direction0) or \
-            (self.position == 'kero' and self.direction1)) and \
-            'skin.right' in data:
-            path = os.path.join(self.dir,
-                                data['skin.right'].replace('\\', '/'))
-        elif (self.position == 'sakura' or self.position == 'kero') and \
-             'skin.left' in data:
-            path = os.path.join(self.dir,
-                                data['skin.left'].replace('\\', '/'))
-        elif 'skin' in data:
-            path = os.path.join(self.dir,
-                                data['skin'].replace('\\', '/'))
+            self.direction = 1 # XXX
+        if self.position in ['sakura', 'kero']:
+            if self.direction == -1:
+                skin = data.get('skin.left', data.get('skin'))
+            else:
+                skin = data.get('skin.right', data.get('skin'))
         else:
+            skin = data.get('skin')
+        if skin is None:
             self.destroy()
             return
+        path = os.path.join(self.dir, skin.replace('\\', '/'))
         try:
             balloon_pixbuf = ninix.pix.create_pixbuf_from_file(path)
         except:
             self.destroy()
             return
-        self.x = left
-        self.y = top
         self.scale = self.get_sakura_status('SurfaceScale') ## FIXME
         w = balloon_pixbuf.get_width()
         h = balloon_pixbuf.get_height()
-        if self.scale != 100:
-            w = max(1, int(w * self.scale / 100))
-            h = max(1, int(h * self.scale / 100))
-            balloon_pixbuf = balloon_pixbuf.scale_simple(
-                w, h, gtk.gdk.INTERP_BILINEAR)
+        w = max(8, int(w * self.scale / 100))
+        h = max(8, int(h * self.scale / 100))
+        balloon_pixbuf = balloon_pixbuf.scale_simple(
+            w, h, gtk.gdk.INTERP_BILINEAR)
         balloon_pixmap, mask_pixmap = balloon_pixbuf.render_pixmap_and_mask(1)
-        del balloon_pixbuf
-        if self.position == 'lefttop':
-            pass
-        elif self.position == 'leftbottom':
-            self.y = top + scrn_h - h
-        elif self.position == 'righttop':
-            self.x = left + scrn_w - w
-        elif self.position == 'rightbottom':
-            self.x = left + scrn_w - w
-            self.y = top + scrn_h - h
-        elif self.position == 'center':
-            self.x = left + (scrn_w - w) / 2
-            self.y = top + (scrn_h - h) / 2
-        elif self.position == 'leftcenter':
-            self.y = top + (scrn_h - h) / 2
-        elif self.position == 'rightcenter':
-            self.x = left + scrn_w -w
-            self.y = top + (scrn_h - h) / 2
-        elif self.position == 'centertop':
-            self.x = left + (scrn_w - w) / 2
-        elif self.position == 'centerbottom':
-            self.x = left + (scrn_w - w) / 2
-            self.y = top + scrn_h - h
-        elif self.position == 'sakura':
-            if self.direction0: # right
-                self.x = s0_x + s0_w
-            else:
-                self.x = s0_x - w
-            self.y = s0_y
-        elif self.position == 'kero':
-            if self.direction1: # right
-                self.x = s1_x + s1_w
-            else:
-                self.x = s1_x - w
-            self.y = s1_y
-        elif self.position == 'sakurab':
-            self.x = b0_x
-            self.y = b0_y
-        elif self.position == 'kerob':
-            self.x = b1_x
-            self.y = b1_y
+        self.x, self.y = self.get_coordinate(w, h)
         if 'offset.x' in data:
-            if (self.position == 'sakura' and not self.direction0) or \
-               (self.position == 'kero' and not self.direction1):
-                self.x -= int(int(data['offset.x']) * self.scale / 100)
-            else:
-                self.x += int(int(data['offset.x']) * self.scale / 100)
+            self.x += self.direction * int(int(data['offset.x']) * self.scale / 100)
         if 'offset.y' in data:
             self.y += int(int(data['offset.y']) * self.scale / 100)
         if 'offset.random' in data:
             self.x += int(int(data['offset.random']) * random.randrange(-1, 2) * self.scale / 100)
             self.y += int(int(data['offset.random']) * random.randrange(-1, 2) * self.scale / 100)
-        if (self.position == 'sakura' and not self.direction0) or \
-           (self.position == 'kero' and not self.direction1):
-            self.x -= int(offset_x * self.scale / 100)
-        else:
-            self.x += int(offset_x * self.scale / 100)
+        self.x += self.direction * int(offset_x * self.scale / 100)
         self.y += int(offset_y * self.scale / 100)
         self.window.move(self.x, self.y)
-        gtk.gdk.flush()
-        self.left = 0
-        self.right = w
-        self.top = 0
-        self.bottom = h
-        if 'disparea.left' in data:
-            self.left = int(int(data['disparea.left']) * self.scale / 100)
-        if 'disparea.right' in data:
-            self.right = int(int(data['disparea.right']) * self.scale / 100)
-        if 'disparea.top' in data:
-            self.top = int(int(data['disparea.top']) * self.scale / 100)
-        if 'disparea.bottom' in data:
-            self.bottom = int(int(data['disparea.bottom']) * self.scale / 100)
+        gtk.gdk.flush() # XXX
+        self.left = int(int(data.get('disparea.left', 0)) * self.scale / 100)
+        self.right = int(int(data.get('disparea.right', w)) * self.scale / 100)
+        self.top = int(int(data.get('disparea.top', 0)) * self.scale / 100)
+        self.bottom = int(int(data.get('disparea.bottom', h)) * self.scale / 100)
         self.window.shape_combine_mask(mask_pixmap, 0, 0)
         self.script = None
         self.darea = gtk.DrawingArea()
@@ -333,49 +227,27 @@ class Balloon:
                 self.surface_gc.foreground = cmap.alloc_color(
                     '#%02x%02x%02x' % (fontcolor_r, fontcolor_g, fontcolor_b))
             default_font_size = 12 # for Windows environment
-            if 'font.size' in data:
-                font_size = int(data['font.size'])
-            else:
-                font_size = default_font_size
+            font_size = int(data.get('font.size', default_font_size))
             font_size = font_size * 3 / 4 # convert from Windows to GTK+
-            font_size = font_size * self.scale / 100
-            font_size *= pango.SCALE
+            font_size = font_size * self.scale / 100 * pango.SCALE
             self.layout = pango.Layout(self.darea.get_pango_context())
             self.font_desc = pango.FontDescription()
             self.font_desc.set_family('Sans') # FIXME
             self.font_desc.set_size(font_size)
-            if 'font.bold' in data and data['font.bold'] == 'on':
+            if data.get('font.bold') == 'on':
                 self.font_desc.set_weight(pango.WEIGHT_BOLD)
             self.layout.set_font_description(self.font_desc)
             self.layout.set_wrap(pango.WRAP_CHAR)
             self.layout.set_width((self.right - self.left) * 1024)
-        if 'slide.vx' in data:
-            self.slide_vx = int(int(data['slide.vx']) * self.scale / 100)
-        else:
-            self.slide_vx = 0
-        if 'slide.vy' in data:
-            self.slide_vy = int(int(data['slide.vy']) * self.scale / 100)
-        else:
-            self.slide_vy = 0
-        if 'slide.autostop' in data:
-            self.slide_autostop = int(data['slide.autostop'])
-        else:
-            self.slide_autostop = 0
-        if 'action.method' in data and \
-           data['action.method'] in ['sinwave', 'vibrate']:
+        self.slide_vx = int(int(data.get('slide.vx', 0)) * self.scale / 100)
+        self.slide_vy = int(int(data.get('slide.vy', 0)) * self.scale / 100)
+        self.slide_autostop = int(data.get('slide.autostop', 0))
+        if data.get('action.method') in ['sinwave', 'vibrate']:
             action = data['action.method']
-            ref0 = 0
-            ref1 = 0
-            ref2 = 0
-            ref3 = 0
-            if 'action.reference0' in data:
-                ref0 = int(data['action.reference0'])
-            if 'action.reference1' in data:
-                ref1 = int(data['action.reference1'])
-            if 'action.reference2' in data:
-                ref2 = int(data['action.reference2'])
-            if 'action.reference3' in data:
-                ref3 = int(data['action.reference3'])
+            ref0 = int(data.get('action.reference0', 0))
+            ref1 = int(data.get('action.reference1', 0))
+            ref2 = int(data.get('action.reference2', 0))
+            ref3 = int(data.get('action.reference3', 0))
             if ref2:
                 self.action = {'method': action,
                                'ref0': ref0,
@@ -389,29 +261,21 @@ class Balloon:
         self.move_notify_time = None
         self.life_time = None
         self.state = ''
-        if 'life' in data:
-            life = data['life']
-            if life == 'auto':
-                self.life_time = 16000
-            elif life in ['infinitely', '0']:
-                pass
-            elif life == 'orusuban':
-                self.state = 'orusuban'
-            else:
-                try:
-                    self.life_time = int(life)
-                except:
-                    pass
-        else:
+        life = data.get('life', 'auto')
+        if life == 'auto':
             self.life_time = 16000
-        self.start_time = time.time()
-        if 'startdelay' in data:
-            self.startdelay = int(data['startdelay'])
+        elif life in ['infinitely', '0']:
+            pass
+        elif life == 'orusuban':
+            self.state = 'orusuban'
         else:
-            self.startdelay = 0
-        self.nooverlap = 0
-        if 'nooverlap' in data:
-            self.nooverlap = 1
+            try:
+                self.life_time = int(life)
+            except:
+                pass
+        self.start_time = time.time()
+        self.startdelay = int(data.get('startdelay', 0))
+        self.nooverlap = int('nooverlap' in data)
         self.talking = self.get_sakura_is_talking()
         self.move_notify_time = time.time()
         self.timeout_id = None
@@ -439,43 +303,111 @@ class Balloon:
             print self.script                                    
         self.timeout_id = gobject.timeout_add(10, self.do_idle_tasks)
 
-    def update_script(self, text, mode):
-        if text:
-            if mode == 2 and self.script is not None:
-                self.script = ''.join((self.script, text))
+    def clickerase(self):
+        return self.data.get('clickerase', 'on') == 'on'
+
+    def dragmove_horizontal(self):
+        return self.data.get('dragmove.horizontal') == 'on'
+
+    def dragmove_vertical(self):
+        return self.data.get('dragmove.vertical') == 'on'
+
+    def create_window(self):
+        window = gtk.Window(type=gtk.WINDOW_POPUP)
+        window.connect('delete_event', self.delete)
+        window.connect('button_press_event', self.button_press)
+        window.connect('button_release_event', self.button_release)
+        window.connect('motion_notify_event', self.motion_notify)
+        window.connect('leave_notify_event', self.leave_notify)
+        window.set_events(gtk.gdk.BUTTON_PRESS_MASK|
+                          gtk.gdk.BUTTON_RELEASE_MASK|
+                          gtk.gdk.POINTER_MOTION_MASK|
+                          gtk.gdk.LEAVE_NOTIFY_MASK)
+        return window
+
+    def get_coordinate(self, w, h):
+        left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
+        s0_x, s0_y, s0_w, s0_h = self.get_sakura_status('SurfaceSakura')
+        s1_x, s1_y, s1_w, s1_h = self.get_sakura_status('SurfaceKero')
+        b0_x, b0_y, b0_w, b0_h = self.get_sakura_status('BalloonSakura')
+        b1_x, b1_y, b1_w, b1_h = self.get_sakura_status('BalloonKero')
+        x = left
+        y = top
+        if self.position == 'lefttop':
+            pass
+        elif self.position == 'leftbottom':
+            y = top + scrn_h - h
+        elif self.position == 'righttop':
+            x = left + scrn_w - w
+        elif self.position == 'rightbottom':
+            x = left + scrn_w - w
+            y = top + scrn_h - h
+        elif self.position == 'center':
+            x = left + (scrn_w - w) / 2
+            y = top + (scrn_h - h) / 2
+        elif self.position == 'leftcenter':
+            y = top + (scrn_h - h) / 2
+        elif self.position == 'rightcenter':
+            x = left + scrn_w - w
+            y = top + (scrn_h - h) / 2
+        elif self.position == 'centertop':
+            x = left + (scrn_w - w) / 2
+        elif self.position == 'centerbottom':
+            x = left + (scrn_w - w) / 2
+            y = top + scrn_h - h
+        elif self.position == 'sakura':
+            if self.direction: # right
+                x = s0_x + s0_w
             else:
-                self.script = text
+                x = s0_x - w
+            y = s0_y
+        elif self.position == 'kero':
+            if self.direction: # right
+                x = s1_x + s1_w
+            else:
+                x = s1_x - w
+            y = s1_y
+        elif self.position == 'sakurab':
+            x = b0_x
+            y = b0_y
+        elif self.position == 'kerob':
+            x = b1_x
+            y = b1_y
+        return x, y
+
+    def update_script(self, text, mode):
+        if not text:
+            return
+        if mode == 2 and self.script is not None:
+            self.script = ''.join((self.script, text))
+        else:
+            self.script = text
+        self.processed_script = None
+        self.processed_text = ''
+        self.text = ''
+        self.script_wait = None
+        self.quick_session = 0
+        try:
+            self.processed_script = self.script_parser.parse(self.script)
+        except ninix.script.ParserError, e:
             self.processed_script = None
-            self.processed_text = ''
-            self.text = ''
-            self.script_wait = None
-            self.quick_session = 0
-            try:
-                self.processed_script = self.script_parser.parse(self.script)
-            except ninix.script.ParserError, e:
-                self.processed_script = None
-                print '-' * 50
-                print e
-                print self.script                                    
+            print '-' * 50
+            print e
+            print self.script                                    
 
     def get_sakura_is_talking(self):
         talking = 0
         try:
-            if self.__sakura.is_talking():
-                talking = 1
+            talking = int(self.__sakura.is_talking())
         except:
             pass
         return talking
 
     def get_sakura_status(self, key):
         if key == 'SurfaceScale':
-            return self.prefs.get('surface_scale')
+            result = self.__sakura.get_surface_scale()
         elif key == 'SurfaceSakura_Shown':
-            if self.__sakura.surface_is_shown(0):
-                s0_shown = 1
-            else:
-                s0_shown = 0
-            return s0_shown
+            result = int(self.__sakura.surface_is_shown(0))
         elif key == 'SurfaceSakura':
             try:
                 s0_x, s0_y = self.__sakura.get_surface_position(0)
@@ -483,13 +415,9 @@ class Balloon:
             except:
                 s0_x, s0_y = 0, 0
                 s0_w, s0_h = 0, 0
-            return s0_x, s0_y, s0_w, s0_h
+            result = s0_x, s0_y, s0_w, s0_h
         elif key == 'SurfaceKero_Shown':
-            if self.__sakura.surface_is_shown(1):
-                s1_shown = 1
-            else:
-                s1_shown = 0
-            return s1_shown
+            result = int(self.__sakura.surface_is_shown(1))
         elif key == 'SurfaceKero':
             try:
                 s1_x, s1_y = self.__sakura.get_surface_position(1)
@@ -497,18 +425,9 @@ class Balloon:
             except:
                 s1_x, s1_y = 0, 0
                 s1_w, s1_h = 0, 0
-            return s1_x, s1_y, s1_w, s1_h
-        elif key == 'BalloonScale':
-            if self.prefs.get('balloon_scalling'):
-                return self.prefs.get('surface_scale')
-            else:
-                return 100
+            result = s1_x, s1_y, s1_w, s1_h
         elif key == 'BalloonSakura_Shown':
-            if self.__sakura.balloon_is_shown(0):
-                b0_shown = 1
-            else:
-                b0_shown = 0
-            return b0_shown
+            result = int(self.__sakura.balloon_is_shown(0))
         elif key == 'BalloonSakura':
             try:
                 b0_x, b0_y = self.__sakura.get_balloon_position(0)
@@ -516,13 +435,9 @@ class Balloon:
             except:
                 b0_x, b0_y = 0, 0
                 b0_w, b0_h = 0, 0
-            return b0_x, b0_y, b0_w, b0_h
+            result = b0_x, b0_y, b0_w, b0_h
         elif key == 'BalloonKero_Shown':
-            if self.__sakura.balloon_is_shown(1):
-                b1_shown = 1
-            else:
-                b1_shown = 0
-            return b1_shown
+            result = int(self.__sakura.balloon_is_shown(1))
         elif key == 'BalloonKero':
             try:
                 b1_x, b1_y = self.__sakura.get_balloon_position(1)
@@ -530,15 +445,14 @@ class Balloon:
             except:
                 b1_x, b1_y = 0, 0
                 b1_w, b1_h = 0, 0
-            return b1_x, b1_y, b1_w, b1_h
+            result = b1_x, b1_y, b1_w, b1_h
         else:
-            return None
+            result = None
+        return result
 
     def do_idle_tasks(self):
         if not self.window:
             return None
-        s_scale = self.get_sakura_status('SurfaceScale')
-        b_scale = self.get_sakura_status('BalloonScale')
         s0_shown = self.get_sakura_status('SurfaceSakura_Shown')
         s1_shown = self.get_sakura_status('SurfaceKero_Shown')
         b0_shown = self.get_sakura_status('BalloonSakura_Shown')
@@ -557,20 +471,11 @@ class Balloon:
                     self.life_time = 300000
         else:
             if self.visible:
-                if self.position == 'sakura' and not s0_shown:
-                    self.destroy()
-                    return None
-                if self.position == 'kero' and not s1_shown:
-                    self.destroy()
-                    return None
-                if self.position == 'sakurab' and not b0_shown:
-                    self.destroy()
-                    return None
-                if self.position == 'kerob' and not b1_shown:
-                    self.destroy()
-                    return None
-                if self.nooverlap and \
-                   not self.talking and sakura_talking:
+                if (self.position == 'sakura' and not s0_shown) or \
+                        (self.position == 'kero' and not s1_shown) or \
+                        (self.position == 'sakurab' and not b0_shown) or \
+                        (self.position == 'kerob' and not b1_shown) or \
+                        (self.nooverlap and not self.talking and sakura_talking):
                     self.destroy()
                     return None
             else:
@@ -584,9 +489,6 @@ class Balloon:
                    not (self.processed_script or self.processed_text):
                     self.destroy()
                     return None
-                else:
-                    last_time = self.life_time * 0.001 + \
-                                self.start_time - time.time()
             if self.action:
                 if  self.action['method'] == 'sinwave':
                     offset = self.action['ref1'] \
@@ -607,28 +509,19 @@ class Balloon:
             if (self.slide_vx != 0 or self.slide_vy != 0) and \
                    self.slide_autostop > 0 and \
                    self.slide_autostop * 0.001 + 0.05 <= time.time() - self.start_time:
-                self.vx = int((self.slide_autostop / 50.0 + 1) * self.slide_vx * self.scale / 100)
-                if (self.position == 'sakura' and not self.direction0) or \
-                   (self.position == 'kero' and not self.direction1):
-                    self.vx = -self.vx
+                self.vx = self.direction * int((self.slide_autostop / 50.0 + 1) * self.slide_vx * self.scale / 100)
                 self.slide_vx = 0
                 self.vy = int((self.slide_autostop / 50.0 + 1) * self.slide_vy * self.scale / 100)
                 self.slide_vy = 0
             if self.slide_vx != 0:
-                self.vx = int(((time.time() - self.start_time) * self.slide_vx * self.scale / 100) / 50 * 1000.)
-                if (self.position == 'sakura' and not self.direction0) or \
-                   (self.position == 'kero' and not self.direction1):
-                    self.vx = -self.vx
+                self.vx = self.direction * int(((time.time() - self.start_time) * self.slide_vx * self.scale / 100) / 50 * 1000.)
             if self.slide_vy != 0:
                 self.vy = int(((time.time() - self.start_time) * self.slide_vy * self.scale / 100) / 50 * 1000.)
             self.window.move(int(self.x + self.action_x + self.vx),
                              int(self.y + self.action_y + self.vy))
             if self.processed_script or self.processed_text:
                 self.interpret_script()
-        if self.talking and not sakura_talking:
-            self.talking = 0
-        else:
-            self.talking = sakura_talking
+        self.talking = 0 if self.talking and not sakura_talking else sakura_talking
         return True
 
     def redraw(self, widget, event):
@@ -704,27 +597,17 @@ class Balloon:
         self.x_root = event.x_root
         self.y_root = event.y_root
         if event.type == gtk.gdk._2BUTTON_PRESS:
-            if self.scale != 100:
-                x = int(event.x * 100 / self.scale)
-                y = int(event.y * 100 / self.scale)
-            else:
-                x = int(event.x)
-                y = int(event.y)
+            x = int(event.x * 100 / self.scale)
+            y = int(event.y * 100 / self.scale)
             self.__sakura.notify_event('OnEBMouseDoubleClick', self.name,
                                        x, y, self.id)
         return True
 
     def button_release(self, window, event):
-        if self.dragged:
-            self.dragged = 0
         self.x_root = None
         self.y_root = None
-        if self.scale != 100:
-            x = int(event.x * 100 / self.scale)
-            y = int(event.y * 100 / self.scale)
-        else:
-            x = int(event.x)
-            y = int(event.y)
+        x = int(event.x * 100 / self.scale)
+        y = int(event.y * 100 / self.scale)
         if event.type == gtk.gdk.BUTTON_RELEASE:
             if event.button == 1:
                 self.__sakura.notify_event('OnEBMouseClick', self.name,
@@ -732,7 +615,7 @@ class Balloon:
             elif event.button == 3:
                 self.__sakura.notify_event('OnEBMouseClick', self.name,
                                            x, y, self.id, 1)
-        if self.clickerase == 'on':
+        if self.clickerase():
             self.destroy()
         return True
 
@@ -742,10 +625,9 @@ class Balloon:
             x_delta = int(event.x_root - self.x_root)
             y_delta = int(event.y_root - self.y_root)
             if event.state & gtk.gdk.BUTTON1_MASK:
-                self.dragged = 1
-                if self.dragmove_horizontal:
+                if self.dragmove_horizontal():
                     self.x += x_delta
-                if self.dragmove_vertical:
+                if self.dragmove_vertical():
                     self.y += y_delta
                 self.window.move(int(self.x + self.action_x + self.vx),
                                  int(self.y + self.action_y + self.vy))
@@ -753,12 +635,8 @@ class Balloon:
                 self.y_root = event.y_root
         if self.move_notify_time is None or \
            time.time() - self.move_notify_time > 500 * 0.001:
-            if self.scale != 100:
-                x = int(event.x * 100 / self.scale)
-                y = int(event.y * 100 / self.scale)
-            else:
-                x = int(event.x)
-                y = int(event.y)
+            x = int(event.x * 100 / self.scale)
+            y = int(event.y * 100 / self.scale)
             self.__sakura.notify_event('OnEBMouseMove', self.name,
                                        x, y, self.id)
             self.move_notify_time = time.time()
