@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  hanayu.py - a "花柚" compatible Saori module for ninix
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -185,7 +185,7 @@ class Saori(SAORI):
 
     def execute(self, argument):
         if not argument:
-            return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+            return self.RESPONSE[400]
         command = argument[0]
         if command == 'show':
             if len(argument) >= 2:
@@ -195,7 +195,7 @@ class Saori(SAORI):
             else:
                 name = ''
             if name not in self.data:
-                return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+                return self.RESPONSE[400]
             if 'graph' in self.data[name] and \
                    self.data[name]['graph'] in ['line', 'bar',
                                                 'radar', 'radar2']:
@@ -234,10 +234,10 @@ class Saori(SAORI):
             if name in self.graphs:
                 self.graphs[name].destroy()
             else:
-                return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+                return self.RESPONSE[400]
         else:
-            return 'SAORI/1.0 400 Bad Request\r\n\r\n'
-        return 'SAORI/1.0 204 No Content\r\n\r\n'
+            return self.RESPONSE[400]
+        return self.RESPONSE[204]
 
 class Graph:
 
@@ -265,8 +265,8 @@ class Graph:
         self.window.connect('button_press_event', self.button_press)
         self.window.set_events(gtk.gdk.BUTTON_PRESS_MASK)
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
-        self.x = left + scrn_w / 2
-        self.y = top + scrn_h / 4
+        self.x = left + scrn_w // 2
+        self.y = top + scrn_h // 4
         self.window.move(self.x, self.y)
         self.darea = gtk.DrawingArea()
         self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
@@ -286,7 +286,7 @@ class Graph:
             except:
                 pixbuf = None
         self.surface_pixmap = gtk.gdk.Pixmap(
-            None, self.width, self.height, gtk.gdk.visual_get_best_depth())
+            None, self.width, self.height, self.darea.get_visual().depth)
         cmap = self.darea.get_colormap()
         self.surface_gc = self.surface_pixmap.new_gc()
         self.surface_gc.foreground = cmap.alloc_color('#FFFFFF')
@@ -295,13 +295,12 @@ class Graph:
         if pixbuf:
             width = pixbuf.get_width()
             height = pixbuf.get_height()
-            xoffset = (self.width - width) / 2
-            yoffset = (self.height - height) / 2
+            xoffset = (self.width - width) // 2
+            yoffset = (self.height - height) // 2
             self.surface_pixmap.draw_pixbuf(self.surface_gc, pixbuf,
                                             0, 0, xoffset, yoffset,
                                             width, height,
                                             gtk.gdk.RGB_DITHER_NONE, 0, 0)
-            del pixbuf
         self.darea.window.draw_drawable(self.surface_gc, self.surface_pixmap,
                                         0, 0, 0, 0, self.width, self.height)
 
@@ -398,12 +397,12 @@ class Line(Graph):
                                       60, 260, 420, 260)
 
     def draw_graph(self):
-        num = len(self.args) / 2
-        step = 368 / num
+        num = len(self.args) // 2
+        step = 368 // num
         for index in range(num):
             self.layout.set_text(self.args[index * 2])
             w, h = self.layout.get_pixel_size()
-            pos_x = 60 + index * step + step / 2 - w / 2
+            pos_x = 60 + index * step + step // 2 - w // 2
             pos_y = 268
             self.surface_pixmap.draw_layout(
                 self.surface_gc, pos_x, pos_y, self.layout)
@@ -441,10 +440,10 @@ class Line(Graph):
             line_width = int(self.data['line.width'])
         self.surface_gc.line_width = line_width
         for index in range(1, num):
-            src_x = 60 + (index - 1) * step + step / 2
+            src_x = 60 + (index - 1) * step + step // 2
             src_y = 220 - int(
                 168 * self.args[(index - 1) * 2 + 1] / (limit_max - limit_min))
-            dst_x = 60 + index * step + step / 2
+            dst_x = 60 + index * step + step // 2
             dst_y = 220 - int(168 * self.args[index * 2 + 1] / (limit_max - limit_min))
             self.surface_pixmap.draw_line(self.surface_gc,
                                           src_x, src_y, dst_x, dst_y)
@@ -478,7 +477,7 @@ class Line(Graph):
                         image, mask = None, None
             if image:
                 w, h = image.get_size()
-                x = 60 + index * step + step / 2 - w / 2
+                x = 60 + index * step + step // 2 - w // 2
                 y = 220 - int(
                     168 * self.args[index * 2 + 1] / (limit_max - limit_min)) - h / 2
                 gc = self.surface_pixmap.new_gc()
@@ -536,7 +535,7 @@ class Radar(Graph):
         if 'frame.width' in self.data:
             frame_width = int(self.data['frame.width'])
         self.surface_gc.line_width = frame_width
-        num = len(self.args) / 2
+        num = len(self.args) // 2
         for index in range(num):
             x = 146 + int(math.cos(math.pi * (0.5 - 2.0 * index / num)) * 114)
             y = 146 - int(math.sin(math.pi * (0.5 - 2.0 * index / num)) * 114)
@@ -544,7 +543,7 @@ class Radar(Graph):
                                           146, 146, x, y)
 
     def draw_graph(self):
-        num = len(self.args) / 2
+        num = len(self.args) // 2
         for index in range(num):
             try:
                 value = self.args[index * 2 + 1]
