@@ -3,7 +3,7 @@
 #  kawari.py - a "華和梨" compatible Shiori module for ninix
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -14,7 +14,6 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
-import __builtin__
 import base64
 import os
 import re
@@ -30,7 +29,7 @@ charset = 'Shift_JIS' # default
 
 def read_dict(path, debug=0):
     global charset
-    f = __builtin__.open(path)
+    f = open(path)
     lineno = 0
     buf = []
     for line in f:
@@ -261,10 +260,10 @@ def parse_text(data, start, stop_pattern=None):
     return i, data[start:i]
 
 def read_local_script(path):
-    f = __builtin__.open(path)
+    f = open(path)
     rdict = {}
     kdict = {}
-    for line in f.readline():
+    for line in f.readlines():
         if line.startswith('#'):
             rdict[unicode(line.strip(), charset, 'ignore')] = \
                  [unicode(f.readline().strip(), charset, 'ignore')]
@@ -431,7 +430,7 @@ class Kawari:
     # internal
     def clear(self, name):
         if self.debug & 128:
-            print '*** clear("%s")' % name.encode('UTF-8', 'ignore')
+            print '*** clear("%s")' % name.encode('utf-8', 'ignore')
         for dic in self.rdictlist:
             if name in dic:
                 del dic[name]
@@ -447,7 +446,7 @@ class Kawari:
 
     def unshift(self, name, value):
         if self.debug & 128:
-            print ('*** unshift("%s", "%s")' % (name, value)).encode('UTF-8',
+            print ('*** unshift("%s", "%s")' % (name, value)).encode('utf-8',
                                                                      'ignore')
         dic = self.get_internal_dict(name)
         i, segments = parse(value, 0)
@@ -457,13 +456,13 @@ class Kawari:
         dic = self.get_internal_dict(name)
         value = self.expand(dic[name].pop(0))
         if self.debug & 128:
-            print ('*** shift("%s") => "%s"' % (name, value)).encode('UTF-8',
+            print ('*** shift("%s") => "%s"' % (name, value)).encode('utf-8',
                                                                      'ignore')
         return value
 
     def push(self, name, value):
         if self.debug & 128:
-            print ('*** push("%s", "%s")' % (name, value)).encode('UTF-8',
+            print ('*** push("%s", "%s")' % (name, value)).encode('utf-8',
                                                                   'ignore')
         dic = self.get_internal_dict(name)
         i, segments = parse(value, 0)
@@ -473,7 +472,7 @@ class Kawari:
         dic = self.get_internal_dict(name)
         value = self.expand(dic[name].pop())
         if self.debug & 128:
-            print ('*** pop("%s") => "%s"' % (name, value)).encode('UTF-8',
+            print ('*** pop("%s") => "%s"' % (name, value)).encode('utf-8',
                                                                    'ignore')
         return value
 
@@ -493,12 +492,12 @@ class Kawari:
                 selection = self.select_compound_phrase(name)
             if selection is None:
                 if self.debug & 8:
-                    print '${%s} not found' % name.encode('UTF-8', 'ignore')
+                    print '${%s} not found' % name.encode('utf-8', 'ignore')
                 return default
             segments, context = selection
         if self.debug & 16:
-            print name.encode('UTF-8', 'ignore'), '=>', \
-                  ''.join(segments).encode('UTF-8', 'ignore')
+            print name.encode('utf-8', 'ignore'), '=>', \
+                  ''.join(segments).encode('utf-8', 'ignore')
         return self.expand(segments, context, depth)
 
     def parse_all(self, data, start=0):
@@ -632,7 +631,7 @@ class Kawari:
                     values = []
                 elif self.debug & 32:
                     print 'kawari.py: syntax error:', \
-                          segments[i].encode('UTF-8', 'ignore')
+                          segments[i].encode('utf-8', 'ignore')
                 continue
             handler = self.kis_commands.get(argv[0])
             try:
@@ -642,11 +641,11 @@ class Kawari:
             except RuntimeError, message:
                 if self.debug & 32:
                     print 'kawari.py: %s: %s' % \
-                          (message, segments[i].encode('UTF-8', 'ignore'))
+                          (message, segments[i].encode('utf-8', 'ignore'))
         result = ''.join(values)
         if self.debug & 64:
-            print '>>>', segments[i].encode('UTF-8', 'ignore')
-            print ''.join(('"', result.encode('UTF-8', 'ignore'), '"'))
+            print '>>>', segments[i].encode('utf-8', 'ignore')
+            print ''.join(('"', result.encode('utf-8', 'ignore'), '"'))
         return i, result
 
     def split_commands(self, data):
@@ -701,14 +700,14 @@ class Kawari:
         if self.debug & 64:
             if else_block:
                 print '>>> $(if %s)$(then %s)$(else %s)$(endif)' % (
-                    ''.join(if_block).encode('UTF-8', 'ignore'),
-                    ''.join(then_block).encode('UTF-8', 'ignore'),
-                    ''.join(else_block).encode('UTF-8', 'ignore'))
+                    ''.join(if_block).encode('utf-8', 'ignore'),
+                    ''.join(then_block).encode('utf-8', 'ignore'),
+                    ''.join(else_block).encode('utf-8', 'ignore'))
             else:
                 print '>>> $(if %s)$(then %s)$(endif)' % (
-                    ''.join(if_block).encode('UTF-8', 'ignore'),
-                    ''.join(then_block).encode('UTF-8', 'ignore'))
-            print ''.join(('"', result.encode('UTF-8', 'ignore'), '"'))
+                    ''.join(if_block).encode('utf-8', 'ignore'),
+                    ''.join(then_block).encode('utf-8', 'ignore'))
+            print ''.join(('"', result.encode('utf-8', 'ignore'), '"'))
         return result
 
     def exec_if(self, if_block, then_block, else_block):
@@ -739,7 +738,7 @@ class Kawari:
         except ValueError:
             raise RuntimeError, 'invalid argument'
         buf = []
-        for i in range(n):
+        for _ in range(n):
             buf.append(self.expand(argv[2]))
         return ''.join(buf)
 
@@ -900,7 +899,7 @@ class Kawari:
             raise RuntimeError, 'syntax error'
         path = self.get_dict_path(self.expand(argv[1]))
         try:
-            f = __builtin__.open(path, 'w')
+            f = open(path, 'w')
             f.write('#\r\n# Kawari save file\r\n#\r\n')
             for i in range(2, len(argv)):
                 name = self.expand(argv[i])
@@ -928,7 +927,7 @@ class Kawari:
             raise RuntimeError, 'syntax error'
         path = self.get_dict_path(self.expand(argv[1]))
         try:
-            linelist = __builtin__.open(path).readlines()
+            linelist = open(path).readlines()
         except IOError:
             raise RuntimeError, 'cannot read file'
         name = self.expand(argv[2])
@@ -1516,7 +1515,7 @@ class ExprParser:
             argc = 3
         if argc > 0:
             buf = [self.STR_EXPR, self.pop()] # fuction
-            for i in range(argc):
+            for _ in range(argc):
                 self.match_space()
                 buf.append(self.get_str_expr())
         elif self.check('quote'):
@@ -1607,7 +1606,7 @@ def scan_ini(kawari_dir, filename, saori_ini, debug=0):
             filename = value.replace('\\', '/').lower().encode('utf-8')
             path = os.path.join(kawari_dir, filename)
             try:
-                __builtin__.open(path).read(64)
+                open(path).read(64)
             except IOError, (errno, message):
                 if debug & 4:
                     print 'kawari.py: read error:', path
@@ -1711,7 +1710,7 @@ class Shiori(Kawari):
             'Shiori: KAWARI compatible module for ninix\n'
             '        Copyright (C) 2001, 2002 by Tamito KAJIYAMA\n'
             '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko\n'
-            '        Copyright (C) 2002-2009 by Shyouzou Sugitani\n'
+            '        Copyright (C) 2002-2010 by Shyouzou Sugitani\n'
             '        Copyright (C) 2003 by Shun-ichi TAHARA\n')
 
     def request(self, req_string):
@@ -1932,7 +1931,7 @@ class Shiori(Kawari):
         return result
 
 
-def open(kawari_dir, debug=0):
+def kawari_open(kawari_dir, debug=0):
     pathlist = [None]
     rdictlist = [{}]
     kdictlist = [{}]
@@ -1949,7 +1948,7 @@ def open(kawari_dir, debug=0):
     return Kawari(kawari_dir, pathlist, rdictlist, kdictlist, debug)
 
 def is_local_script(path):
-    line = __builtin__.open(path).readline()
+    line = open(path).readline()
     return line.startswith('[SAKURA]')
 
 ###   TEST   ###
@@ -1969,23 +1968,23 @@ def test():
         if opt in ['-d', '--debug']:
             debug = int(val)
     print 'reading kawari.ini...'
-    kawari = open(kawari_dir, debug)
+    kawari = kawari_open(kawari_dir, debug)
     for rdict in kawari.rdictlist:
         for k, v in rdict.iteritems():
-            print k.encode('UTF-8', 'ignore')
+            print k.encode('utf-8', 'ignore')
             for p in v:
-                print ''.join(('\t', ''.join(p).encode('UTF-8', 'ignore')))
+                print ''.join(('\t', ''.join(p).encode('utf-8', 'ignore')))
     for kdict in kawari.kdictlist:
         for k, v in kdict.iteritems():
             print ''.join(
-                ('[ "', '", "'.join(k).encode('UTF-8', 'ignore'), '" ]'))
+                ('[ "', '", "'.join(k).encode('utf-8', 'ignore'), '" ]'))
             for p in v:
-                print ''.join(('\t', ''.join(p).encode('UTF-8', 'ignore')))
+                print ''.join(('\t', ''.join(p).encode('utf-8', 'ignore')))
     while 1:
         print '=' * 40
         s = kawari.getaistringrandom()
         print '-' * 40
-        print unicode(s, charset, 'ignore').encode('UTF-8', 'ignore')
+        print unicode(s, charset, 'ignore').encode('utf-8', 'ignore')
         try:
             raw_input()
         except (EOFError, KeyboardInterrupt):
