@@ -2,7 +2,7 @@
 #
 #  kawari8.py - a (Real) 華和梨 loader for ninix
 #  Copyright (C) 2002, 2003 by ABE Hideaki <abe-xx@eos.dricas.com>
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -33,16 +33,15 @@ class Shiori:
 
     def find(self, topdir, dll_name):
         result = 0
-        if _kawari8:
-            if os.path.isfile(os.path.join(topdir, 'kawarirc.kis')):
-                result = 205
+        if _kawari8 and os.path.isfile(os.path.join(topdir, 'kawarirc.kis')):
+            result = 205
         return result
 
     def show_description(self):
         sys.stdout.write(
             'Shiori: Real Kawari8 loader for ninix\n'
             '        Copyright (C) 2002, 2003 by ABE Hideaki\n'
-            '        Copyright (C) 2002-2009 by Shyouzou Sugitani\n'
+            '        Copyright (C) 2002-2010 by Shyouzou Sugitani\n'
             '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko\n')
 
     def load(self, topdir):
