@@ -3,7 +3,7 @@
 #  misaka.py - a "美坂" compatible Shiori module for ninix
 #  Copyright (C) 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -18,7 +18,6 @@
 # - ${foo}
 # - $(foo) ?
 
-import __builtin__
 import os
 import re
 import StringIO
@@ -74,7 +73,7 @@ def list_dict(top_dir):
 
 def read_misaka_ini(top_dir, _debug=0):
     path = os.path.join(top_dir, 'misaka.ini')
-    f = __builtin__.open(path)
+    f = open(path)
     filelist = []
     debug = 0
     error = 0
@@ -658,10 +657,7 @@ class Parser:
             self.lexer.pop()
             self.lexer.skip_space()
             buf.append(self.get_and_expr())
-        if len(buf) > 2:
-            return buf
-        else:
-            return buf[1]
+        return buf if len(buf) > 2 else buf[1]
 
     def get_and_expr(self):
         buf = [NODE_AND_EXPR]
@@ -674,10 +670,7 @@ class Parser:
             self.lexer.pop()
             self.lexer.skip_space()
             buf.append(self.get_sub_expr())
-        if len(buf) > 2:
-            return buf
-        else:
-            return buf[1]
+        return buf if len(buf) > 2 else buf[1]
 
     def get_sub_expr(self):
         token, lexeme = self.lexer.look_ahead()
@@ -718,9 +711,7 @@ class Parser:
             self.lexer.pop()
             self.lexer.skip_space()
             buf.append(self.get_mul_expr())
-        if len(buf) > 2:
-            return [buf]
-        return buf[1]
+        return [buf] if len(buf) > 2 else buf[1]
 
     def get_mul_expr(self):
         buf = [NODE_MUL_EXPR]
@@ -734,9 +725,7 @@ class Parser:
             self.lexer.pop()
             self.lexer.skip_space()
             buf.append(self.get_pow_expr())
-        if len(buf) > 2:
-            return [buf]
-        return buf[1]
+        return [buf] if len(buf) > 2 else buf[1]
 
     def get_pow_expr(self):
         buf = [NODE_POW_EXPR]
@@ -749,9 +738,7 @@ class Parser:
             self.lexer.pop()
             self.lexer.skip_space()
             buf.append(self.get_unary_expr())
-        if len(buf) > 2:
-            return [buf]
-        return buf[1]
+        return [buf] if len(buf) > 2 else buf[1]
 
     def get_unary_expr(self):
         token, lexeme = self.lexer.look_ahead()
@@ -785,10 +772,10 @@ class Parser:
         indent = '  ' * depth
         if node[0] == NODE_TEXT:
             print ''.join((indent, 'TEXT')), \
-                  ''.join(('"', node[1].encode('UTF-8', 'ignore'), '"'))
+                  ''.join(('"', node[1].encode('utf-8', 'ignore'), '"'))
         elif node[0] == NODE_STRING:
             print ''.join((indent, 'STRING')), \
-                  ''.join(('"', node[1].encode('UTF-8', 'ignore'), '"'))
+                  ''.join(('"', node[1].encode('utf-8', 'ignore'), '"'))
         elif node[0] == NODE_VARIABLE:
             print ''.join((indent, 'VARIABLE'))
             self.dump_list(node[1], depth + 1)
@@ -1014,7 +1001,7 @@ class Shiori:
             'Shiori: MISAKA compatible module for ninix\n'
             '        Copyright (C) 2002 by Tamito KAJIYAMA\n'
             '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko\n'
-            '        Copyright (C) 2002-2009 by Shyouzou Sugitani\n')
+            '        Copyright (C) 2002-2010 by Shyouzou Sugitani\n')
 
     def reset(self):
         self.dict = {}
@@ -1055,7 +1042,7 @@ class Shiori:
             for filename in filelist:
                 path = os.path.join(self.misaka_dir, filename)
                 try:
-                    f = __builtin__.open(path)
+                    f = open(path)
                 except IOError:
                     if self.debug & 4:
                         print 'cannot read', filename
@@ -1068,15 +1055,14 @@ class Shiori:
                     detector.feed(line)
                     if detector.done:
                         break
-                else:
-                    detector.close()
-                    if detector.result['confidence'] > 0.98: # XXX
-                        self.charset = detector.result['encoding']
-                        break
+                detector.close()
+                if detector.result['confidence'] > 0.98: # XXX
+                    self.charset = detector.result['encoding']
+                    break
         for filename in filelist:
             path = os.path.join(self.misaka_dir, filename)
             try:
-                f = __builtin__.open(path)
+                f = open(path)
             except IOError:
                 if self.debug & 4:
                     print 'cannot read', filename
@@ -1155,7 +1141,7 @@ class Shiori:
 
     def load_database(self):
         try:
-            f = __builtin__.open(self.dbpath)
+            f = open(self.dbpath)
         except IOError:
             return
         while 1:
@@ -1165,7 +1151,7 @@ class Shiori:
             header = self.strip_newline(line).split()
             if header[0] == 'SCHOLAR':
                 try:
-                    name = unicode(header[1], 'UTF-8')
+                    name = unicode(header[1], 'utf-8')
                 except UnicodeError:
                     try:
                         name = unicode(header[1], 'EUC-JP')
@@ -1175,7 +1161,7 @@ class Shiori:
                         break
                 value = self.strip_newline(f.readline())
                 try:
-                    value = unicode(value, 'UTF-8')
+                    value = unicode(value, 'utf-8')
                 except UnicodeError:
                     try:
                         value = unicode(value, 'EUC-JP')
@@ -1192,7 +1178,7 @@ class Shiori:
                         print 'misaka.py: malformed database (ignored)'
                     break
                 try:
-                    name = unicode(header[2], 'UTF-8')
+                    name = unicode(header[2], 'utf-8')
                 except UnicodeError:
                     try:
                         name = unicode(header[2], 'EUC-JP')
@@ -1201,10 +1187,10 @@ class Shiori:
                             print 'misaka.py: malformed database (ignored)'
                         break
                 array = Array(self, name)
-                for i in range(size):
+                for _ in range(size):
                     value = self.strip_newline(f.readline())
                     try:
-                        value = unicode(value, 'UTF-8')
+                        value = unicode(value, 'utf-8')
                     except UnicodeError:
                         try:
                             value = unicode(value, 'EUC-JP')
@@ -1220,7 +1206,7 @@ class Shiori:
 
     def save_database(self):
         try:
-            f = __builtin__.open(self.dbpath, 'w')
+            f = open(self.dbpath, 'w')
         except IOError:
             if self.debug & 4:
                 print 'misaka.py: cannot write database (ignored)'
@@ -1232,12 +1218,12 @@ class Shiori:
                     continue
                 f.write(
                     'SCHOLAR %s\n%s\n' % \
-                    (name.encode('UTF-8'), unicode(value).encode('UTF-8')))
+                    (name.encode('utf-8'), unicode(value).encode('utf-8')))
             elif value_type == TYPE_ARRAY:
                 f.write(
-                    'ARRAY %d %s\n' % (len(value), name.encode('UTF-8')))
+                    'ARRAY %d %s\n' % (len(value), name.encode('utf-8')))
                 for item in value:
-                    f.write('%s\n' % self.expand(item).encode('UTF-8'))
+                    f.write('%s\n' % self.expand(item).encode('utf-8'))
             else:
                 raise RuntimeError, 'should not reach here'
         f.close()
@@ -1338,10 +1324,7 @@ class Shiori:
 
     def get_ref(self, num):
         key = ''.join(('Reference', str(num)))
-        if key in self.req_header:
-            return self.req_header[key]
-        else:
-            return None
+        return self.req_header.get(key)
 
     # internal
     def reset_random_talk_interval(self):
@@ -1373,11 +1356,7 @@ class Shiori:
 
     def expand_args(self, nodelist):
         tmp = self.expand(nodelist)
-        if tmp.startswith('$'):
-            result = self.eval_variable(nodelist)
-        else:
-            result = tmp
-        return result
+        return self.eval_variable(nodelist) if tmp.startswith('$') else tmp
 
     def eval(self, node):
         if node[0] == NODE_TEXT:
@@ -1811,17 +1790,13 @@ class Shiori:
         if len(args) != 1:
             return ''
         buf = self.expand_args(args[0])
-        if not buf:
-            return ''
-        return buf[0]
+        return '' if not buf else buf[0]
 
     def exec_substringlast(self, args):
         if len(args) != 1:
             return ''
         buf = self.expand_args(args[0])
-        if not buf:
-            return ''
-        return buf[-1]
+        return '' if not buf else buf[-1]
 
     def exec_length(self, args):
         if len(args) != 1:
@@ -1867,9 +1842,7 @@ class Shiori:
             return 'false'
         buf0 = self.expand_args(args[0])
         buf1 = self.expand_args(args[1])
-        if buf0 and buf1 and buf0[-1] == buf1[0]:
-            return 'true'
-        return 'false'
+        return 'true' if buf0 and buf1 and buf0[-1] == buf1[0] else 'false'
 
     def exec_append(self, args):
         if len(args) == 2:
@@ -2071,29 +2044,20 @@ class Shiori:
                 if key:
                     saori_header.append(key)
                     saori_value[key] = value
-        if 'Result' in saori_value:
-            return saori_value['Result']
-        else:
-            return ''
+        return saori_value.get('Result', '')
 
     def load_saori(self, args):
         if not args:
             return ''
-        else:
-            result = self.saori_library.load(self.expand_args(args[0]),
-                                             self.misaka_dir)
-            if not result:
-                result = ''
-            return str(result)
+        result = self.saori_library.load(self.expand_args(args[0]),
+                                         self.misaka_dir)
+        return '' if not result else str(result)
 
     def unload_saori(self, args):
         if not args:
             return ''
-        else:
-            result = self.saori_library.unload(self.expand_args(args[0]))
-            if not result:
-                result = ''
-            return str(result)
+        result = self.saori_library.unload(self.expand_args(args[0]))
+        return '' if not result else str(result)
 
     def exec_isghostexists(self, args):
         result = 'false'
@@ -2134,7 +2098,7 @@ class Shiori:
         '$pop':                 exec_pop,
         '$popmatchl':           exec_popmatchl,
         '$index':               exec_index,
-        '$insentece':           exec_insentence,
+        '$insentence':          exec_insentence,
         '$substringw':          exec_substringw,
         '$substringwl':         exec_substringwl,
         '$substringwr':         exec_substringwr,
@@ -2181,7 +2145,7 @@ class SaoriLibrary:
         return result
 
 
-def open(top_dir, debug=0):
+def misaka_open(top_dir, debug=0):
     misaka = Shiori(top_dir, debug)
     misaka.load()
     return misaka
@@ -2202,7 +2166,7 @@ def test_lexer(filelist, charset='Shift_JIS'):
     lexer = Lexer(charset=charset, debug=1)
     for filename in filelist:
         print 'Reading', filename, '...'
-        lexer.read(__builtin__.open(filename))
+        lexer.read(open(filename))
     token_names = {
         TOKEN_WHITESPACE:    'TOKEN_WHITESPACE',
         TOKEN_NEWLINE:       'TOKEN_NEWLINE',
@@ -2229,19 +2193,19 @@ def test_lexer(filelist, charset='Shift_JIS'):
                      TOKEN_COMMA, TOKEN_SEMICOLON]:
             print repr(lexeme)
         else:
-            print lexeme.encode('UTF-8', 'ignore')
+            print lexeme.encode('utf-8', 'ignore')
 
 def test_parser(filelist, charset='Shift_JIS'):
     for filename in filelist:
         print 'Reading', filename, '...'
         parser = Parser(charset=charset, debug=1)
-        parser.read(__builtin__.open(filename))
+        parser.read(open(filename))
         common, dic = parser.get_dict()
         if common is not None:
             print 'Common'
             parser.dump_node(common, depth=2)
         for name, parameters, sentences in dic:
-            print 'Group', name.encode('UTF-8', 'ignore')
+            print 'Group', name.encode('utf-8', 'ignore')
             if parameters:
                 print 'Parameter:'
                 for parameter in parameters:
