@@ -3,7 +3,7 @@
 #  niseshiori.py - a "偽栞" compatible Shiori module for ninix
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -14,7 +14,6 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
-import __builtin__
 import random
 import time
 import sys
@@ -89,7 +88,7 @@ class NiseShiori:
 
     def load_database(self, path):
         try:
-            f = __builtin__.open(path)
+            f = open(path)
         except IOError:
             return
         dic = {}
@@ -115,7 +114,7 @@ class NiseShiori:
         if self.dbpath is None:
             return
         try:
-            f = __builtin__.open(self.dbpath, 'w')
+            f = open(self.dbpath, 'w')
         except IOError:
             print 'niseshiori.py: cannot write database (ignored)'
             return
@@ -135,7 +134,7 @@ class NiseShiori:
 
     def read_dict(self, path):
         # read dict file and decrypt if necessary
-        f = __builtin__.open(path)
+        f = open(path)
         if path.endswith('.dtx'):
             buf = self.decrypt(f.read())
         else:
@@ -153,9 +152,7 @@ class NiseShiori:
                 continue
             elif i == 1 and line.startswith('#Charset:'):
                 charset = line[9:].strip()
-                if charset == 'UTF-8':
-                    decode = lambda line: unicode(line, 'utf-8')
-                elif charset in ['EUC-JP', 'EUC-KR']:
+                if charset in ['UTF-8', 'EUC-JP', 'EUC-KR']: # XXX
                     decode = lambda line: unicode(line, charset)
                 continue
             elif line.startswith('/*'):
@@ -731,10 +728,10 @@ class NiseShiori:
         if self.debug & 16:
             if isinstance(key, tuple):
                 key = '\\%s[%s]' % (key[0] or '', key[1])
-            print key.encode('UTF-8', 'ignore'), '=>', \
-                  s.encode('UTF-8', 'ignore')
+            print key.encode('utf-8', 'ignore'), '=>', \
+                  s.encode('utf-8', 'ignore')
             print ' ' * len(key),
-            print '=>', t.encode('UTF-8', 'ignore')
+            print '=>', t.encode('utf-8', 'ignore')
         return t
 
     def find_chains(self, key, context):
@@ -778,15 +775,9 @@ class NiseShiori:
                 result = ''
             else:
                 n = int(name[4])
-                if self.reference[n] is None:
-                    result = ''
-                else:
-                    result = self.reference[n]
+                result = '' if self.reference[n] is None else self.reference[n]
         elif name == '%jpentry':
-            if self.jump_entry is None:
-                result = ''
-            else:
-                result = self.jump_entry
+            result = '' if self.jump_entry is None else self.jump_entry
         elif name == '%year':
             result = str(self.current_time[0])
         elif name == '%month':
@@ -821,10 +812,7 @@ class NiseShiori:
             else:
                 result = '偽栞 for ninix'
         elif name == '%sender':
-            if self.sender:
-                result = self.sender
-            else:
-                result = ''
+            result = self.sender if self.sender else ''
         elif name == '%ghost':
             if self.to:
                 result = self.to
@@ -838,9 +826,7 @@ class NiseShiori:
 
     def eval_expression(self, expr):
         tree = self.expr_parser.parse(expr)
-        if tree is None:
-            return None
-        return self.interp_expr(tree)
+        return None if tree is None else self.interp_expr(tree)
 
     def __interp_add_expr(self, tree):
         value = self.interp_expr(tree[0])
@@ -1067,7 +1053,7 @@ class Shiori(NiseShiori):
             'Shiori: NiseShiori compatible module for ninix\n'
             '        Copyright (C) 2001, 2002 by Tamito KAJIYAMA\n'
             '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko\n'
-            '        Copyright (C) 2002-2009 by Shyouzou Sugitani\n'
+            '        Copyright (C) 2002-2010 by Shyouzou Sugitani\n'
             '        Copyright (C) 2003 by Shun-ichi TAHARA\n')
 
     def request(self, req_string):
@@ -1143,7 +1129,7 @@ class Shiori(NiseShiori):
         return result
 
 
-def open(shiori_dir, debug=0):
+def ns_open(shiori_dir, debug=0):
     ns = NiseShiori(debug)
     ns.load(shiori_dir)
     return ns
@@ -1156,7 +1142,7 @@ def test():
     if not list_dict(top_dir):
         print 'no dictionary'
         return
-    ns = open(top_dir, 4 + 8 + 16)
+    ns = ns_open(top_dir, 4 + 8 + 16)
     dump_dict(ns)
     while 1:
         print ns.getaistringrandom()
@@ -1170,26 +1156,26 @@ def dump_dict(ns):
     for k, v in ns.dict.iteritems():
         if isinstance(k, tuple):
             k = '(%s, %s)' % k
-        print k.encode('UTF-8', 'ignore')
-        for e in v: print '\t', e.encode('UTF-8', 'ignore')
+        print k.encode('utf-8', 'ignore')
+        for e in v: print '\t', e.encode('utf-8', 'ignore')
     print '*' * 50
     print 'CHAINS'
     for t, chain_list in ns.type_chains.iteritems():
-        print t.encode('UTF-8', 'ignore')
+        print t.encode('utf-8', 'ignore')
         for c, w in chain_list:
-            print ('-> %s, %s' % (c, w)).encode('UTF_8', 'ignore')
+            print ('-> %s, %s' % (c, w)).encode('utf-8', 'ignore')
     for key, dic in ns.word_chains.iteritems():
-        print ('(%s, %s)' % key).encode('UTF-8', 'ignore')
+        print ('(%s, %s)' % key).encode('utf-8', 'ignore')
         for t, chain_list in dic.iteritems():
-            prefix = '-> %s' % t.encode('UTF-8', 'ignore')
+            prefix = '-> %s' % t.encode('utf-8', 'ignore')
             for c, w in chain_list:
-                print prefix, ('-> %s, %s' % (c, w)).encode('UTF-8', 'ignore')
+                print prefix, ('-> %s, %s' % (c, w)).encode('utf-8', 'ignore')
                 prefix = ''.join(('   ', ' ' * len(t)))
     print '*' * 50
     print 'KEYWORDS'
     for (t, w), s in ns.keywords.iteritems():
-        print t.encode('UTF-8', 'ignore'), w.encode('UTF-8', 'ignore')
-        print '->', s.encode('UTF-8', 'ignore')
+        print t.encode('utf-8', 'ignore'), w.encode('utf-8', 'ignore')
+        print '->', s.encode('utf-8', 'ignore')
     print '*' * 50
     print 'RESPONSES'
     for k, v in ns.responses.iteritems():
@@ -1198,22 +1184,22 @@ def dump_dict(ns):
     print '*' * 50
     print 'GREETINGS'
     for k, v in ns.greetings.iteritems():
-        print k.encode('UTF-8', 'ignore')
+        print k.encode('utf-8', 'ignore')
         print '->', v
     print '*' * 50
     print 'EVENTS'
     for k, v in ns.events.iteritems():
         print_condition(k)
         for e in v:
-            print '->', e.encode('UTF-8', 'ignore')
+            print '->', e.encode('utf-8', 'ignore')
     
 def print_condition(condition):
     prefix = 'Condition'
     for cond_type, expr in condition:
         if cond_type == NiseShiori.COND_COMPARISON:
-            print prefix, ("'%s' '%s' '%s'" % expr).encode('UTF-8', 'ignore')
+            print prefix, ("'%s' '%s' '%s'" % expr).encode('utf-8', 'ignore')
         elif cond_type == NiseShiori.COND_STRING:
-            print prefix, "'%s'" % expr.encode('UTF-8', 'ignore')
+            print prefix, "'%s'" % expr.encode('utf-8', 'ignore')
         prefix = '      and'
 
 if __name__ == '__main__':
