@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  saori_cpuid.py - a saori_cpuid compatible Saori module for ninix
-#  Copyright (C) 2003-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -45,9 +45,9 @@ class Saori(SAORI):
 
     def execute(self, argument):
         if not argument:
-            return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+            return self.RESPONSE[400]
         if len(argument) > 1 and argument[1] == 0:
-            return 'SAORI/1.0 204 No Content\r\n\r\n'
+            return self.RESPONSE[204]
         value = ''
         if argument[0] == 'platform':
             value = sys.platform ## os.name
@@ -62,4 +62,4 @@ class Saori(SAORI):
         if value:
             return 'SAORI/1.0 200 OK\r\nResult: %s\r\n\r\n' % value
         else:
-            return 'SAORI/1.0 204 No Content\r\n\r\n'
+            return self.RESPONSE[204]
