@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  ssu.py - a ssu compatible Saori module for ninix
-#  Copyright (C) 2003-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -56,18 +56,18 @@ class Saori(SAORI):
     def request(self, req):
         req_type, argument, charset = self.evaluate_request(req)
         if not req_type:
-            return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+            return self.RESPONSE[400]
         elif req_type == 'GET Version':
-            return 'SAORI/1.0 204 No Content\r\n\r\n'
+            return self.RESPONSE[204]
         elif req_type == 'EXECUTE':
             if argument[0] not in self.function:
-                return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+                return self.RESPONSE[400]
             name = argument[0]
             argument = argument[1:]
             if self.function[name][1] == [None]:
                 pass
             elif len(argument) not in self.function[name][1]:
-                return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+                return self.RESPONSE[400]
             self.value = []
             result = self.function[name][0](argument, charset)
             if result is not None and result != '':
@@ -80,9 +80,9 @@ class Saori(SAORI):
                 s = ''.join((s, '\r\n'))
                 return s
             else:
-                return 'SAORI/1.0 204 No Content\r\n\r\n'
+                return self.RESPONSE[204]
         else:
-            return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+            return self.RESPONSE[400]
 
     def ssu_is_empty(self, args, charset):
         if not args:
