@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  textcopy.py - a TEXTCOPY compatible Saori module for ninix
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -26,36 +26,27 @@ class Saori(SAORI):
 
     def __init__(self):
         SAORI.__init__(self)
-        self.window = None
-        self.entry = None
+        self.clipboard = None
 
     def check_import(self):
-        if gtk is None:
-            return 0
-        else:
-            return 1
+        return int(gtk is not None)
 
     def setup(self):
-        self.window = gtk.Window()
-        self.entry = gtk.Entry()
-        self.window.add(self.entry)
-        self.entry.realize()
+        self.clipboard = gtk.Clipboard(selection='PRIMARY')
         return 1
 
     def finalize(self):
-        self.entry = None
-        self.window = None
+        self.clipboard = None
         return 1
 
     def execute(self, argument):
-        if not argument or self.entry is None:
-            return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+        if not argument or self.clipboard is None:
+            return self.RESPONSE[400]
         text = argument[0].encode('utf-8')
-        self.entry.set_text(text)
-        self.entry.select_region(0, len(text))
+        self.clipboard.set_text(text)
         if len(argument) >= 2 and argument[1] != 0:
             return 'SAORI/1.0 200 OK\r\n' \
                    'Result: %s\r\n\r\n' % \
                    argument[0].encode(self.charset, 'replace')
         else:
-            return 'SAORI/1.0 204 No Content\r\n\r\n'
+            return self.RESPONSE[204]
