@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  wmove.py - a wmove.dll compatible Saori module for ninix
-#  Copyright (C) 2003-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -70,55 +70,41 @@ class Saori(SAORI):
     def __check_argument(self, argument):
         name = argument[0]
         result = 1
-        if name in ['MOVE', 'MOVE_INSIDE']:
-            if len(argument) != 4 or \
-               argument[1] not in [self.sakura_name, self.kero_name]: ## FIXME: HWND
+        list_hwnd = [self.sakura_name, self.kero_name] ## FIXME: HWND
+        if name in ['MOVE', 'MOVE_INSIDE', 'MOVETO', 'MOVETO_INSIDE']:
+            if len(argument) != 4 or argument[1] not in list_hwnd:
                 result = 0
-        elif name in ['MOVETO', 'MOVETO_INSIDE']:
-            if len(argument) != 4 or \
-               argument[1] not in [self.sakura_name, self.kero_name]: ## FIXME: HWND
-                result = 0
-        elif name == 'ZMOVE':
-            if len(argument) != 3 or \
-               argument[1] not in [self.sakura_name, self.kero_name]: ## FIXME: HWND
+        elif name in ['ZMOVE', 'WAIT']:
+            if len(argument) != 3 or argument[1] not in list_hwnd:
                 result = 0
         elif name in ['STANDBY', 'STANDBY_INSIDE']:
             if len(argument) != 6 or \
-               argument[1] not in [self.sakura_name, self.kero_name] or \
-	       argument[2] not in [self.sakura_name, self.kero_name]: ## FIXME: HWND
+               argument[1] not in list_hwnd or argument[2] not in list_hwnd:
                 result = 0
-        elif name == 'GET_POSITION':
-            if len(argument) != 2 or \
-               argument[1] not in [self.sakura_name, self.kero_name]: ## FIXME: HWND
+        elif name in ['GET_POSITION', 'CLEAR']:
+            if len(argument) != 2 or argument[1] not in list_hwnd:
                 result = 0
         elif name == 'GET_DESKTOP_SIZE':
             if len(argument) != 1:
                 result = 0
-        elif name == 'WAIT':
-            if len(argument) != 3 or \
-               argument[1] not in [self.sakura_name, self.kero_name]: ## FIXME: HWND
-                result = 0
-        elif name == 'CLEAR':
-            if len(argument) != 2 or \
-               argument[1] not in [self.sakura_name, self.kero_name]: ## FIXME: HWND
-                result = 0
         elif name == 'NOTIFY':
             if len(argument) < 3 or len(argument) > 8 or \
-               argument[1] not in [self.sakura_name, self.kero_name]: ## FIXME: HWND
+               argument[1] not in list_hwnd:
                 result = 0
         return result
 
     def request(self, req): ## FIXME
         req_type, argument = self.evaluate_request(req)
+
         if not req_type:
-            return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+            result = self.RESPONSE[400]
         elif req_type == 'GET Version':
-            return 'SAORI/1.0 204 No Content\r\n\r\n'
+            result = self.RESPONSE[204]
         elif req_type == 'EXECUTE':
             if not argument:
-                return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+                result = self.RESPONSE[400]
             elif not self.__check_argument(argument):
-                return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+                result = self.RESPONSE[400]
             else:
                 name = argument[0]
                 if name == 'GET_POSITION':
@@ -127,53 +113,53 @@ class Saori(SAORI):
                     elif argument[1] == self.kero_name: ## FIXME: HWND
                         side = 1
                     else:
-                        return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+                        return self.RESPONSE[400]
                     try:
                         x, y = self.__sakura.get_surface_position(side)
                         w, h = self.__sakura.get_surface_size(side)
-                        return 'SAORI/1.0 200 OK\r\n' \
-                               'Result: %d\r\n' \
-                               'Value0: %d\r\n' \
-                               'Value1: %d\r\n' \
-                               'Value2: %d\r\n\r\n' % (x, x, x + w / 2, x + w)
+                        result = 'SAORI/1.0 200 OK\r\n' \
+                            'Result: %d\r\n' \
+                            'Value0: %d\r\n' \
+                            'Value1: %d\r\n' \
+                            'Value2: %d\r\n\r\n' % (x, x, x + w / 2, x + w)
                     except:
-                        return 'SAORI/1.0 500 Internal Server Error\r\n\r\n'
+                        result = self.RESPONSE[500]
                 elif name == 'GET_DESKTOP_SIZE':
 	            try:
                         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
-                        return 'SAORI/1.0 200 OK\r\n' \
-                               'Result: %d\r\n' \
-                               'Value0: %d\r\n' \
-                               'Value1: %d\r\n\r\n' % (scrn_w, scrn_w, scrn_h)
+                        result = 'SAORI/1.0 200 OK\r\n' \
+                            'Result: %d\r\n' \
+                            'Value0: %d\r\n' \
+                            'Value1: %d\r\n\r\n' % (scrn_w, scrn_w, scrn_h)
 	            except:
-                        return 'SAORI/1.0 500 Internal Server Error\r\n\r\n'
+                        result = self.RESPONSE[500]
                 else:
                     self.enqueue_commands(name, argument[1:])
                     if self.timeout_id is None:
                         self.do_idle_tasks()
-                    return 'SAORI/1.0 204 No Content\r\n\r\n'
+                    result = self.RESPONSE[204]
         else:
-            return 'SAORI/1.0 400 Bad Request\r\n\r\n'
+            result = self.RESPONSE[400]
+        return result
 
     def enqueue_commands(self, command, args):
-	if command in ['MOVE', 'MOVE_INSIDE', 'MOVETO', 'MOVETO_INSIDE',
-                       'ZMOVE', 'WAIT', 'NOTIFY']:
-            if args[0] == self.sakura_name: ## FIXME: HWND
-	        self.commands[0].append([command, args[1:]])
-            elif args[0] == self.kero_name: ## FIXME: HWND
-	        self.commands[1].append([command, args[1:]])
-	elif command in ['STANDBY', 'STANDBY_INSIDE']:
-	    self.commands[0] = []
-	    self.commands[1] = []
-            if args[0] == self.sakura_name: ## FIXME: HWND
-	        self.commands[0].append([command, args[1:]])
-            elif args[0] == self.kero_name: ## FIXME: HWND
-	        self.commands[1].append([command, args[1:]])
-	elif command == 'CLEAR':
-            if args[0] == self.sakura_name: ## FIXME: HWND
-	        self.commands[0] = []
-            elif args[0] == self.kero_name: ## FIXME: HWND
-	        self.commands[1] = []
+        #assert command in ['MOVE', 'MOVE_INSIDE', 'MOVETO', 'MOVETO_INSIDE',
+        #                   'ZMOVE', 'WAIT', 'NOTIFY',
+        #                   'STANDBY', 'STANDBY_INSIDE',
+        #                   'CLEAR']
+        if args[0] == self.sakura_name: ## FIXME: HWND
+            side = 0
+        elif args[0] == self.kero_name: ## FIXME: HWND
+            side = 1
+        else:
+            return # XXX
+	if command == 'CLEAR':
+            self.commands[side] = []
+        else:
+            if command in ['STANDBY', 'STANDBY_INSIDE']:
+                self.commands[0] = []
+                self.commands[1] = []
+            self.commands[side].append([command, args[1:]])
 
     def do_idle_tasks(self):
 	for side in [0, 1]:
