@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
-#  Copyright (C) 2004-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -40,7 +40,7 @@ def test():
     entry_db.add('#temp0', '\hほうほう。\e')
     entry_db.add('#temp2', '\hいい感じだね。\e')
     entry_db.add('#temp3', '\hなるほど。\e')
-    for i in range(5):
+    for _ in range(5):
         print '#temp0', entry_db.get('#temp0')
     for key in ['#temp1', '#temp2', '#temp3']:
         print key, entry_db.get(key)
