@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -34,12 +34,6 @@ def get_ninix_user():
         return get_ninix_home()
     return os.path.expanduser(NINIX_USER)
 
-def get_gtkrc():
-    path = os.path.join(get_ninix_user(), 'gtkrc')
-    if os.path.exists(path):
-        return path
-    return os.path.join(get_ninix_home(), 'gtkrc')
-
 def get_pango_fontrc():
     return os.path.join(get_ninix_user(), 'pango_fontrc')
 
@@ -228,6 +222,7 @@ def read_kinoko_ini(top_dir):
             f.close()
             return None
         lineno = 0
+        error = None
         for line in f:
             lineno += 1
             if line.endswith(chr(0)): # XXX
@@ -247,6 +242,9 @@ def read_kinoko_ini(top_dir):
             elif name in ['ontop', 'baseposition', 'baseadjust']:
                 kinoko[name] = int(value)
         f.close()
+        if error:
+            sys.stderr.write('Error: %s\n%s (skipped)\n' % (error, path))
+            return None
     if kinoko['title']:
         return kinoko
     else:
@@ -274,6 +272,7 @@ def read_katochan_txt(top_dir): ## FIXME
         f = open(path)
         name = None
         lineno = 0
+        error = None
         for line in f:
             lineno += 1
             if not line.strip():
@@ -286,8 +285,14 @@ def read_katochan_txt(top_dir): ## FIXME
                 break
             else:
                 value = line.strip()
-                if name in ['name', 'category', 'before.script', 'hit.script',
-                            'after.script', 'end.script', 'dodge.script']:
+                if name in ['name', 'category']:
+                    katochan[name] = unicode(value, 'Shift_JIS', 'ignore')
+                if name.startswith('before.script') or \
+                        name.startswith('hit.script') or \
+                        name.startswith('after.script') or \
+                        name.startswith('end.script') or \
+                        name.startswith('dodge.script'):
+                    ## FIXME: should be array
                     katochan[name] = unicode(value, 'Shift_JIS', 'ignore')
                 elif name in ['before.fall.speed', 'before.slide.magnitude',
                               'before.slide.sinwave.degspeed',
@@ -308,8 +313,12 @@ def read_katochan_txt(top_dir): ## FIXME
                               'end.leave.direction',
                               'dodge.wave', 'dodge.wave.loop']:
                     katochan[name] = value
-                name = None
+                else:
+                    name = None
         f.close()
+        if error:
+            sys.stderr.write('Error: %s\n%s (skipped)\n' % (error, path))
+            return None
     if katochan['name']:
         return katochan
     else:
@@ -318,19 +327,19 @@ def read_katochan_txt(top_dir): ## FIXME
 def read_descript_txt(top_dir):
     path = os.path.join(top_dir, 'descript.txt')
     if is_readable(path):
-        return ninix.config.open(path)
+        return ninix.config.create_from_file(path)
     return None
 
 def read_install_txt(top_dir):
     path = os.path.join(top_dir, 'install.txt')
     if is_readable(path):
-        return ninix.config.open(path)
+        return ninix.config.create_from_file(path)
     return None
 
 def read_alias_txt(top_dir):
     path = os.path.join(top_dir, 'alias.txt')
     if is_readable(path):
-        return ninix.alias.open(path)
+        return ninix.alias.create_from_file(path)
     return None
 
 def find_makoto_dll(top_dir):
@@ -365,7 +374,7 @@ def find_surface_dir(top_dir):
     buf = []
     path = os.path.join(top_dir, 'surface.txt')
     if os.path.exists(path):
-        config = ninix.config.open(path)
+        config = ninix.config.create_from_file(path)
         for name, subdir in config.items():
             desc = read_descript_txt(os.path.join(top_dir, subdir.lower()))
             if desc is None:
@@ -395,7 +404,7 @@ def read_surface_info(surface_dir):
     filename_alias = {}
     path = os.path.join(surface_dir, 'alias.txt')
     if os.path.exists(path):
-        dic = ninix.alias.open(path)
+        dic = ninix.alias.create_from_file(path)
         for basename, alias in dic.iteritems():
             if basename.startswith('surface'):
                 filename_alias[alias] = basename
@@ -415,12 +424,12 @@ def read_surface_info(surface_dir):
         key = ''.join(('surface', str(int(match.group(1)))))
         txt = os.path.join(surface_dir, ''.join((basename, 's.txt')))
         if is_readable(txt):
-            config = ninix.config.open(txt)
+            config = ninix.config.create_from_file(txt)
         else:
             config = ninix.config.null_config()
         txt = os.path.join(surface_dir, ''.join((basename, 'a.txt')))
         if is_readable(txt):
-            config.update(ninix.config.open(txt))
+            config.update(ninix.config.create_from_file(txt))
         surface[key] = (img, config)
     # find surfaces.txt
     alias = None
@@ -514,12 +523,12 @@ def read_surfaces_txt(surface_dir):
                     key = ''.join((key[:7], str(int(key[7:]))))
                 except ValueError:
                     pass
-                config_list.append((key, ninix.config.new_config(buf)))
+                config_list.append((key, ninix.config.create_from_buffer(buf)))
         except ValueError, error:
             print_error('%s: %s (parsing not completed)' % (path, error))
             break
     if alias_buffer:
-        config_list.append(('__alias__', ninix.alias.new_alias(alias_buffer)))
+        config_list.append(('__alias__', ninix.alias.create_from_buffer(alias_buffer)))
     config_list.append(('__tooltips__', tooltips))
     return config_list
 
@@ -574,7 +583,7 @@ def read_balloon_info(balloon_dir):
         key = match.group(1)
         txt = os.path.join(balloon_dir, 'balloon%ss.txt' % key)
         if is_readable(txt):
-            config = ninix.config.open(txt)
+            config = ninix.config.create_from_file(txt)
         else:
             config = ninix.config.null_config()
         balloon[key] = (img, config)
