@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2004-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -28,8 +28,8 @@ import ninix.seriko
 
 class Menu:
 
-    def __init__(self, kinoko, accelgroup):
-        self.__kinoko = kinoko
+    def __init__(self, callback, accelgroup):
+        self.callback = callback
         ui_info = '''
         <ui>
           <popup name='popup'>
@@ -43,12 +43,12 @@ class Menu:
         '''
         self.__menu_list = {
             'settings': [('Settings', None, ('Settings...(_O)'), None,
-                          '', self.__kinoko.edit_preferences),
+                          '', self.callback['edit_preferences']),
                          '/ui/popup/Settings'],
             'skin':     [('Skin', None, _('Skin(_K)'), None),
                          None, '/ui/popup/Skin'],
             'exit':     [('Exit', None, _('Exit(_Q)'), None,
-                          '', self.__kinoko.close),
+                          '', self.callback['close']),
                          '/ui/popup/Exit'],
             }
         self.__skin_list = None
@@ -66,7 +66,7 @@ class Menu:
             self.__menu_list[key][1] = ui_manager.get_widget(path)
 
     def popup(self, button):
-        skin_list = self.__kinoko.get_skin_list()
+        skin_list = self.callback['get_skin_list']()
         self.__set_skin_menu(skin_list)
         self.__popup_menu.popup(
             None, None, None, button, gtk.get_current_event_time())
@@ -77,7 +77,7 @@ class Menu:
             menu = gtk.Menu()
             for skin in list:
                 item = gtk.MenuItem(skin['title'])
-                item.connect('activate', self.__kinoko.select_skin, (skin))
+                item.connect('activate', self.callback['select_skin'], (skin))
                 menu.add(item)
                 item.show()
             self.__menu_list[key][1].set_submenu(menu)
@@ -86,10 +86,12 @@ class Menu:
         else:
             self.__menu_list[key][1].hide()
 
+
 class Nayuki:
 
-    def __init__(self, kinoko):
-        self.kinoko = kinoko
+    def __init__(self):
+        pass
+
 
 class Kinoko:
 
@@ -112,8 +114,8 @@ class Kinoko:
         if event in ['set position', 'set surface']:
             self.skin.set_position()
             self.skin.show()
-        elif event == 'set scale': ## FIXME
-            scale = self.target.prefs.get('surface_scale')
+        elif event == 'set scale':
+            scale = self.target.get_surface_scale()
             self.skin.set_scale(scale)
         elif event == 'hide':
             if not self.target.surface_is_shown(0):
@@ -136,13 +138,24 @@ class Kinoko:
             ##print 'OBSERVER(kinoko): ignore -', event
             pass
 
+    def load_skin(self):
+        scale = self.target.get_surface_scale()
+        skin_callback ={
+            'edit_preferences': self.edit_preferences,
+            'close': self.close,
+            'get_skin_list': self.get_skin_list,
+            'select_skin': self.select_skin,
+            'get_target_window': lambda *a: self.target.surface.window[0].window, # XXX
+            'get_kinoko_position': self.target.get_kinoko_position,
+            }
+        self.skin = Skin(self.data, self.accelgroup, skin_callback, scale)
+
     def load(self, data, target):
         self.data = data
         self.target = target
         self.target.set_observer(self)
         self.accelgroup = gtk.AccelGroup()
-        scale = self.target.prefs.get('surface_scale')
-        self.skin = Skin(self.data, self.accelgroup, self, scale)
+        self.load_skin()
         if self.skin is None:
             return 0
         else:
@@ -179,22 +192,26 @@ class Kinoko:
         self.send_event('OnKinokoObjectChanging')
         self.skin.destroy()
         self.data = args
-        scale = self.target.prefs.get('surface_scale')
-        self.skin = Skin(self.data, self.accelgroup, self, scale)
+        self.load_skin()
         if self.skin is None:
             return 0
         else:
             self.send_event('OnKinokoObjectChanged')
         return 1
 
+
 class Skin:
 
-    def __init__(self, data, accelgroup, kinoko, scale):
+    def __init__(self, data, accelgroup, callback, scale):
         self.frame_buffer = []
         self.data = data
         self.accelgroup = accelgroup
-        self.kinoko = kinoko
-        self.__menu = Menu(self.kinoko, self.accelgroup)
+        self.callback = callback
+        menu_callback ={}
+        for key in ['edit_preferences', 'close',
+                    'get_skin_list', 'select_skin']:
+            menu_callback[key] = self.callback[key]
+        self.__menu = Menu(menu_callback, self.accelgroup)
         self.__scale = scale
         self.__shown = 0
         self.surface_id = 0 # dummy
@@ -217,30 +234,25 @@ class Skin:
         if self.data['animation'] is not None:
             path = os.path.join(self.data['dir'], self.data['animation'])
             self.seriko = ninix.seriko.Controler(
-                {'': ninix.seriko.get_actors(ninix.config.open(path))})
+                {'': ninix.seriko.get_actors(ninix.config.create_from_file(path))})
         else:
             base, ext = os.path.splitext(self.data['base'])
             path = os.path.join(self.data['dir'], ''.join((base, 'a.txt')))
             if os.path.exists(path):
                 self.seriko = ninix.seriko.Controler(
-                    {'': ninix.seriko.get_actors(ninix.config.open(path))})
+                    {'': ninix.seriko.get_actors(ninix.config.create_from_file(path))})
             else:
                 self.seriko = ninix.seriko.Controler({'': []})
         path = os.path.join(self.data['dir'], self.data['base'])
         try:
             self.pixbuf = ninix.pix.create_pixbuf_from_file(path)
-            scale = self.__scale
-            if scale != 100:
-                w = max(1, self.pixbuf.get_width() * scale / 100)
-                h = max(1, self.pixbuf.get_height() * scale / 100)
-                surface_pixbuf = self.pixbuf.scale_simple(
-                    w, h, gtk.gdk.INTERP_BILINEAR)
-            else:
-                surface_pixbuf = self.pixbuf.copy()
+            w = max(8, self.pixbuf.get_width() * self.__scale / 100)
+            h = max(8, self.pixbuf.get_height() * self.__scale / 100)
+            surface_pixbuf = self.pixbuf.scale_simple(
+                w, h, gtk.gdk.INTERP_BILINEAR)
             image, mask = surface_pixbuf.render_pixmap_and_mask(255)
-            del surface_pixbuf
         except: ## FIXME
-            self.kinoko.close(None)
+            self.callback['close'](None)
             return
         ## FIXME
         self.w, self.h = image.get_size()
@@ -254,7 +266,7 @@ class Skin:
         self.window.add(self.darea)
         self.darea.realize()
         self.darea.window.set_back_pixmap(image, False)
-        target_window = self.kinoko.target.surface.window[0].window # XXX
+        target_window = self.callback['get_target_window']()
         if self.data['ontop']:
             self.window.set_transient_for(target_window)
         else:
@@ -281,23 +293,19 @@ class Skin:
         self.seriko.append_actor(frame, actor)
 
     def set_position(self, xoffset=0, yoffset=0):
-        base_x, base_y = self.kinoko.target.get_kinoko_position(
+        base_x, base_y = self.callback['get_kinoko_position'](
             self.data['baseposition'])
         a, b = [(0.5, 1), (0.5, 0), (0, 0.5), (1, 0.5), (0, 1),
                 (1, 1), (0, 0), (1, 0), (0.5, 0.5)][self.data['baseadjust']]
-        offsetx = self.data['offsetx']
-        offsety = self.data['offsety']
-        scale = self.__scale
-        if scale != 100:
-            offsetx = offsetx * scale / 100
-            offsety = offsety * scale / 100
+        offsetx = self.data['offsetx'] * self.__scale / 100
+        offsety = self.data['offsety'] * self.__scale / 100
         self.x = base_x - int(self.w * a) + offsetx + xoffset
         self.y = base_y - int(self.h * b) + offsety + yoffset
         self.window.move(self.x, self.y)
 
     def set_scale(self, scale):
         self.__scale = scale
-        self.udpate()
+        self.update()
         self.set_position()
 
     def get_surface(self): ## FIXME
@@ -369,14 +377,11 @@ class Skin:
                 return
         else:
             surface_pixbuf = self.create_surface_pixbuf()
-        scale = self.__scale
-        if scale != 100:
-            w = max(1, surface_pixbuf.get_width() * scale / 100)
-            h = max(1, surface_pixbuf.get_height() * scale / 100)
-            surface_pixbuf = surface_pixbuf.scale_simple(
-                w, h, gtk.gdk.INTERP_BILINEAR)
+        w = max(8, surface_pixbuf.get_width() * self.__scale / 100)
+        h = max(8, surface_pixbuf.get_height() * self.__scale / 100)
+        surface_pixbuf = surface_pixbuf.scale_simple(
+            w, h, gtk.gdk.INTERP_BILINEAR)
         image, mask = surface_pixbuf.render_pixmap_and_mask(255)
-        del surface_pixbuf
         self.w, self.h = image.get_size()
         self.darea.window.set_back_pixmap(image, False)
         self.window.shape_combine_mask(mask, 0, 0)
@@ -404,7 +409,7 @@ class Skin:
         self.seriko.invoke(self, actor_id, update)
 
     def delete(self, widget, event):
-        self.kinoko.close(None)
+        self.callback['close'](None)
 
     def destroy(self): ## FIXME
         self.seriko.terminate(self)
