@@ -3,7 +3,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003-2005 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -14,11 +14,12 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
-import getopt
+import optparse
 import os
 import signal
 import socket
 import sys
+import shutil
 import random
 import gettext
 import traceback
@@ -45,22 +46,20 @@ import ninix.sakura
 import ninix.sstp
 import ninix.communicate
 import ninix.ngm
-import ninix.version
+import ninix.plugin
 
 import ninix.nekodorif
 import ninix.kinoko
 
-USAGE = '''\
-Usage: ninix [options]
-Options:
-  -H, --homedir DIR   ninix home directory (default: ~/.ninix)
-  -U, --userdir DIR   user data directory (default: ninix home directory)
-  --sstp-port NUM     additional port for listening SSTP requests
-  -h, --help          show this message
-'''
-
-def usage():
-    raise SystemExit, USAGE
+USAGE = 'Usage: ninix [options]'
+parser = optparse.OptionParser(USAGE)
+parser.add_option('-H', '--homedir', type='string', dest='homedir',
+                  help='ninix home directory (default: ~/.ninix)')
+parser.add_option('-U', '--userdir', type='string', dest='userdir',
+                  help='user data directory (default: ninix home directory)')
+parser.add_option('--sstp-port', type='int', dest='sstp_port',
+                  help='additional port for listening SSTP requests')
+parser.add_option('--debug', type='int', dest='debug', help='debug')
 
 def _traceback(exception_type, value, tb):
     response_id = 1
@@ -114,18 +113,10 @@ def main():
     if 'DISPLAY' not in os.environ:
         raise SystemExit, 'Error: cannot open display (abort)\n'
     # parse command line arguments
-    try:
-        options, rest = getopt.getopt(sys.argv[1:], 'H:U:R:ph',
-            ['homedir=', 'userdir=', 'sstp-port=', 'debug=', 'help'])
-    except getopt.error, e:
-        sys.stderr.write('Error: %s\n' % str(e))
-        usage()
+    (options, rest) = parser.parse_args()
     if rest:
-        usage()
-    home_dir = os.environ.get('NINIX_HOME')
-    user_dir = os.environ.get('NINIX_USER')
+        parser.error('Unknown option(s)')
     sstp_port = [9801, 11000]
-    debug = 0
     # check environment variables
     val = os.environ.get('NINIX_SSTP_PORT')
     if val:
@@ -142,21 +133,16 @@ def main():
         else:
             sstp_port.extend(port)
     # parse command line options
-    for opt, val in options:
-        if opt in ['-H', '--homedir']:
-            home_dir = val
-        elif opt in ['-U', '--userdir']:
-            user_dir = val
-        elif opt == '--sstp-port':
-            num = int(val)
-            if num < 1024:
-                sys.stderr.write('Invalid --sstp-port number (ignored)\n')
-            else:
-                sstp_port.append(num)
-        elif opt == '--debug':
-            debug = int(val)
+    home_dir = options.homedir if options.homedir is not None else \
+        os.environ.get('NINIX_HOME')
+    user_dir = options.userdir if options.userdir is not None else \
+        os.environ.get('NINIX_USER')
+    if options.sstp_port is not None:
+        if options.sstp_port < 1024:
+            sys.stderr.write('Invalid --sstp-port number (ignored)\n')
         else:
-            usage()
+            sstp_port.append(options.sstp_port)
+    debug = options.debug if options.debug is not None else 0
     if home_dir:
         if not os.path.isabs(os.path.expanduser(home_dir)):
             home_dir = os.path.join(os.getcwd(), home_dir)
@@ -209,8 +195,6 @@ def main():
         print 'balloon:', desc.get('name', '<none>').encode('utf-8', 'ignore')
     for plugin_name in map(lambda p: p[0], plugins):
         print 'plugin:', plugin_name.encode('utf-8', 'ignore')
-    # read gtkrc
-    gtk.rc_parse(ninix.home.get_gtkrc())
     # start
     sys.stdout.write('loading...')
     sys.stdout.flush()
@@ -224,26 +208,15 @@ def main():
     lockfile.close()
 
 
+class SSTPControler:
 
-class Application:
-
-    def __init__(self, config, sstp_port=[9801, 11000], debug=0):
-        self.ghosts, self.shells, self.balloons, self.plugins, \
-                     self.nekoninni, self.katochan, self.kinoko = config
+    def __init__(self, callback, sstp_port):
+        self.callback = callback
         self.sstp_port = sstp_port
-        self.debug = debug
-        self.plugin_pids = []
         self.sstp_servers = []
         self.__sstp_queue = []
         self.__sstp_flag = 0
         self.__current_sender = None
-        # create preference dialog
-        self.prefs = ninix.prefs.PreferenceDialog(self)
-        # create usage dialog
-        self.usage_dialog = UsageDialog()
-        self.__communicate = ninix.communicate.Communicate()
-        # create ghost manager
-        self.__ngm = ninix.ngm.NGM(self)
 
     def enqueue_script_if_ghost(self, if_ghost, script, sender, handle,
                                 address, show_sstp_marker,
@@ -252,6 +225,16 @@ class Application:
             (if_ghost, script, sender, handle, address, show_sstp_marker,
              use_translator, entry_db))
 
+    def check_request_queue(self, sender):
+        count = 0
+        for request in self.__sstp_queue:
+            if request[2].split(' / ')[0] == sender.split(' / ')[0]:
+                count += 1
+        if self.__sstp_flag and \
+           self.__current_sender.split(' / ')[0] == sender.split(' / ')[0]:
+            count += 1
+        return str(count), str(len(self.__sstp_queue))
+
     def set_sstp_flag(self, sender):
         self.__sstp_flag = 1
         self.__current_sender = sender
@@ -263,20 +246,16 @@ class Application:
     def handle_sstp_queue(self):
         if self.__sstp_flag or not self.__sstp_queue:
             return
-        if_ghost = self.__sstp_queue[0][0]
-        sakura = self.get_ghost(if_ghost)
-        if sakura is not None:
-            self.select_current_sakura(if_ghost)
-        elif not self.prefs.get('allowembryo'): # Refuse
-            self.__sstp_queue.pop(0)
-            return
         if_ghost, script, sender, handle, address, show_sstp_marker, \
                   use_translator, entry_db = self.__sstp_queue.pop(0)
+        if not self.callback['if_ghost'](if_ghost) and \
+                not self.callback['get_preference']('allowembryo'): # Refuse
+            return
+        self.callback['select_current_sakura'](if_ghost)
         self.set_sstp_flag(sender)
-        sakura = self.ghost_list[self.current_sakura[1]]['instance']
-        sakura.enter_temp_mode()
-        sakura.enqueue_script(script, sender, handle, address,
-                              show_sstp_marker, use_translator, entry_db)
+        self.callback['enqueue_script'](
+            script, sender, handle, address,
+            show_sstp_marker, use_translator, entry_db, temp_mode=True)
 
     def receive_sstp_request(self):
         try:
@@ -287,10 +266,151 @@ class Application:
         except ValueError: # may happen when ninix is terminated
             return
 
+    def get_sstp_port(self):
+        if not self.sstp_servers:
+            return None
+        return self.sstp_servers[0].server_address[1]
+
+    def quit(self):
+        for server in self.sstp_servers:
+            server.close()
+
+    def start_servers(self):
+        for port in self.sstp_port:
+            sstp_callback = {
+                'enqueue_script_if_ghost': self.enqueue_script_if_ghost,
+                'check_request_queue': self.check_request_queue,
+                }
+            for key in ['get_sakura_cantalk', 'get_event_response',
+                        'if_ghost', 'keep_silence',
+                        'get_ghost_names', 'get_ghost_name',
+                        'enqueue_event', 'enqueue_script']:
+                sstp_callback[key] = self.callback[key]
+            try:
+                server = ninix.sstp.SSTPServer(('', port), sstp_callback)
+            except socket.error, (code, message):
+                sys.stderr.write('Port %d: %s (ignored)\n' % (port, message))
+                continue
+            self.sstp_servers.append(server)
+            print 'Serving SSTP on port', port
+
+
+class Application(object):
+
+    def __init__(self, config, sstp_port=[9801, 11000], debug=0):
+        self.ghosts, self.shells, self.balloons, self.plugins, \
+                     self.nekoninni, self.katochan, self.kinoko = config ## FIXME
+        self.debug = debug
+        # create preference dialog
+        prefs_callback = {
+            'notify_preference_changed': self.notify_preference_changed,
+            'get_balloons': self.get_balloons,
+            }
+        self.prefs = ninix.prefs.PreferenceDialog(prefs_callback)
+        sstp_controler_callback = {
+            'get_sakura_cantalk': self.get_sakura_cantalk,
+            'get_event_response': self.get_event_response,
+            'if_ghost': self.if_ghost,
+            'keep_silence': self.keep_silence,
+            'get_ghost_names': self.get_ghost_names,
+            'get_ghost_name': self.get_ghost_name,
+            'enqueue_event': self.enqueue_event,
+            'enqueue_script': self.enqueue_script,
+            'select_current_sakura': self.select_current_sakura,
+            'get_preference': self.prefs.get,
+            }
+        self.sstp_controler = SSTPControler(sstp_controler_callback, sstp_port)
+        plugin_callback = {
+            'get_sstp_port': self.sstp_controler.get_sstp_port,
+            }
+        self.plugin_controler = ninix.plugin.PluginControler(
+            self.plugins, plugin_callback)
+        # create usage dialog
+        self.usage_dialog = UsageDialog()
+        self.communicate = ninix.communicate.Communicate()
+        # create ghost manager
+        ngm_callback = {
+            'update_sakura': self.update_sakura,
+            }
+        self.__ngm = ninix.ngm.NGM(ngm_callback)
+        self._current_sakura = None
+
+    def _set_current_sakura(self, item):
+        self._current_sakura = item
+        self.prefs.set_current_sakura(*self.get_current_sakura_name())
+
+    def _get_current_sakura(self):
+        return self._current_sakura
+
+    current_sakura = property(_get_current_sakura, _set_current_sakura)
+
     def create_ghost(self, data, debug):
         default_path = os.path.join(os.environ['PYTHONPATH'], 'ninix/dll')
-        return ninix.sakura.Sakura(self, data, default_path,
-                                   self.__communicate, debug)
+        sakura_callback = {
+            'change_sakura': self.change_sakura,
+            'close_all': self.close_all_ghosts,
+            'close_ghost': self.close_ghost,
+            'edit_preferences': self.prefs.edit_preferences,
+            'find_balloon_by_name': self.find_balloon_by_name,
+            'get_preference': self.prefs.get,
+            'get_balloon_list': self.get_balloon_list,
+            'get_balloon_description': self.get_balloon_description,
+            'get_ghost_list': self.get_ghost_list,
+            'get_kinoko_list': self.get_kinoko_list,
+            'get_nekodorif_list': self.get_nekodorif_list,
+            'get_plugin_list': self.get_plugin_list,
+            'get_shell_list': self.get_shell_list,
+            'get_otherghostname': self.communicate.get_otherghostname,
+            'open_ghost_manager': self.open_ghost_manager,
+            'rebuild_ghostdb': self.communicate.rebuild_ghostdb,
+            'reset_sstp_flag': self.sstp_controler.reset_sstp_flag,
+            'request_shell': self.request_shell,
+            'request_balloon': self.request_balloon,
+            'show_usage': self.show_usage,
+            'select_plugin': self.select_plugin,
+            'select_kinoko': self.select_kinoko,
+            'select_nekodorif': self.select_nekodorif,
+            'select_ghost': self.select_ghost,
+            'select_ghost_by_name': self.select_ghost_by_name,
+            'select_shell_by_name': self.select_shell_by_name,
+            'start_sakura': self.start_sakura_cb,
+            'stop_sakura': self.stop_sakura,
+            'select_current_sakura': self.select_current_sakura,
+            'send_message': self.communicate.send_message,
+            'update_sakura': self.update_sakura,
+            'vanish_sakura': self.vanish_sakura,
+            }
+        return ninix.sakura.Sakura(data, default_path, sakura_callback, debug)
+
+    def get_sakura_cantalk(self):
+        sakura = self.get_current_sakura_instance()
+        return sakura.cantalk
+
+    def get_event_response(self, event, *arglist, **argdict):
+        sakura = self.get_current_sakura_instance()
+        return sakura.get_event_response(*event)
+
+    def keep_silence(self, quiet):
+        sakura = self.get_current_sakura_instance()
+        sakura.keep_silence(quiet)
+
+    def get_ghost_name(self):
+        sakura = self.get_current_sakura_instance()
+        return '%s,%s' % (sakura.get_selfname(), sakura.get_keroname())
+
+    def enqueue_event(self, event, *arglist, **argdict):
+        sakura = self.get_current_sakura_instance()
+        sakura.enqueue_event(event, *arglist, **argdict)
+
+    def enqueue_script(self, script, sender, handle,
+                       host, show_sstp_marker, use_translator,
+                       db=None, request_handler=None, temp_mode=False):
+        sakura = self.get_current_sakura_instance()
+        if temp_mode:
+            sakura.enter_temp_mode()
+        sakura.enqueue_script(script, sender, handle,
+                              host, show_sstp_marker, use_translator,
+                              db, request_handler)
 
     def get_working_ghost(self, cantalk=0):
         working_list = [item['instance'] for item in self.ghost_list \
@@ -324,12 +444,12 @@ class Application:
             icon_path = None
         if icon_path is not None:
             try:
-                pixbuf = ninix.pix.create_pixbuf_from_file(icon_path, is_pnr=0)
+                pixbuf = ninix.pix.create_pixbuf_from_file(
+                    icon_path, is_pnr=False)
             except:
                 pixbuf = None
             if pixbuf is not None:
                 icon = pixbuf.scale_simple(16, 16, gtk.gdk.INTERP_BILINEAR)
-                del pixbuf
             else:
                 icon = None
         else:
@@ -384,6 +504,9 @@ class Application:
             set_list.append(item)
         return set_list
 
+    def get_balloons(self):
+        return self.balloons
+
     def get_balloon_list(self): ## FIXME
         balloon_list = []
         for i in range(len(self.balloons)):
@@ -397,24 +520,8 @@ class Application:
             balloon_list.append(item)
         return balloon_list
 
-    def get_plugin_list(self):
-        plugin_list = []
-        for i, plugin in enumerate(self.plugins):
-            plugin_name = plugin[0]
-            if not menu_items:
-                continue
-            item = {}
-            item['name'] = plugin_name
-            item['icon'] = None
-            item_list = []
-            menu_items = plugin[3]
-            for j, menu_item in enumerate(menu_items):
-                label = menu_item[0]
-                value = (i, j)
-                item_list.append((label, value))
-            item['items'] = item_list
-            plugin_list.append(item)
-        return plugin_list
+    def get_plugin_list(self): ## FIXME
+        return self.plugin_controler.get_plugin_list()
 
     def get_nekodorif_list(self):
         nekodorif_list = []
@@ -498,40 +605,17 @@ class Application:
     def run(self): ## FIXME
         self.load()
         # set SIGCHLD handler
-        signal.signal(signal.SIGCHLD, self.terminate_plugin)
+        signal.signal(signal.SIGCHLD, self.plugin_controler.terminate_plugin)
         # start SSTP server
-        for port in self.sstp_port:
-            try:
-                server = ninix.sstp.SSTPServer(('', port), self)
-            except socket.error, (code, message):
-                sys.stderr.write('Port %d: %s (ignored)\n' % (port, message))
-                continue
-            self.sstp_servers.append(server)
-            print 'Serving SSTP on port', port
+        self.sstp_controler.start_servers()
         # start plugins
-        try:
-            os.setpgid(0, 0)
-        except OSError:
-            pass
-        for plugin_name, plugin_dir, startup, menu_items in self.plugins: ## FIXME
-            if startup is not None:
-                self.exec_plugin(plugin_dir, startup)
+        self.plugin_controler.start_plugins()
         self.set_timeout()
         gtk.main()
 
-    def get_current_sakura(self):
+    def get_current_sakura_instance(self):
         return self.ghost_list[self.current_sakura[1]]['instance']
 
-    def check_request_queue(self, sender):
-        count = 0
-        for request in self.__sstp_queue:
-            if request[2].split(' / ')[0] == sender.split(' / ')[0]:
-                count += 1
-        if self.__sstp_flag and \
-           self.__current_sender.split(' / ')[0] == sender.split(' / ')[0]:
-            count += 1
-        return str(count), str(len(self.__sstp_queue))
-
     def get_ghost_names(self): ## FIXME
         name_list = []
         for item in self.ghost_list: ## FIXME
@@ -564,6 +648,16 @@ class Application:
         else:
             return 0
 
+    def update_sakura(self, name, sender):
+        item = self.find_ghost_by_name(name, 0)
+        if item is None:
+            return
+        sakura = self.select_ghost_from_list(item)
+        if not sakura.is_running():
+            self.start_sakura(item, init=1)
+        sakura.enqueue_script('\![updatebymyself]\e', sender,
+                              None, None, 0, 0, None)
+
     def select_current_sakura(self, ifghost=None):
         if ifghost is not None:
             for item in self.ghost_list:
@@ -583,10 +677,14 @@ class Application:
                     break
                 else:
                     pass
+            else:
+                return
         else:
             working_list = self.get_working_ghost(cantalk=1)
             if working_list:
                 self.current_sakura = random.choice(working_list).current
+            else:
+                return ## FIXME
 
     def close_ghost(self, sakura):
         if not self.get_working_ghost():
@@ -601,8 +699,7 @@ class Application:
     def quit(self):
         gobject.source_remove(self.timeout_id)
         self.usage_dialog.close()
-        for server in self.sstp_servers:
-            server.close()
+        self.sstp_controler.quit() ## FIXME
         self.save_preferences()
         gtk.main_quit()
         # stop plugins
@@ -625,9 +722,10 @@ class Application:
         except IOError:
             sys.stderr.write('Cannot write preferences to file (ignored).\n')
 
-    def select_ghost(self, sakura, sequential, event=1):
+    def select_ghost(self, sakura, sequential, event=1, vanished=0):
         if len(self.range_ghosts()) < 2:
             return
+        # select another ghost
         set_type, i, j = sakura.current
         if set_type == 's':
             i = 0
@@ -639,7 +737,9 @@ class Application:
             index_list = self.range_ghosts()
             index_list.remove(i)
             i = random.choice(index_list)
-        self.change_sakura(sakura, ('g', i, 0), 'automatic', event)
+        if self.current_sakura == sakura.current: # XXX
+            self.current_sakura = ('g', i, 0)
+        self.change_sakura(sakura, ('g', i, 0), 'automatic', event, vanished)
 
     def select_shell_by_name(self, sakura, name, event=1):
         set_type, i, j = sakura.current
@@ -664,20 +764,10 @@ class Application:
             return
         self.change_sakura(sakura, item, 'automatic', event)
 
-    def select_sakura(self, event, args):
-        sakura, item = args
-        if sakura.busy(): ## FIXME
-            gtk.gdk.beep()
-            return
-        self.change_sakura(sakura, item, 'manual')
-
     def change_sakura(self, sakura, item, method, event=1, vanished=0):
         set_type, i, j = item
         assert self.ghosts[i] is not None
-        if sakura.current == item: ## FIXME: needs reloading?
-            assert sakura.get_ghost_time() != 0
-            if not sakura.notify_event('OnGhostChanged', sakura.get_selfname()):
-                sakura.notify_event('OnBoot', default=None)
+        if sakura.current == item: # XXX: needs reloading?
             return
         if set_type == 'g':
             desc, shiori_dir, use_makoto, surface_set, balloon, prefix, \
@@ -705,7 +795,7 @@ class Application:
             proc()
         elif sakura.current[0] == set_type and sakura.current[1] == i:
             assert sakura.current[2] != j
-            sakura.enqueue_event('OnShellChanging', name, path, proc=proc)
+            sakura.enqueue_event('OnShellChanging', name, surface_dir, proc=proc)
         else:
             name = surface_desc.get('sakura.name', desc.get('sakura.name'))
             sakura.enqueue_event('OnGhostChanging', name, method, proc=proc)
@@ -746,15 +836,13 @@ class Application:
             name = self.ghost_list[prev[1]]['instance'].get_selfname()
         else:
             name = None
-        sakura.set_use_pna(self.prefs.get('use_pna')) ## FIXME
-        sakura.set_surface_scale(self.prefs.get('surface_scale')) ## FIXME
-        sakura.set_balloon_scalling(self.prefs.get('balloon_scalling')) ## FIXME
-        sakura.set_surface_alpha(self.prefs.get('surface_alpha')) ## FIXME
-        sakura.set_balloon_alpha(self.prefs.get('balloon_alpha')) ## FIXME
-        sakura.set_animation_quality(self.prefs.get('animation_quality')) ## FIXME
-        sakura.set_seriko_inactive(self.prefs.get('seriko_inactive')) ## FIXME
+        sakura.notify_preference_changed()
         sakura.start(item, init, temp, vanished, ghost_changed, name)
 
+    def notify_preference_changed(self):
+        for sakura in self.get_working_ghost():
+            sakura.notify_preference_changed()
+
     def start_sakura_cb(self, event, item):
         self.start_sakura(item, init=1)
 
@@ -767,14 +855,6 @@ class Application:
         desc, balloon = self.balloons[number]
         return desc, balloon
 
-    def reload(self, sakura):
-        if sakura.busy():
-            gtk.gdk.beep()
-            return
-        def proc(self=self):
-            self.stop_sakura(sakura, self.reload_current_sakura, sakura)
-        sakura.enqueue_event('OnNinixReloading', proc=proc)
-
     def reload_current_sakura(self, sakura):
         self.save_preferences()
         set_type, i, j = sakura.current
@@ -791,89 +871,37 @@ class Application:
             self.shells.append(shell_conf[0]) ## FIXME
         self.update_ghost_list(i, sakura)
         item = (set_type, i, j)
-        self.start_sakura(item, item)
-        sakura.notify_event('OnNinixReloaded')
-        sakura.notify_event('OnBoot', default=ninix.version.VERSION_INFO)
+        self.start_sakura(item, item, init=1) 
 
     def vanish_sakura(self, sakura):
         # remove ghost
         prefix = sakura.get_prefix()
         for filename in os.listdir(prefix):
-            if filename != 'HISTORY':
-                command = ''.join(('rm -rf ', os.path.join(prefix, filename)))
-                print command
-                if os.system(command):
-                    print '***FAILED***'
-        # select another ghost
+            if os.path.isfile(os.path.join(prefix, filename)):
+                if filename != 'HISTORY':
+                    try:
+                        os.remove(os.path.join(prefix, filename))
+                    except:
+                        print '*** REMOVE FAILED *** :', filename
+            else: # dir
+                try:
+                    shutil.rmtree(os.path.join(prefix, filename))
+                except:
+                    print '*** REMOVE FAILED *** :', filename
         set_type, i, j = sakura.current
-        if set_type == 's':
-            i = 0
-        index_list = self.range_ghosts()
-        index_list.remove(i)
-        next_i = random.choice(index_list)
-        assert i != next_i
-        if self.current_sakura == sakura.current:
-            self.current_sakura = ('g', next_i, 0)
-        # reload and start the new ghost
+        self.select_ghost(sakura, 0, vanished=1)
         self.ghosts[i] = None
         self.update_ghost_list(i, sakura)
-        self.change_sakura(sakura, ('g', next_i, 0), 'automatic', vanished=1)
 
-    def select_plugin(self, event, item):
-        i, j = item
-        plugin_name, plugin_dir, startup, menu_items = self.plugins[i]
-        label, argv = menu_items[j]
-        self.exec_plugin(plugin_dir, argv)
+    def select_plugin(self, event, item): ## FIXME
+        self.plugin_controler.select_plugin(event, item)
 
-    def select_nekodorif(self, event, item): ## FIXME
-        target, nekodorif_dir = item
+    def select_nekodorif(self, nekodorif_dir, target):
         ninix.nekodorif.Nekoninni().load(nekodorif_dir, self.katochan, target)
 
-    def select_kinoko(self, event, item): ## FIXME
-        target, data = item
+    def select_kinoko(self, data, target):
         ninix.kinoko.Kinoko(self.kinoko).load(data, target)
 
-    def terminate_plugin(self, signum, frame):
-        for pid in self.plugin_pids[:]:
-            try:
-                (pid, status) = os.waitpid(pid, os.WNOHANG)
-            except OSError:
-                ##print 'Process %d not found.' % pid
-                self.plugin_pids.remove(pid)
-            if pid > 0:
-                ##print 'Process %d terminated.' % pid
-                self.plugin_pids.remove(pid)
-        ##print 'Running subprocesses:', self.plugin_pids
-
-    def get_sstp_port(self):
-        if not self.sstp_servers:
-            return None
-        return self.sstp_servers[0].server_address[1]
-
-    def exec_plugin(self, plugin_dir, argv):
-        ##print 'exec_plugin:', ' '.join(argv)
-        if not os.path.exists(argv[0]):
-            return
-        port = self.get_sstp_port()
-        if port is None:
-            port = 'none'
-        environ = os.environ.copy()
-        environ['NINIX_PID'] = str(os.getpid())
-        environ['NINIX_SSTP_PORT'] = str(port)
-        environ['NINIX_PLUGIN_DIR'] = plugin_dir
-        try:
-            pid = os.fork()
-        except OSError:
-            sys.stderr.write('Error: %s failed (ignored)\n' % argv[0])
-            return
-        if pid == 0:
-            os.chdir(plugin_dir)
-            try:
-                os.execve(argv[0], argv, environ)
-            except OSError:
-                raise SystemExit, 'Error: %s failed (abort)\n' % argv[0]
-        self.plugin_pids.append(pid)
-
     def open_ghost_manager(self):
         self.__ngm.show_dialog()
 
@@ -920,8 +948,8 @@ class Application:
         self.timeout_id = gobject.timeout_add(100, self.do_idle_tasks) # 100ms
 
     def do_idle_tasks(self):
-        self.handle_sstp_queue()
-        self.receive_sstp_request()
+        self.sstp_controler.handle_sstp_queue()
+        self.sstp_controler.receive_sstp_request()
         self.set_timeout()
 
 
@@ -964,7 +992,8 @@ class UsageDialog:
         if ai_list:
             path = random.choice(ai_list)
             assert os.path.exists(path)
-            self.pixbuf = ninix.pix.create_pixbuf_from_file(path, is_pnr=0)
+            self.pixbuf = ninix.pix.create_pixbuf_from_file(
+                path, is_pnr=False)
             self.pixbuf.saturate_and_pixelate(self.pixbuf, 1.0, True)
         else:
             self.pixbuf = None
