@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2004-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -58,8 +58,8 @@ import ninix.home
 
 class Menu:
 
-    def __init__(self, nekoninni, accelgroup):
-        self.__nekoninni = nekoninni
+    def __init__(self, callback, accelgroup):
+        self.callback = callback
         ui_info = '''
         <ui>
           <popup name='popup'>
@@ -73,12 +73,12 @@ class Menu:
         '''
         self.__menu_list = {
             'settings': [('Settings', None, _('Settings...(_O)'), None,
-                          '', self.__nekoninni.edit_preferences),
+                          '', self.callback['edit_preferences']),
                          '/ui/popup/Settings'],
             'katochan': [('Katochan', None,_('Katochan(_K)'), None),
                          '/ui/popup/Katochan'],
             'exit':     [('Exit', None,_('Exit(_Q)'), None,
-                          '', self.__nekoninni.close),
+                          '', self.callback['close']),
                          '/ui/popup/Exit'],
             }
         self.__katochan_list = None
@@ -96,7 +96,7 @@ class Menu:
             self.__menu_list[key][1] = ui_manager.get_widget(path)
 
     def popup(self, button):
-        katochan_list = self.__nekoninni.get_katochan_list()
+        katochan_list = self.callback['get_katochan_list']()
         self.__set_katochan_menu(katochan_list)
         self.__popup_menu.popup(
             None, None, None, button, gtk.get_current_event_time())
@@ -108,7 +108,7 @@ class Menu:
             for katochan in list:
                 item = gtk.MenuItem(katochan['name'])
                 item.connect(
-                    'activate', self.__nekoninni.select_katochan, (katochan))
+                    'activate', self.callback['select_katochan'], (katochan))
                 menu.add(item)
                 item.show()
             self.__menu_list[key][1].set_submenu(menu)
@@ -131,8 +131,8 @@ class Nekoninni:
                 self.skin.set_position()
             if self.katochan is not None and self.katochan.loaded:
                 self.katochan.set_position()
-        elif event == 'set scale': ## FIXME
-            scale = self.target.prefs.get('surface_scale')
+        elif event == 'set scale':
+            scale = self.target.get_surface_scale()
             if self.skin is not None:
                 self.skin.set_scale(scale)
             if self.katochan is not None:
@@ -150,8 +150,17 @@ class Nekoninni:
         self.target = target
         self.target.set_observer(self)
         self.accelgroup = gtk.AccelGroup()
-        scale = self.target.prefs.get('surface_scale')
-        self.skin = Skin(self.dir, self.accelgroup, self, scale)
+        scale = self.target.get_surface_scale()
+        skin_callback ={
+            'finalize': self.finalize,
+            'has_katochan': self.has_katochan,
+            'drop_katochan': self.drop_katochan,
+            'edit_preferences': self.edit_preferences,
+            'close': self.close,
+            'get_katochan_list': lambda *a: self.katochan_list,
+            'select_katochan': self.select_katochan,
+            }
+        self.skin = Skin(self.dir, self.accelgroup, skin_callback, scale)
         if self.skin is None:
             return 0
         self.katochan_list = katochan
@@ -188,10 +197,7 @@ class Nekoninni:
         self.target.notify_event('OnNekodorifObject%s' % event, *args)
 
     def has_katochan(self):
-        if self.katochan is not None:
-            return 1
-        else:
-            return 0
+        return int(self.katochan is not None)
 
     def select_katochan(self, widget, args):
         self.launch_katochan(args)
@@ -207,7 +213,13 @@ class Nekoninni:
     def launch_katochan(self, katochan):
         if self.katochan:
             self.katochan.destroy()
-        self.katochan = Katochan(self, self.target)
+        katochan_callback = {
+            'finalize': self.finalize,
+            'send_event': self.send_event,
+            'get_mode': lambda *a: self.mode,
+            'delete_katochan': self.delete_katochan,
+            }
+        self.katochan = Katochan(katochan_callback, self.target)
         self.katochan.load(katochan)
 
     def edit_preferences(self, action):
@@ -226,25 +238,23 @@ class Nekoninni:
     def close(self, action):
         self.finalize()
 
-    def get_katochan_list(self): ## FIXME
-        return self.katochan_list
 
 class Skin:
 
-    def __init__(self, dir, accelgroup, nekoninni, scale):
+    def __init__(self, dir, accelgroup, callback, scale):
         self.dir = dir
         self.accelgroup = accelgroup
-        self.nekoninni = nekoninni
-        self.__dragged = 0
+        self.callback = callback
         self.x_root = None
         self.y_root = None
         self.__scale = scale
-        self.__menu = Menu(self.nekoninni, self.accelgroup)
+        menu_callback = {}
+        for key in ['edit_preferences', 'close',
+                    'get_katochan_list', 'select_katochan']:
+            menu_callback[key] = self.callback[key]
+        self.__menu = Menu(menu_callback, self.accelgroup)
         path = os.path.join(self.dir, 'omni.txt')
-        if os.path.isfile(path) and os.path.getsize(path) == 0:
-            self.omni = 1
-        else:
-            self.omni = 0
+        self.omni = int(os.path.isfile(path) and os.path.getsize(path) == 0)
         self.window = gtk.Window()
         name, top_dir = ninix.home.read_profile_txt(dir) # XXX
         self.window.set_title(name)
@@ -282,7 +292,7 @@ class Skin:
         pass
 
     def delete(self, widget, event):
-        self.nekoninni.finalize()
+        self.callback['finalize']()
 
     def destroy(self): ## FIXME
         self.window.destroy()
@@ -297,9 +307,9 @@ class Skin:
                 ##    gtk.get_current_event_time())
                 pass
             elif event.type == gtk.gdk._2BUTTON_PRESS: # double click
-                if self.nekoninni.has_katochan():
+                if self.callback['has_katochan']():
                     self.start() ## FIXME
-                    self.nekoninni.drop_katochan()
+                    self.callback['drop_katochan']()
         elif event.button == 3:                
             if event.type == gtk.gdk.BUTTON_PRESS:
                 self.__menu.popup(event.button)
@@ -317,15 +327,12 @@ class Skin:
             path = os.path.join(self.dir, 'surface%d.png' % self.id[0])
         try:
             pixbuf = ninix.pix.create_pixbuf_from_file(path)
-            scale = self.__scale
-            if scale != 100:
-                w = max(1, pixbuf.get_width() * scale / 100)
-                h = max(1, pixbuf.get_height() * scale / 100)
-                pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
+            w = max(8, pixbuf.get_width() * self.__scale / 100)
+            h = max(8, pixbuf.get_height() * self.__scale / 100)
+            pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
             image, mask = pixbuf.render_pixmap_and_mask(255)
-            del pixbuf
         except:
-            self.nekoninni.finalize()
+            self.callback['finalize']()
             return
         self.w, self.h = image.get_size()
         self.darea.set_size_request(self.w, self.h)
@@ -366,14 +373,12 @@ class Skin:
         self.id[0] = 0
         self.set_surface()
 
-    def button_release(self,  widget, event): ## FIXME
-        if self.__dragged:
-            self.__dragged = 0
-            self.x_root = None
-            self.y_root = None
+    def button_release(self,  widget, event):
+        self.x_root = None
+        self.y_root = None
         return True
 
-    def motion_notify(self,  widget, event): ## FIXME
+    def motion_notify(self,  widget, event):
         if event.is_hint:
             x, y, state = self.darea.window.get_pointer()
         else:
@@ -383,7 +388,6 @@ class Skin:
                self.y_root is not None:
                 x_delta = int(event.x_root - self.x_root)
                 y_delta = int(event.y_root - self.y_root)
-                self.__dragged = 1
                 self.move(x_delta, y_delta)
                 self.x_root = event.x_root
                 self.y_root = event.y_root
@@ -415,10 +419,10 @@ class Katochan:
                      'other'      # 上記カテゴリに当てはまらないもの
                      ]
 
-    def __init__(self, nekoninni, target):
+    def __init__(self, callback, target):
         self.side = 0
         self.target = target
-        self.nekoninni = nekoninni
+        self.callback = callback
         self.settings = {}
         self.settings['state'] = 'before'
         self.settings['fall.type'] = 'gravity'
@@ -485,10 +489,8 @@ class Katochan:
         self.settings['slide.sinwave.degspeed'] = self.data.get(
             ''.join((timing, 'slide.sinwave.degspeed')), 30)
         self.settings['wave'] = self.data.get(''.join((timing, 'wave')), None)
-        if self.data.get(''.join((timing, 'wave.loop'))) == 'on':
-            self.settings['wave.loop'] = 1
-        else:
-            self.settings['wave.loop'] = 0
+        self.settings['wave.loop'] = int(
+            self.data.get(''.join((timing, 'wave.loop'))) == 'on')
 
     def set_scale(self, scale):
         self.__scale = scale
@@ -500,26 +502,22 @@ class Katochan:
             return
         target_x, target_y = self.target.get_surface_position(self.side)
         target_w, target_h = self.target.get_surface_size(self.side)
-        scale = self.__scale
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
         self.x = target_x + target_w / 2 - self.w / 2 + \
-                 self.offset_x * scale / 100
-        self.y = top + self.offset_y * scale / 100
+                 self.offset_x * self.__scale / 100
+        self.y = top + self.offset_y * self.__scale / 100
         self.window.move(self.x, self.y)
 
     def set_surface(self):
         path = os.path.join(self.data['dir'], 'surface%d.png' % self.id)
         try:
             pixbuf = ninix.pix.create_pixbuf_from_file(path)
-            scale = self.__scale
-            if scale != 100:
-                w = pixbuf.get_width() * scale / 100
-                h = pixbuf.get_height() * scale / 100
-                pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
+            w = max(8, pixbuf.get_width() * self.__scale / 100)
+            h = max(8, pixbuf.get_height() * self.__scale / 100)
+            pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
             image, mask = pixbuf.render_pixmap_and_mask(255)
-            del pixbuf
         except:
-            self.nekoninni.finalize()
+            self.callback['finalize']()
             return
         self.w, self.h = image.get_size()
         self.darea.set_size_request(self.w, self.h)
@@ -529,7 +527,7 @@ class Katochan:
 
     def load(self, data):
         self.data = data
-        self.__scale = self.target.prefs.get('surface_scale')
+        self.__scale = self.target.get_surface_scale()
         self.set_state('before')
         ##print 'DATA:', self.data
         if 'category' in self.data:
@@ -548,11 +546,11 @@ class Katochan:
             elif self.data['target'] == 'kero':
                 self.side = 1
             else:
-                self.side = 0
+                self.side = 0 # XXX
         else:
-            self.side = 0
-        if self.nekoninni.mode == 1:
-            self.nekoninni.send_event('Emerge')
+            self.side = 0 # XXX
+        if self.callback['get_mode']() == 1:
+            self.callback['send_event']('Emerge')
         else:
             if 'before.script' in self.data:
                 pass ## FIXME
@@ -654,10 +652,10 @@ class Katochan:
             if self.check_collision():
                 self.set_state('hit')
                 self.hit = 1
-                if self.nekoninni.mode == 1:
+                if self.callback['get_mode']() == 1:
                     self.id = 1
                     self.set_surface()
-                    self.nekoninni.send_event('Hit')
+                    self.callback['send_event']('Hit')
                 else:
                     pass ## FIXME
             if self.check_mikire():
@@ -666,10 +664,10 @@ class Katochan:
             if self.hit_stop >= self.data.get('hit.waittime', 0):
                 self.set_state('after')
                 self.set_movement('after')
-                if self.nekoninni.mode == 1:
+                if self.callback['get_mode'] == 1:
                     self.id = 2
                     self.set_surface()
-                    self.nekoninni.send_event('Drop')
+                    self.callback['send_event']('Drop')
                 else:
                     pass ## FIXME
             else:
@@ -681,18 +679,18 @@ class Katochan:
             if self.check_mikire():
                 self.set_state('end')
         elif self.settings['state'] == 'end':
-            if self.nekoninni.mode == 1:
-                self.nekoninni.send_event('Vanish')
+            if self.callback['get_mode']() == 1:
+                self.callback['send_event']('Vanish')
             else:
                 pass ## FIXME
-            self.nekoninni.delete_katochan()
+            self.callback['delete_katochan']()
             return False
         elif self.settings['state'] == 'dodge':
-            if self.nekoninni.mode == 1:
-                self.nekoninni.send_event('Dodge')
+            if self.callback['get_mode'] == 1:
+                self.callback['send_event']('Dodge')
             else:
                 pass ## FIXME
-            self.nekoninni.delete_katochan()
+            self.callback['delete_katochan']()
             return False
         else:
             pass ## check collision and mikire
