@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  Copyright (C) 2001-2004 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2004-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -14,7 +14,6 @@
 import codecs
 import os
 import re
-import sys
 import time
 import urllib
 import webbrowser
@@ -66,27 +65,25 @@ class Catalog_xml:
 
     # data handling functions
     def get(self, entry, key):
-        if key in entry:
-            return entry[key]
-        else:
-            return None
+        return entry.get(key) # XXX
 
     # updates etc
-    def network_update(self):
+    def network_update(self, updatehook):
         last_update = self.last_update
         self.last_update = time.strftime('%Y-%m-%d %H:%M:%S')
         if self.cgi:
             priority = self.cgi.keys()
             priority.sort()
-            url = self.cgi[priority[-1]]
+            url = self.cgi[priority[-1]][-1]
         else:
             url = MASTERLIST_URL
         try:
             f = urllib.urlopen(url, urllib.urlencode(
-                {'time': '"%s"' % last_update, 'charset': 'EUC-JP'}))
+                {'time': '"%s"' % last_update, 'charset': 'UTF-8'}))
         except:
             return ## FIXME
-        self.import_from_fileobj(f)
+        for _ in self.import_from_fileobj(f):
+            updatehook()
         self.save_MasterList()
         f.close()
 
@@ -94,7 +91,8 @@ class Catalog_xml:
     def load_MasterList(self):
         try:
             f = open(os.path.join(self.datadir, 'MasterList.xml'), 'r')
-            self.import_from_fileobj(f)
+            for _ in self.import_from_fileobj(f):
+                pass
             f.close()
         except IOError:
             return
@@ -126,27 +124,33 @@ class Catalog_xml:
         new_entry = {}
         set_id = None
         node = []
+        re_list = re.compile('<GhostList>')
+        re_setid = re.compile('<FileSet ID="?([0-9]+)"?>')
+        re_set = re.compile('</FileSet>')
+        re_priority = re.compile('<RequestCgiUrl Priority="?([0-9]+)"?>(.+)</RequestCgiUrl>')
+        re_misc = re.compile('<(.+)>(.+)</(.+)>')
         for line in fileobj:
+            yield
             assert nest >= 0
             line = unicode(line, encoding, 'ignore').encode('utf-8')
             if not line:
                 continue
-            m = re.compile('<GhostList>').search(line)
+            m = re_list.search(line)
             if m:
                 nest += 1
                 continue
-            m = re.compile('<FileSet ID="?([0-9]+)"?>').search(line)
+            m = re_setid.search(line)
             if m:
                 nest += 1
                 set_id = int(m.group(1))
                 continue
-            m = re.compile('</FileSet>').search(line)
+            m = re_set.search(line)
             if m:
                 nest -= 1
                 new_entry[set_id] = self.create_entry(node)
                 node = []
                 continue
-            m = re.compile('<RequestCgiUrl Priority="?([0-9]+)"?>(.+)</RequestCgiUrl>').search(line)
+            m = re_priority.search(line)
             if m:
                 g = m.groups()
                 priority = int(g[0])
@@ -156,7 +160,7 @@ class Catalog_xml:
                 else:
                     self.cgi[priority] = [url]
                 continue
-            m = re.compile('<(.+)>(.+)</(.+)>').search(line)
+            m = re_misc.search(line)
             if m:
                 g = m.groups()
                 if set_id is not None:
@@ -197,13 +201,13 @@ class Catalog_xml:
                 text = text.replace('>', '&gt;')
                 text = text.replace('"', '&quot;')
                 text = text.replace('\'', '&apos;')
-                text = text.encode('EUC-JP')
+                text = text.encode('UTF-8')
             else:
                 text = '' # fileobj.write('  <%s/>\n' % key)
             fileobj.write('  <%s>%s</%s>\n' % (key, text, key))
 
     def export_to_fileobj(self, fileobj):
-        fileobj.write('<?xml version="1.0" encoding="EUC-JP" ?>\n')
+        fileobj.write('<?xml version="1.0" encoding="UTF-8" ?>\n')
         fileobj.write('<GhostList>\n')
         fileobj.write('<LastUpdate>%s</LastUpdate>\n' % self.last_update)
         for key in ['SakuraPreviewBaseUrl', 'KeroPreviewBaseUrl',
@@ -237,39 +241,35 @@ class Catalog(Catalog_xml):
     def image_filename(self, set_id, side):
         p = '%s_%d.png' % (self.TYPE[side], set_id)
         d = os.path.join(self.datadir, p)
-        if os.path.exists(d):
-            return d
-        return None
+        return d if os.path.exists(d) else None
 
-    def retrieve_image(self, set_id, side):
+    def retrieve_image(self, set_id, side, updatehook):
         p = '%s_%d.png' % (self.TYPE[side], set_id)
         d = os.path.join(self.datadir, p)
         if not os.path.exists(d):
-            if side == 0:
-                url = self.url['SakuraPreviewBaseUrl']
-            else:
-                url = self.url['KeroPreviewBaseUrl']
+            url = self.url['SakuraPreviewBaseUrl'] if side == 0 else \
+                self.url['KeroPreviewBaseUrl']
             try:
-                urllib.urlretrieve(''.join((url, p)), d)
+                urllib.urlretrieve(''.join((url, p)), d, updatehook)
             except:
                 return ## FIXME
 
-class InstallationManager(list):
-
-    def __init__(self, ngm):
-        self.master = ngm
-
-    def set_current_ghost(self, set_id):
-        self.current_ghost = set_id
-
-    def install_ghost(self, dialog):
-        a = self.master.ui.catalog.get(self.current_ghost, 'ArchiveUrl')
-        os.system("%s '%s'" % (self.master.ninix_install_cmd, a))
+#class InstallationManager(list):
+#
+#    def __init__(self, ngm):
+#        self.master = ngm
+#
+#    def set_current_ghost(self, set_id):
+#        self.current_ghost = set_id
+#
+#    def install_ghost(self, dialog):
+#        a = self.master.ui.catalog.get(self.current_ghost, 'ArchiveUrl')
+#        os.system("%s '%s'" % (self.master.ninix_install_cmd, a))
 
 class SearchDialog:
 
-    def __init__(self, ui):
-        self.ui = ui
+    def __init__(self, callback):
+        self.callback = callback
         self.dialog = gtk.Dialog()
         self.dialog.connect('delete_event', self.cancel)
         self.dialog.set_modal(True)
@@ -304,7 +304,7 @@ class SearchDialog:
 
     def ok(self, widget, event=None):
         word = unicode(self.get_pattern(), 'utf-8')
-        self.ui.search(word)
+        self.callback['search'](word)
         self.hide()
         return True
 
@@ -313,8 +313,8 @@ class SearchDialog:
         return True
 
 class UI:
-    def __init__(self, ngm):
-        self.ngm = ngm
+    def __init__(self, callback):
+        self.callback = callback
         self.ui_info = '''
         <ui>
           <menubar name='MenuBar'>
@@ -355,7 +355,7 @@ class UI:
             ( 'Settings', None,
               _('Settings(_O)'), None,
               None,
-              lambda *a: self.ngm.open_preference_dialog() ),
+              lambda *a: self.callback['open_preference_dialog']() ),
             ( 'DB Network Update', None,
               _('DB Network Update(_N)'), None,
               None,
@@ -367,15 +367,15 @@ class UI:
             ( 'Mask', None,
               _('Mask(_M)'), None,
               None,
-              lambda *a: self.ngm.open_mask_dialog() ),
+              lambda *a: self.callback['open_mask_dialog']() ),
             ( 'Reset to Default', None,
               _('Reset to Default(_Y)'), None,
               None,
-              lambda *a: self.ngm.reset_to_default() ),
+              lambda *a: self.callback['reset_to_default']() ),
             ( 'Show All', None,
               _('Show All(_Z)'), None,
               None,
-              lambda *a: self.ngm.show_all() ),
+              lambda *a: self.callback['show_all']() ),
             )
         self.opened = 0
         self.textview = [None, None]
@@ -384,7 +384,10 @@ class UI:
         self.button = {}
         self.url = {}
         self.search_word = ''
-        self.search_dialog = SearchDialog(self)
+        search_dialog_callback = {
+            'search': self.search,
+            }
+        self.search_dialog = SearchDialog(search_dialog_callback)
         self.create_dialog()
 
     def create_dialog(self):
@@ -438,8 +441,13 @@ class UI:
         self.statusbar.show()
 
     def network_update(self):
-        self.ngm.network_update()
+        self.window.set_sensitive(False)
+        def updatehook(*args):
+            while gtk.events_pending():
+                gtk.main_iteration()
+        self.callback['network_update'](updatehook)
         self.update()
+        self.window.set_sensitive(True)
 
     def open_search_dialog(self):
         self.search_dialog.show(default=self.search_word)
@@ -447,24 +455,24 @@ class UI:
     def search(self, word):
         if word:
             self.search_word = word
-            if self.ngm.search(word):
+            if self.callback['search'](word):
                 self.update()
             else:
                 pass ## FIXME
 
     def search_forward(self):
         if self.search_word:
-            if self.ngm.search_forward(self.search_word):
+            if self.callback['search_forward'](self.search_word):
                 self.update()
             else:
                 pass ## FIXME
 
     def show_next(self):
-        self.ngm.next()
+        self.callback['next']()
         self.update()
 
     def show_previous(self):
-        self.ngm.previous()
+        self.callback['previous']()
         self.update()
 
     def create_surface_area(self, side):
@@ -485,20 +493,17 @@ class UI:
 
     def update_surface_area(self):
         for side in [0, 1]:
-            if side == 0:
-                name = self.ngm.get('SakuraName')
-            else:
-                name = self.ngm.get('KeroName')
+            name = self.callback['get']('SakuraName') if side == 0 else \
+                self.callback['get']('KeroName')
             textbuffer = self.textview[side].get_buffer()
             textbuffer.set_text(name)
-            filename = self.ngm.get_image_filename(side)
+            filename = self.callback['get_image_filename'](side)
             darea = self.darea[side]
             darea.realize()
             if filename is not None:
                 try:
                     pixbuf = ninix.pix.create_pixbuf_from_file(filename)
                     image, mask = pixbuf.render_pixmap_and_mask(255)
-                    del pixbuf
                 except:
                     darea.window.set_back_pixmap(None, False)
                 else:
@@ -518,12 +523,12 @@ class UI:
         box.set_layout(gtk.BUTTONBOX_SPREAD)
         box.show()
         button = gtk.Button(_('Install'))
-        button.connect('clicked', lambda b, w=self: w.ngm.install_current())
+        button.connect('clicked', lambda b, w=self: w.callback['install_current']())
         box.add(button)
         button.show()
         self.button['install'] = button
         button = gtk.Button(_('Update'))
-        button.connect('clicked', lambda b, w=self: w.ngm.update_current())
+        button.connect('clicked', lambda b, w=self: w.callback['update_current']())
         box.add(button)
         button.show()
         self.button['update'] = button
@@ -564,55 +569,44 @@ class UI:
                      (_('Version:'), 'Version'),
                      (_('AIName:'), 'AIName')]
         text = ''
-        text = ''.join((text, self.ngm.get('Name'), '\n'))
+        text = ''.join((text, self.callback['get']('Name'), '\n'))
         for item in info_list:
-            text = ''.join((text, item[0], self.ngm.get(item[1]), '\n'))
+            text = ''.join((text, item[0], self.callback['get'](item[1]), '\n'))
         text = ''.join((text,
-                        self.ngm.get('SakuraName'),
-                        _('SurfaceList:'), self.ngm.get('SakuraSurfaceList'),
+                        self.callback['get']('SakuraName'),
+                        _('SurfaceList:'), self.callback['get']('SakuraSurfaceList'),
                         '\n'))
         text = ''.join((text,
-                        self.ngm.get('KeroName'),
-                        _('SurfaceList:'), self.ngm.get('KeroSurfaceList'),
+                        self.callback['get']('KeroName'),
+                        _('SurfaceList:'), self.callback['get']('KeroSurfaceList'),
                         '\n'))
         textbuffer = self.info.get_buffer()
         textbuffer.set_text(text)
-        url = self.ngm.get('HPUrl')
-        text = self.ngm.get('HPTitle')
+        url = self.callback['get']('HPUrl')
+        text = self.callback['get']('HPTitle')
         self.url['HP'][0] = url
         label = self.url['HP'][1]
         label.set_markup('<span foreground="blue">%s</span>' % text)
-        url = self.ngm.get('PublicUrl')
-        text = ''.join((self.ngm.get('Name'), _(' Web Page')))
+        url = self.callback['get']('PublicUrl')
+        text = ''.join((self.callback['get']('Name'), _(' Web Page')))
         self.url['Public'][0] = url
         label = self.url['Public'][1]
-        label.set_markup('<span foreground="blue">%s</span>' % text)
-        
-        target_dir = os.path.join(self.ngm.home_dir, 'ghost',
-                                  self.ngm.get('InstallDir'))
-        if not os.path.isdir(target_dir) and \
-           self.ngm.get('ArchiveUrl') != 'No data':
-            self.button['install'].set_sensitive(True)
-        else:
-            self.button['install'].set_sensitive(False)
-        if os.path.isdir(target_dir) and \
-           self.ngm.get('GhostType') == 'ゴースト' and \
-           self.ngm.get('UpdateUrl') != 'No data':
-            self.button['update'].set_sensitive(True)
-        else:
-            self.button['update'].set_sensitive(False)
+        label.set_markup('<span foreground="blue">%s</span>' % text)        
+        target_dir = os.path.join(self.callback['get_home_dir'](), 'ghost',
+                                  self.callback['get']('InstallDir'))
+        self.button['install'].set_sensitive(
+            bool(not os.path.isdir(target_dir) and
+                 self.callback['get']('ArchiveUrl') != 'No data'))
+        self.button['update'].set_sensitive(
+            bool(os.path.isdir(target_dir) and
+                 self.callback['get']('GhostType') == 'ゴースト' and
+                 self.callback['get']('UpdateUrl') != 'No data'))
 
     def update(self):
         self.update_surface_area()
         self.update_info_area()
-        if self.ngm.exist_next():
-            self.button['next'].set_sensitive(True)
-        else:
-            self.button['next'].set_sensitive(False)
-        if self.ngm.exist_previous():
-            self.button['previous'].set_sensitive(True)
-        else:
-            self.button['previous'].set_sensitive(False)
+        self.button['next'].set_sensitive(bool(self.callback['exist_next']()))
+        self.button['previous'].set_sensitive(bool(self.callback['exist_previous']()))
 
     def show(self):
         if self.opened:
@@ -624,16 +618,35 @@ class UI:
     def close(self):
         self.window.hide()
         self.opened = 0
+        return True
 
 class NGM:
 
-    def __init__(self, app):
+    def __init__(self, callback):
+        self.callback = callback
         self.current = 0
-        self.app = app
         self.opened = 0
         self.home_dir = ninix.home.get_ninix_home()
         self.catalog = Catalog(os.path.join(self.home_dir, 'ngm/data'))
-        self.ui = UI(self)
+        ui_callback = {
+            'open_preference_dialog': self.open_preference_dialog,
+            'open_mask_dialog': self.open_mask_dialog,
+            'reset_to_default': self.reset_to_default,
+            'show_all': self.show_all,
+            'network_update': self.network_update,
+            'search': self.search,
+            'search_forward': self.search_forward,
+            'next': self.next,
+            'previous': self.previous,
+            'get': self.get,
+            'get_image_filename': self.get_image_filename,
+            'get_home_dir': lambda *a: self.home_dir,
+            'exist_next': self.exist_next,
+            'exist_previous': self.exist_previous,
+            'install_current': self.install_current,
+            'update_current': self.update_current,
+            }
+        self.ui = UI(ui_callback)
 
     def get(self, element):
         if self.current in self.catalog.data:
@@ -665,11 +678,11 @@ class NGM:
     def open_preference_dialog(self):
         pass
 
-    def network_update(self):
-        self.catalog.network_update()
+    def network_update(self, updatehook):
+        self.catalog.network_update(updatehook)
         for set_id in self.catalog.data:
             for side in [0, 1]:
-                self.catalog.retrieve_image(set_id, side)
+                self.catalog.retrieve_image(set_id, side, updatehook)
 
     def open_mask_dialog(self):
         pass
@@ -687,10 +700,7 @@ class NGM:
 
     def exist_next(self):
         next = self.current + 1
-        if next < ID_LIMIT and next in self.catalog.data:
-            return True
-        else:
-            return False
+        return bool(next < ID_LIMIT and next in self.catalog.data)
 
     def previous(self):
         previous = self.current - 1
@@ -701,10 +711,7 @@ class NGM:
     def exist_previous(self):
         previous = self.current - 1
         assert previous < ID_LIMIT
-        if previous in self.catalog.data:
-            return True
-        else:
-            return False
+        return bool(previous in self.catalog.data)
 
     def show_dialog(self):
         self.ui.show()
@@ -713,12 +720,7 @@ class NGM:
         pass
 
     def update_current(self): ## FIXME
-        item = self.app.find_ghost_by_name(self.get('Name'), 0)
-        sakura = self.app.select_ghost_from_list(item)
-        if not sakura.is_running():
-            self.app.start_sakura(item, init=1)
-        sakura.enqueue_script('\![updatebymyself]\e', 'NGM',
-                              None, None, 0, 0, None)
+        self.callback['update_sakura'](self.get('Name'), 'NGM')
 
 
 if __name__ == '__main__':
