@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2003-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -98,7 +98,7 @@ def create_pixbuf_from_DDP_file(path):
     loader.close()
     return pixbuf
 
-def create_pixbuf_from_file(path, is_pnr=1, use_pna=0):
+def create_pixbuf_from_file(path, is_pnr=True, use_pna=False):
     head, tail = os.path.split(path)
     basename, suffix = os.path.splitext(tail)
     if suffix == '.dgp':
@@ -129,13 +129,11 @@ def create_pixbuf_from_file(path, is_pnr=1, use_pna=0):
             assert pna_pixbuf.get_bits_per_sample() / 8 == 1
             pixbuf_array = pixbuf.get_pixels_array()
             pixbuf_array[:,:,3] = pna_array[:,:,0]
-            del pna_pixbuf
     return pixbuf
 
 def create_pixmap_from_file(path):
     pixbuf = create_pixbuf_from_file(path)
     pixmap, mask = pixbuf.render_pixmap_and_mask(1)
-    del pixbuf
     return pixmap, mask
 
 def get_workarea():
