@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2004-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003-2005 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -22,35 +22,17 @@ import gobject
 import ninix.home
 
 
-range_scale = [(' 100%',  100),
-               ('  90%',   90),
-               ('  80%',   80),
-               ('  70%',   70),
-               ('  60%',   60),
-               ('  50%',   50),
-               ('  40%',   40),
-               (' 200%',  200),
-               (' 300%',  300),
-               ('1000%', 1000)]
-
-range_script_speed = [(_('None'), -1), # ordinal (-1: no wait)
-                      (''.join(('1 (', _('Fast'), ')')), 0),
-                      ('2', 1),
-                      ('3', 2),
-                      ('4', 3),
-                      ('5', 4),
-                      ('6', 6),
-                      (''.join(('7 (', _('Slow'), ')')), 8)]
+range_scale = range(100, 30, -10) + [200, 300, 1000]
+range_script_speed = range(-1, 5) + [6, 8] # -1: no wait
 
 # default settings
 DEFAULT_BALLOON_FONTS = 'Sans'
 
 def get_default_surface_scale():
-    return range_scale[0][1]
+    return range_scale[0]
 
 def get_default_script_speed():
-    i = len(range_script_speed) / 2
-    return range_script_speed[i][1]
+    return range_script_speed[len(range_script_speed) // 2]
 
 
 class Preferences(dict):
@@ -81,31 +63,17 @@ class Preferences(dict):
             f.write('%s: %s\n' % (key, self[key]))
         f.close()
 
-    def _get(self, name, default, conv):
+    def get_with_type(self, name, conv, default):
         value = self.get(name)
         if value:
+            if conv is None:
+                return value
             try:
                 return conv(value)
             except ValueError:
                 pass
         return default
 
-    def getint(self, name, default=None):
-        return self._get(name, default, int)
-
-    def getfloat(self, name, default=None):
-        return self._get(name, default, float)
-
-
-def new_prefs():
-    filename = ninix.home.get_preferences()
-    return Preferences(filename)
-
-def load_prefs():
-    prefs = new_prefs()
-    prefs.load()
-    return prefs
-
 
 class PreferenceDialog:
 
@@ -128,8 +96,8 @@ class PreferenceDialog:
                   'seriko_inactive': int,
                   }
 
-    def __init__(self, app):
-        self.app = app
+    def __init__(self, callback):
+        self.callback = callback
         self.window = gtk.Dialog()
         self.window.set_title('Preferences')
         self.window.connect('delete_event', self.cancel)
@@ -163,90 +131,95 @@ class PreferenceDialog:
         button.show()
 
     def load(self):
-        self.__prefs = load_prefs()
+        filename = ninix.home.get_preferences()
+        self.__prefs = Preferences(filename)
+        self.save = self.__prefs.save
+        self.__prefs.load()
         self.reset()
+        self.update() # XXX
         self.__saved_prefs = self.__prefs.copy()
-        self.notify() ## FIXME
+        self.callback['notify_preference_changed']()
         
     def reset(self): ### FIXME ###
-        self.set_balloon_fonts(self.get('balloon_fonts', DEFAULT_BALLOON_FONTS))
-        name = self.get('default_balloon')
-        if self.app.find_balloon_by_name(name) is not None:
-            self.set_default_balloon(name)
-        self.set_ignore_default(self.get('ignore_default', 0))
-        self.set_surface_scale(self.get('surface_scale',
-                                        get_default_surface_scale()))
-        self.set_script_speed(self.get('script_speed',
-                                       get_default_script_speed()))
-        self.set_balloon_scalling(self.get('balloon_scalling'))
-        self.set_allowembryo(self.get('allowembryo'))
-        self.set_check_collision(self.get('check_collision', 0))
-        self.set_use_pna(self.get('use_pna'))
-        self.set_sink_after_talk(self.get('sink_after_talk'))
-        self.set_surface_alpha(self.get('surface_alpha'))
-        self.set_balloon_alpha(self.get('balloon_alpha'))
-        self.set_raise_before_talk(self.get('raise_before_talk'))
-        self.set_animation_quality(self.get('animation_quality'))
-        self.set_seriko_inactive(self.get('seriko_inactive'))
+        self.fontsel.set_font_name(
+            self.get('balloon_fonts', DEFAULT_BALLOON_FONTS))
+        self.set_default_balloon(self.get('default_balloon'))
+        self.ignore_button.set_active(
+            bool(self.get('ignore_default', 0)))
+        scale = self.get('surface_scale', get_default_surface_scale())
+        self.surface_scale_combo.set_active(
+            range_scale.index(get_default_surface_scale() if scale not in range_scale else scale))
+        script_speed = self.get('script_speed', get_default_script_speed())
+        self.script_speed_combo.set_active(
+            range_script_speed.index(get_default_script_speed() if script_speed not in range_script_speed else script_speed))
+        self.balloon_scalling_button.set_active(
+            bool(self.get('balloon_scalling')))
+        self.allowembryo_button.set_active(
+            bool(self.get('allowembryo')))
+        self.check_collision_button.set_active(
+            bool(self.get('check_collision', 0)))
+        self.use_pna_button.set_active(
+            bool(self.get('use_pna')))
+        self.sink_after_talk_button.set_active(
+            bool(self.get('sink_after_talk')))
+        self.surface_alpha_adjustment.set_value(self.get('surface_alpha', 1.0))
+        self.balloon_alpha_adjustment.set_value(self.get('balloon_alpha', 1.0))
+        self.raise_before_talk_button.set_active(
+            bool(self.get('raise_before_talk')))
+        self.animation_quality_adjustment.set_value(
+            self.get('animation_quality', 1.0))
+        self.seriko_inactive_button.set_active(
+            bool(self.get('seriko_inactive')))
 
     def get(self, name, default=None):
         assert name in self.PREFS_TYPE
-        if self.PREFS_TYPE[name] == int:
-            return self.__prefs.getint(name, default)
-        elif self.PREFS_TYPE[name] == float:
-            return self.__prefs.getfloat(name, default)
-        else:
-            return self.__prefs.get(name, default)
+        return self.__prefs.get_with_type(name, self.PREFS_TYPE[name], default)
 
-    def save(self): ## FIXME: check key is valid
-        name, surface = self.app.get_current_sakura_name() ## FIXME
-        self.__prefs['sakura_name'] = self.__saved_prefs['sakura_name'] = name
-        self.__prefs['sakura_surface'] = self.__saved_prefs['sakura_surface'] = surface
-        self.__prefs.save()
+    def set_current_sakura(self, name, surface):
+        key = 'sakura_name'
+        self.__prefs[key] = self.__saved_prefs[key] = name
+        key = 'sakura_surface'
+        self.__prefs[key] = self.__saved_prefs[key] = surface
 
     def edit_preferences(self):
         self.show()
 
     def update(self): ## FIXME
-        self.__prefs['allowembryo'] = str(1 if self.allowembryo_button.get_active() else 0)
+        self.__prefs['allowembryo'] = str(int(self.allowembryo_button.get_active()))
         self.__prefs['balloon_fonts'] = self.fontsel.get_font_name()
-        name = None
         selected = self.balloon_treeview.get_selection().get_selected()
         if selected:
             model, listiter = selected
             name = model.get_value(listiter, 0)
-        if self.app.find_balloon_by_name(name) is not None:
             self.__prefs['default_balloon'] = name
-        self.__prefs['ignore_default'] = 1 if self.ignore_button.get_active() else 0
-        label, value = range_scale[self.surface_scale_combo.get_active()]
-        self.__prefs['surface_scale'] = str(int(value))
-        label, value = range_script_speed[self.script_speed_combo.get_active()]
-        self.__prefs['script_speed'] = str(int(value))
-        self.__prefs['balloon_scalling'] = str(1 if self.balloon_scalling_button.get_active() else 0)
-        self.__prefs['check_collision'] = str(1 if self.check_collision_button.get_active() else 0)
-        self.__prefs['use_pna'] = str(1 if self.use_pna_button.get_active() else 0)
-        self.__prefs['sink_after_talk'] = str(1 if self.sink_after_talk_button.get_active() else 0)
-        self.__prefs['raise_before_talk'] = str(1 if self.raise_before_talk_button.get_active() else 0)
+        self.__prefs['ignore_default'] = str(int(self.ignore_button.get_active()))
+        self.__prefs['surface_scale'] = str(int(range_scale[self.surface_scale_combo.get_active()]))
+        self.__prefs['script_speed'] = str(int(range_script_speed[self.script_speed_combo.get_active()]))
+        self.__prefs['balloon_scalling'] = str(int(self.balloon_scalling_button.get_active()))
+        self.__prefs['check_collision'] = str(int(self.check_collision_button.get_active()))
+        self.__prefs['use_pna'] = str(int(self.use_pna_button.get_active()))
+        self.__prefs['sink_after_talk'] = str(int(self.sink_after_talk_button.get_active()))
+        self.__prefs['raise_before_talk'] = str(int(self.raise_before_talk_button.get_active()))
         self.__prefs['surface_alpha'] = str(float(self.surface_alpha_adjustment.get_value()))
         self.__prefs['balloon_alpha'] = str(float(self.balloon_alpha_adjustment.get_value()))
         self.__prefs['animation_quality'] = str(float(self.animation_quality_adjustment.get_value()))
-        self.__prefs['seriko_inactive'] = str(1 if self.seriko_inactive_button.get_active() else 0)
+        self.__prefs['seriko_inactive'] = str(int(self.seriko_inactive_button.get_active()))
 
     def ok(self, widget):
         self.hide()
         self.update()
         self.__saved_prefs.update(self.__prefs)
-        self.notify() ## FIXME
+        self.callback['notify_preference_changed']()
 
     def apply(self, widget):
         self.update()
-        self.notify() ## FIXME
+        self.callback['notify_preference_changed']()
 
     def cancel(self, widget, event=None):
         self.hide()
         self.__prefs.update(self.__saved_prefs)
         self.reset()
-        self.notify() ## FIXME
+        self.callback['notify_preference_changed']()
         return True
 
     def show(self):
@@ -288,8 +261,8 @@ class PreferenceDialog:
         hbox.pack_start(label, False)
         label.show()
         self.surface_scale_combo = gtk.combo_box_new_text()
-        for label, value in range_scale:
-            self.surface_scale_combo.append_text(label)
+        for value in range_scale:
+            self.surface_scale_combo.append_text('%4d%%' % value)
         hbox.pack_start(self.surface_scale_combo, False)
         self.surface_scale_combo.show()
         button = gtk.CheckButton(unicode(_('Scale Balloon'), 'utf-8'))
@@ -310,7 +283,7 @@ class PreferenceDialog:
         box.pack_start(scrolled, True)
         scrolled.show()
         model = gtk.ListStore(gobject.TYPE_STRING)
-        for desc, balloon in self.app.balloons:
+        for desc, balloon in self.callback['get_balloons']():
             name = desc.get('name', '')
             listiter = model.append()
             model.set_value(listiter, 0, name)
@@ -406,7 +379,15 @@ class PreferenceDialog:
         hbox.pack_start(label, False)
         label.show()
         self.script_speed_combo = gtk.combo_box_new_text()
-        for label, value in range_script_speed:
+        for index in range(len(range_script_speed)):
+            if index == 0:
+                label = _('None')
+            elif index == 1:
+                label = ''.join(('1 (', _('Fast'), ')'))
+            elif index == len(range_script_speed) - 1:
+                label = ''.join((str(index), ' (', _('Slow'), ')'))
+            else:
+                label = str(index)
             self.script_speed_combo.append_text(label)
         hbox.pack_start(self.script_speed_combo, False)
         self.script_speed_combo.show()
@@ -442,46 +423,6 @@ class PreferenceDialog:
         button.show()
         return page
 
-    def notify(self): ## FIXME
-        for sakura in self.app.get_working_ghost():
-            sakura.reset_balloon_fonts() ## FIXME
-            flag = self.get('balloon_scalling')
-            sakura.set_balloon_scalling(flag) ## FIXME
-            scale = self.get('surface_scale')
-            sakura.set_surface_scale(scale) ## FIXME
-            flag = self.get('use_pna')
-            sakura.set_use_pna(flag) ## FIXME
-            alpha = self.get('surface_alpha')
-            sakura.set_surface_alpha(alpha) ## FIXME
-            alpha = self.get('balloon_alpha')
-            sakura.set_balloon_alpha(alpha) ## FIXME
-            quality = self.get('animation_quality')
-            sakura.set_animation_quality(quality) ## FIXME
-            flag = self.get('seriko_inactive')
-            sakura.set_seriko_inactive(flag) ## FIXME
-
-    def set_balloon_fonts(self, name):
-        self.fontsel.set_font_name(name)
-
-    def set_balloon_scalling(self, flag):
-        self.balloon_scalling_button.set_active(True if flag else False)
-
-    def set_script_speed(self, speed):
-        index = 0
-        for label, value in range_script_speed:
-            if speed == value:
-                self.script_speed_combo.set_active(index)
-                break
-            index += 1
-
-    def set_surface_scale(self, scale):
-        index = 0
-        for label, value in range_scale:
-            if scale == value:
-                self.surface_scale_combo.set_active(index)
-                break
-            index += 1
-
     def set_default_balloon(self, name):
         model = self.balloon_treeview.get_model()
         listiter = model.get_iter_first()
@@ -495,33 +436,3 @@ class PreferenceDialog:
             listiter = model.get_iter_first()
             assert listiter is not None
             self.balloon_treeview.get_selection().select_iter(listiter)
-
-    def set_ignore_default(self, flag):
-        self.ignore_button.set_active(True if flag else False)
-
-    def set_check_collision(self, flag):
-        self.check_collision_button.set_active(True if flag else False)
-
-    def set_allowembryo(self, flag):
-        self.allowembryo_button.set_active(True if flag else False)
-
-    def set_use_pna(self, flag):
-        self.use_pna_button.set_active(True if flag else False)
-
-    def set_sink_after_talk(self, flag):
-        self.sink_after_talk_button.set_active(True if flag else False)
-
-    def set_raise_before_talk(self, flag):
-        self.raise_before_talk_button.set_active(True if flag else False)
-
-    def set_surface_alpha(self, alpha):
-        self.surface_alpha_adjustment.set_value(1.0 if alpha is None else alpha)
-
-    def set_balloon_alpha(self, alpha):
-        self.balloon_alpha_adjustment.set_value(1.0 if alpha is None else alpha)
-
-    def set_animation_quality(self, quality):
-        self.animation_quality_adjustment.set_value(1.0 if quality is None else quality)
-
-    def set_seriko_inactive(self, flag):
-        self.seriko_inactive_button.set_active(True if flag else False)
