@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -28,9 +28,14 @@ if 'DISPLAY' in os.environ:
     import gtk ## FIXME
     import gobject
 
+# For some reasons gst parses sys.argv on import. This is a workaround.
+# (See https://bugzilla.gnome.org/show_bug.cgi?id=549879 .)
+argv = sys.argv
+sys.argv = []
 import pygst
 pygst.require("0.10")
 import gst
+sys.argv = argv
 
 import ninix.surface
 import ninix.balloon
@@ -61,12 +66,9 @@ class Sakura:
     except:
         name2codepoint = None
 
-    def __init__(self, app, data, default_path, communicate, debug=0):
-        self.app = app
-        self.prefs = app.prefs
-        self.char = 2 # 'sakura' and 'kero'
+    def __init__(self, data, default_path, callback, debug=0):
+        self.callback = callback
         self.debug = debug
-        self.communicate = communicate
         self.sstp_handle = None
         self.sstp_entry_db = None
         self.sstp_request_handler = None
@@ -74,6 +76,7 @@ class Sakura:
             self.script_parser = ninix.script.Parser(error='strict')
         else:
             self.script_parser = ninix.script.Parser(error='loose')
+        self.char = 2 # 'sakura' and 'kero'
         self.script_queue = []
         self.script_mode = self.BROWSE_MODE
         self.script_post_proc = []
@@ -93,7 +96,11 @@ class Sakura:
         ##
         self.old_otherghostname = None ## FIXME
         # create vanish dialog
-        self.__vanish_dialog = VanishDialog(self)
+        vanish_dialog_callback = {
+            'notify_vanish_confirmation': self.notify_vanish_confirmation,
+            'notify_event': self.notify_event,
+            }
+        self.__vanish_dialog = VanishDialog(vanish_dialog_callback)
         self.cantalk = 1
         self.__sender = 'ninix-aya'
         self.__charset = 'Shift_JIS'
@@ -103,24 +110,76 @@ class Sakura:
             'shiori', default_path, saori_lib=saori_lib)
         self.__temp_mode = 0
         self.__observer = []
-        self.balloon = ninix.balloon.Balloon(self, self.prefs, debug)
-        self.surface = ninix.surface.Surface(self, self.prefs, debug)
+        balloon_callback = {
+            'is_paused': self.is_paused,
+            'notify_balloon_click': self.notify_balloon_click,
+            'notify_event': self.notify_event,
+            'notify_link_selection': self.notify_link_selection,
+            'notify_user_teach': self.notify_user_teach,
+            'position_balloons': self.position_balloons,
+            'reset_idle_time': self.reset_idle_time,
+            }
+        for key in ['get_preference']:
+            balloon_callback[key] = self.callback[key]
+        self.balloon = ninix.balloon.Balloon(balloon_callback, debug)
+        surface_callback = {
+            'about': self.about,
+            'busy': self.busy,
+            'close_sakura': self.close,
+            'enqueue_event': self.enqueue_event,
+            'getstring': self.getstring,
+            'get_balloon_size': self.get_balloon_size,
+            'get_current_balloon_name': self.get_current_balloon_name,
+            'get_keroname': self.get_keroname,
+            'get_own_balloon_name': self.get_own_balloon_name,
+            'get_prefix': self.get_prefix,
+            'get_selfname': self.get_selfname,
+            'is_running': self.is_running,
+            'network_update': self.network_update,
+            'notify_deiconified': self.notify_deiconified,
+            'notify_event': self.notify_event,
+            'notify_iconified': self.notify_iconified,
+            'notify_observer': self.notify_observer,
+            'notify_site_selection': self.notify_site_selection,
+            'notify_surface_click': self.notify_surface_click,
+            'notify_surface_mouse_motion': self.notify_surface_mouse_motion,
+            'reset_idle_time': self.reset_idle_time,
+            'select_balloon': self.select_balloon,
+            'select_kinoko': self.select_kinoko,
+            'select_nekodorif': self.select_nekodorif,
+            'select_sakura': self.select_sakura,
+            'set_balloon_direction': self.set_balloon_direction,
+            'set_balloon_position': self.set_balloon_position,
+            'vanish_sakura': self.vanish,
+            'get_current_item': self.get_current_item,
+            }
+        for key in ['close_all', 'get_preference', 'edit_preferences',
+                    'open_ghost_manager', 'show_usage', 'start_sakura',
+                    'get_ghost_list', 'get_shell_list', 'get_balloon_list',
+                    'get_nekodorif_list', 'get_kinoko_list',
+                    'get_plugin_list', 'select_plugin']:
+            surface_callback[key] = self.callback[key]
+        self.surface = ninix.surface.Surface(surface_callback, debug)
         self.keep_silence(False)
         desc, shiori_dir, use_makoto, surface_set, balloon, prefix, \
             shiori_dll, shiori_name = data
         self.new(desc, shiori_dir, use_makoto, surface_set, balloon, prefix,
                  shiori_dll, shiori_name)
 
-    def get_callback(self, name): ## FIXME
-        callback = {'close all':        self.app.close_all_ghosts,
-                    'select sakura':    self.app.select_sakura,
-                    'start':            self.app.start_sakura_cb,
-                    'select plugin':    self.app.select_plugin,
-                    'select nekodorif': self.app.select_nekodorif,
-                    'select kinoko':    self.app.select_kinoko,
-                    }
-        if name in callback:
-            return callback[name]
+    def get_current_item(self):
+        return self.current
+
+    def select_sakura(self, event, item):
+        if self.busy():
+            gtk.gdk.beep()
+            return
+        self.callback['change_sakura'](self, item, 'manual')
+
+    def select_nekodorif(self, event, item):
+        self.callback['select_nekodorif'](item, self)
+
+    def select_kinoko(self, event, item):
+        self.callback['select_kinoko'](item, self)
 
     def set_observer(self, observer):
         if observer not in self.__observer:
@@ -149,7 +208,11 @@ class Sakura:
         self.shiori_name = shiori_name
         name = (shiori_dll, shiori_name)
         self.shiori = self.__dll.request(name)
-        self.updateman = ninix.update.NetworkUpdate(self)
+        update_callback = {
+            'enqueue_event': self.enqueue_event,
+            'check_event_queue': self.check_event_queue,
+            }
+        self.updateman = ninix.update.NetworkUpdate(update_callback)
         self.audio_player = gst.element_factory_make("playbin", "player")
         fakesink = gst.element_factory_make("fakesink", "fakesink")
         self.audio_player.set_property("video-sink", fakesink)
@@ -281,8 +344,10 @@ class Sakura:
                                   show_sstp_marker, use_translator,
                                   db, request_handler))
 
-    reset_event = ['OnGhostChanging', 'OnShellChanging', 'OnVanishSelected',
-                   'OnNinixReloading'] ## FIXME
+    reset_event = ['OnGhostChanging', 'OnShellChanging', 'OnVanishSelected']
+
+    def check_event_queue(self):
+        return bool(self.event_queue)
 
     def enqueue_event(self, event, *arglist, **argdict): ## FIXME
         for key in argdict:
@@ -327,17 +392,14 @@ class Sakura:
         return self.script_mode == self.PAUSE_MODE
 
     def is_talking(self):
-        return 1 if self.processed_script or self.processed_text else 0
+        return int(self.processed_script or self.processed_text)
 
     def busy(self, check_updateman=True):
-        if self.time_critical_session or \
-                self.balloon.user_interaction or \
-                self.event_queue or \
-                self.passivemode or \
-                (check_updateman and self.updateman.is_active()):
-            return True
-        else:
-            return False
+        return bool(self.time_critical_session or \
+                        self.balloon.user_interaction or \
+                        self.event_queue or \
+                        self.passivemode or \
+                        (check_updateman and self.updateman.is_active()))
 
     def get_silent_time(self):
         return self.silent_time
@@ -357,6 +419,23 @@ class Sakura:
     def reset_idle_time(self):
         self.idle_start = time.time()
 
+    def notify_preference_changed(self): ## FIXME
+        self.balloon.reset_fonts()
+        flag = self.callback['get_preference']('use_pna')
+        self.set_use_pna(flag)
+        scale = self.callback['get_preference']('surface_scale')
+        self.set_surface_scale(scale)
+        flag = self.callback['get_preference']('balloon_scalling')
+        self.set_balloon_scalling(flag)
+        alpha = self.callback['get_preference']('surface_alpha')
+        self.set_surface_alpha(alpha)
+        alpha = self.callback['get_preference']('balloon_alpha')
+        self.set_balloon_alpha(alpha)
+        quality = self.callback['get_preference']('animation_quality')
+        self.set_animation_quality(quality)
+        flag = self.callback['get_preference']('seriko_inactive')
+        self.set_seriko_inactive(flag)
+
     def set_use_pna(self, flag): ## FIXME
         self.surface.set_use_pna(flag)
         self.balloon.set_use_pna(flag)
@@ -384,8 +463,8 @@ class Sakura:
         result = self.surface.get_position(side)
         return result if result is not None else (0, 0)
 
-    def reset_balloon_fonts(self): ## FIXME: remove
-        self.balloon.reset_fonts()
+#    def reset_balloon_fonts(self): ## FIXME: remove
+#        self.balloon.reset_fonts()
 
     def set_balloon_position(self, side, x, y):
         self.balloon.set_position(side, x, y)
@@ -402,10 +481,10 @@ class Sakura:
         return result if result is not None else (0, 0)
 
     def balloon_is_shown(self, side):
-        return 1 if self.balloon and self.balloon.is_shown(side) else 0
+        return int(self.balloon and self.balloon.is_shown(side))
 
     def surface_is_shown(self, side):
-        return 1 if self.surface and self.surface.is_shown(side) else 0
+        return int(self.surface and self.surface.is_shown(side))
 
     def is_URL(self, s):
         return s.startswith('http://') or \
@@ -413,13 +492,7 @@ class Sakura:
                s.startswith('file:/')
 
     def is_anchor(self, link_id):
-        return 1 if len(link_id) == 2 and link_id[0] == 'anchor' else 0
-
-    def execute_command(self, command, arg): ## FIXME
-        if '%s' not in command:
-            sys.stderr.write('cannot execute command (%s missing)\n')
-            return
-        os.system(' '.join((command.replace('%s', arg, 1), '&')))
+        return int(len(link_id) == 2 and link_id[0] == 'anchor')
 
     def vanish(self):
         if self.busy():
@@ -431,7 +504,7 @@ class Sakura:
     def vanish_by_myself(self):
         self.vanished_count += 1
         self.ghost_time = 0
-        self.app.stop_sakura(self.app.vanish_sakura)
+        self.callback['stop_sakura'](self.callback['vanish_sakura'])
 
     def get_name(self):
         return self.desc.get('name', unicode(_('Sakura&Unyuu'), 'utf-8'))
@@ -535,7 +608,7 @@ class Sakura:
             result = ''
         if to and result:
             def proc():
-                self.communicate.send_message(
+                self.callback['send_message'](
                     to, self.get_selfname(), result)
             communication = proc
         else:
@@ -554,28 +627,34 @@ class Sakura:
             default = ninix.version.VERSION_INFO
         if init:
             if self.ghost_time == 0:
-                if not self.notify_event('OnFirstBoot', self.vanished_count):
-                    self.notify_event('OnBoot', default=default)
+                if not self.notify_event('OnFirstBoot', self.vanished_count,
+                                         None, None, None, None, None, None,
+                                         self.surface.name):
+                    self.notify_event('OnBoot', self.surface.name, default=default)
             else:
-                self.notify_event('OnBoot', default=default)
+                self.notify_event('OnBoot', self.surface.name, default=default)
             left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
             self.notify_event('OnDisplayChange',
                               gtk.gdk.visual_get_best_depth(),
                               scrn_w, scrn_h, event_type='NOTIFY')
         elif vanished:
             if self.ghost_time == 0:
-                if self.notify_event('OnFirstBoot', self.vanished_count):
+                if self.notify_event('OnFirstBoot', self.vanished_count,
+                                     None, None, None, None, None, None,
+                                     self.surface.name):
                     return
             elif self.notify_event('OnVanished', name):
                 return
-            self.notify_event('OnBoot', default=default)
+            self.notify_event('OnBoot', self.surface.name, default=default)
         elif ghost_changed:
             if self.ghost_time == 0:
-                if self.notify_event('OnFirstBoot', self.vanished_count):
+                if self.notify_event('OnFirstBoot', self.vanished_count,
+                                     None, None, None, None, None, None,
+                                     self.surface.name):
                     return
             elif self.notify_event('OnGhostChanged', name):
                 return
-            self.notify_event('OnBoot', default=default)
+            self.notify_event('OnBoot', self.surface.name, default=default)
         else:
             self.notify_event('OnShellChanged', name, name, path)
 
@@ -583,13 +662,13 @@ class Sakura:
         def proc(self=self):
             self.vanished_count += 1
             self.ghost_time = 0
-            gobject.idle_add(self.app.vanish_sakura, self) ## FIXME
+            gobject.idle_add(self.callback['vanish_sakura'], self)
         self.enqueue_event('OnVanishSelected', proc=proc)
         self.vanished = 1 ## FIXME
 
     def notify_iconified(self):
         self.cantalk = 0
-        self.app.select_current_sakura()
+        self.callback['select_current_sakura']()
         if not self.passivemode:
             self.reset_script(1)
             self.stand_by(1)
@@ -598,7 +677,7 @@ class Sakura:
     def notify_deiconified(self):
         if not self.cantalk:
             self.cantalk = 1
-            self.app.select_current_sakura()
+            self.callback['select_current_sakura']()
             if not self.passivemode:
                 self.notify_event('OnWindowStateRestore')
 
@@ -707,7 +786,8 @@ class Sakura:
 
     month_names = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
                    'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
-    boot_event = ['OnBoot', 'OnFirstBoot', 'OnGhostChanged', 'OnShellChanged']
+    boot_event = ['OnBoot', 'OnFirstBoot', 'OnGhostChanged', 'OnShellChanged',
+                  'OnUpdateComplete']
     reset_event = ['OnVanishSelecting', 'OnVanishCancel'] ## FIXME
 
     def notify_event(self, event, *arglist, **argdict):
@@ -771,7 +851,9 @@ class Sakura:
             return
         name = self.get_own_balloon_name()
         if item != name:
-            desc, balloon = self.app.get_balloon_description(item)
+            desc, balloon = self.callback['get_balloon_description'](item)
+        else:
+            desc, balloon = self.own_balloon
         self.balloon.hide_all()
         self.set_balloon(desc, balloon)
         self.balloon.set_balloon_default()
@@ -810,19 +892,18 @@ class Sakura:
         self.position_balloons()
 
     def identify_window(self, win):
-        if self.surface.identify_window(win) or \
-           self.balloon.identify_window(win):
-            return True
-        else:
-            return False
+        return bool(self.surface.identify_window(win) or \
+                        self.balloon.identify_window(win))
 
     def set_surface_default(self, side=None):
         self.surface.set_surface_default(side)
 
+    def get_surface_scale(self):
+        return self.surface.get_scale()
+
     def set_surface_scale(self, scale):
         self.surface.set_scale(scale)
         self.balloon.set_scale(scale)
-        self.notify_observer('set scale')
 
     def get_surface_size(self, side):
         result = self.surface.get_surface_size(side)
@@ -838,10 +919,7 @@ class Sakura:
         return self.surface.get_surface(side)
 
     def surface_is_shown(self, side):
-        if self.surface and self.surface.is_shown(side):
-            return True
-        else:
-            return False
+        return bool(self.surface and self.surface.is_shown(side))
 
     def get_kinoko_position(self, baseposition):
         side = 0
@@ -932,7 +1010,7 @@ class Sakura:
         if set_type == 'g':
             print 'ghost', item
             if not self.surface_set:
-                shell_name, surface_set, balloon = self.app.request_shell(0)
+                shell_name, surface_set, balloon = self.callback['request_shell'](0)
                 name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
                       surface_set[0]
                 surface_set = None
@@ -943,7 +1021,7 @@ class Sakura:
                 balloon = self.own_balloon
         elif set_type == 's':
             print 'shell', item
-            shell_name, surface_set, balloon = self.app.request_shell(i)
+            shell_name, surface_set, balloon = self.callback['request_shell'](i)
             name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
                   surface_set[j]
         if not surface_set:
@@ -956,15 +1034,15 @@ class Sakura:
             name = prev_name
         self.set_surface(surface_desc, surface_alias, surface, surface_name,
                          surface_dir, surface_tooltips)
-        if self.prefs.get('ignore_default') or not balloon:
+        if self.callback['get_preference']('ignore_default') or not balloon:
             balloon_name = self.desc.get('balloon', '')
             number = None
             if balloon_name:
-                number = self.app.find_balloon_by_name(balloon_name)
+                number = self.callback['find_balloon_by_name'](balloon_name)
             if number is None:
-                default_balloon = self.prefs.get('default_balloon')
-                number = self.app.find_balloon_by_name(default_balloon) or 0
-            balloon = self.app.request_balloon(number)
+                default_balloon = self.callback['get_preference']('default_balloon')
+                number = self.callback['find_balloon_by_name'](default_balloon) or 0
+            balloon = self.callback['request_balloon'](number)
         desc, balloon = balloon
         self.set_balloon(desc, balloon)
         if not temp:
@@ -977,6 +1055,7 @@ class Sakura:
     def restart(self):
         self.load_history()
         self.vanished = 0
+        self.__boot = [0, 0]
         self.old_otherghostname = None ## FIXME
         self.reset_script(1)
         self.surface.reset_alignment()
@@ -991,7 +1070,7 @@ class Sakura:
         self.notify_observer('finalize')
         self.__running = 0
         self.save_history()
-        self.communicate.rebuild_ghostdb(self, None)
+        self.callback['rebuild_ghostdb'](self, None)
         self.hide_all()
         self.surface.finalize()
         self.balloon.finalize()
@@ -1004,13 +1083,13 @@ class Sakura:
         if self.clock[0] != second: ## FIXME
             if not self.__temp_mode:
                 self.ghost_time += 1
-            self.communicate.rebuild_ghostdb(
+            self.callback['rebuild_ghostdb'](
                 self,
                 self.get_selfname(),
                 self.get_surface_id(0),
-                self.get_surface_id(1)) ## FIXME
-            otherghostname = self.communicate.get_otherghostname(
-                self.get_selfname()) ## FIXME
+                self.get_surface_id(1))
+            otherghostname = self.callback['get_otherghostname'](
+                self.get_selfname())
             if otherghostname != self.old_otherghostname:
                 args = []
                 args.extend(otherghostname)
@@ -1050,7 +1129,7 @@ class Sakura:
             self.__balloon_life = 0
             self.stand_by(0)
             self.notify_event('OnBalloonClose', self.__current_script)
-            if self.prefs.get('sink_after_talk'):
+            if self.callback['get_preference']('sink_after_talk'):
                 self.surface.lower_all()
         elif self.event_queue and self.handle_event():
             pass
@@ -1106,11 +1185,11 @@ class Sakura:
                 if self.__temp_mode == 1:
                     time.sleep(1.4)
                     self.finalize()
-                    self.app.close_ghost(self)
-                    self.app.reset_sstp_flag()
+                    self.callback['close_ghost'](self)
+                    self.callback['reset_sstp_flag']()
                     return False
                 else:
-                    self.app.reset_sstp_flag()
+                    self.callback['reset_sstp_flag']()
                     self.leave_temp_mode()
                     return True
             else:
@@ -1120,7 +1199,7 @@ class Sakura:
             self.hide_all()
             sys.stdout.write('reloading....\n')
             self.shiori.unload()
-            self.app.stop_sakura(self, self.app.reload_current_sakura, self)
+            self.callback['stop_sakura'](self, self.callback['reload_current_sakura'], self)
             self.restart() ## FIXME
             sys.stdout.write('done.\n')
             self.enqueue_event(*self.reload_event)
@@ -1140,7 +1219,7 @@ class Sakura:
         return True
 
     def quit(self):
-        self.app.stop_sakura(self)
+        self.callback['stop_sakura'](self)
 
     ###   SCRIPT PLAYER   ###
     def start_script(self, script, origin=None):
@@ -1173,7 +1252,7 @@ class Sakura:
         self.balloon.set_balloon_default()
         self.current_time = time.localtime(time.time())
         self.reset_idle_time()
-        if self.prefs.get('raise_before_talk'):
+        if self.callback['get_preference']('raise_before_talk'):
             self.raise_all()
 
     def __yen_e(self, args):
@@ -1313,12 +1392,12 @@ class Sakura:
             self.script_wait = time.time() + amount
 
     def __yen_w(self, args):
-        script_speed = self.prefs.get('script_speed')
+        script_speed = self.callback['get_preference']('script_speed')
         if not self.quick_session and script_speed >= 0:
             self.__set_weight(args[0], 0.05) # 50ms
 
     def __yen__w(self, args):
-        script_speed = self.prefs.get('script_speed')
+        script_speed = self.callback['get_preference']('script_speed')
         if not self.quick_session and script_speed >= 0:
             self.__set_weight(args[0], 0.001) # 1ms
 
@@ -1399,10 +1478,10 @@ class Sakura:
         self.quit()
 
     def __yen_plus(self, args):
-        self.app.select_ghost(self, 1)
+        self.callback['select_ghost'](self, 1)
 
     def __yen__plus(self, args):
-        self.app.select_ghost(self, 0)
+        self.callback['select_ghost'](self, 0)
 
     def __yen_m(self, args):
         self.write_sstp_handle(self.expand_meta(args[0]))
@@ -1467,14 +1546,14 @@ class Sakura:
                 else:
                     self.balloon.open_inputbox(args[2])
         elif args[0:2] == ['open', 'configurationdialog']:
-            self.prefs.edit_preferences()
+            self.callback['edit_preferences']()
         elif args[0:2] == ['change', 'shell'] and argc > 2:
-            self.app.select_shell_by_name(self, args[2], 0)
+            self.callback['select_shell_by_name'](self, args[2], 0)
         elif args[0:2] == ['change', 'ghost'] and argc > 2:
             if args[2] == 'random':
-                self.app.select_ghost(self, 0, 0)
+                self.callback['select_ghost'](self, 0, 0)
             else:
-                self.app.select_ghost_by_name(self, args[2], 0)
+                self.callback['select_ghost_by_name'](self, args[2], 0)
         elif args[0:1] == ['updatebymyself']:
             if not self.busy(check_updateman=False):
                 self.__update()
@@ -1535,32 +1614,21 @@ class Sakura:
         self.raise_surface(self.script_side)
 
     def __yen_f(self, args):
+        if len(args) != 2: ## FIXME
+            return
+        tag = None
         if args[0] == 'sup':
-            assert len(args) == 2
-            if args[1] == 'true':
-                self.balloon.append_meta(self.script_side, '<sup>')
-            else:
-                self.balloon.append_meta(self.script_side, '</sup>')
+            tag = '<sup>' if args[1] == 'true' else '</sup>'
         elif args[0] == 'sub':
-            assert len(args) == 2
-            if args[1] == 'true':
-                self.balloon.append_meta(self.script_side, '<sub>')
-            else:
-                self.balloon.append_meta(self.script_side, '</sub>')
+            tag = '<sub>' if args[1] == 'true' else '</sub>'
         elif args[0] == 'strike':
-            assert len(args) == 2
-            if args[1] in ['true', '1', 1]:
-                self.balloon.append_meta(self.script_side, '<s>')
-            else:
-                self.balloon.append_meta(self.script_side, '</s>')
+            tag = '<s>' if args[1] in ['true', '1', 1] else '</s>'
         elif args[0] == 'underline':
-            assert len(args) == 2
-            if args[1] in ['true', '1', 1]:
-                self.balloon.append_meta(self.script_side, '<u>')
-            else:
-                self.balloon.append_meta(self.script_side, '</u>')
+            tag = '<u>' if args[1] in ['true', '1', 1] else '</u>'
         else:
             pass ## FIXME
+        if tag is not None:
+            self.balloon.append_meta(self.script_side, tag)
 
     __script_tag = {
         r'\e': __yen_e,
@@ -1617,7 +1685,7 @@ class Sakura:
             count = self.balloon.get_text_count(self.script_side)
             if self.surface.invoke_talk(self.script_side, surface_id, count):
                 self.balloon.reset_text_count(self.script_side)
-            script_speed = self.prefs.get('script_speed')
+            script_speed = self.callback['get_preference']('script_speed')
             if script_speed > 0:
                 self.script_wait = time.time() + script_speed * 0.02
             return
@@ -1632,7 +1700,7 @@ class Sakura:
             text = self.expand_meta(node[1])
             if self.anchor:
                 self.anchor[1] = ''.join((self.anchor[1], text))
-            script_speed = self.prefs.get('script_speed')
+            script_speed = self.callback['get_preference']('script_speed')
             if not self.quick_session and script_speed >= 0:
                 self.processed_text = text
             else:
@@ -1695,7 +1763,7 @@ class Sakura:
                 buf.append(str(scrn_h))
             elif chunk[1] == '%et':
                 buf.append(
-                    unicode('%d万年' % self.current_time[7], 'UTF-8'))
+                    unicode('%d万年' % self.current_time[7], 'utf-8'))
             elif chunk[1] == '%exh':
                 buf.append(str(self.get_uptime()))
             elif chunk[1] in ['%ms', '%mz', '%ml', '%mc', '%mh', \
@@ -1748,9 +1816,6 @@ class Sakura:
             return
         self.start_script(ninix.version.VERSION_INFO)
 
-    def reload(self):
-        self.app.reload(self)
-
     def __update(self):
         if self.updateman.is_active():
             return
@@ -1776,8 +1841,8 @@ class Sakura:
 
 class VanishDialog:
 
-    def __init__(self, sakura):
-        self.__sakura = sakura
+    def __init__(self, callback):
+        self.callback = callback
         self.window = gtk.Dialog()
         self.window.connect('delete_event', self.cancel)
         self.window.set_title('Vanish')
@@ -1807,12 +1872,12 @@ class VanishDialog:
 
     def ok(self, widget, event=None):
         self.window.hide()
-        self.__sakura.notify_vanish_confirmation()
+        self.callback['notify_vanish_confirmation']()
         return True
 
     def cancel(self, widget, event=None):
         self.window.hide()
-        self.__sakura.notify_event('OnVanishCancel')
+        self.callback['notify_event']('OnVanishCancel')
         return True
 
 
