@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -16,7 +16,6 @@ import re
 import sys
 import random
 
-import gtk
 import gobject
 
 def print_error(message):
@@ -41,10 +40,7 @@ class Controler:
         self.__inactive = False
 
     def is_active(self):
-        if self.__inactive:
-            return 0
-        else:
-            return 1
+        return not bool(self.__inactive)
 
     def set_base_id(self, window, surface_id):
         if surface_id == -2:
@@ -71,7 +67,7 @@ class Controler:
             self.timeout_id = gobject.timeout_add(int(step * 1000.0), self.update, window)
 
     def set_inactive(self, flag, window):
-        self.__inactive = True if flag else False
+        self.__inactive = bool(flag)
         self.reset(window, window.surface_id)
 
     def update_frame(self, window):
@@ -224,10 +220,8 @@ class Controler:
         self.terminate(window)
         self.frame['next'] = 0
         self.set_base_id(window, window.surface_id)
-        if surface_id in self.seriko:
-            self.base_id = surface_id
-        else:
-            self.base_id = window.surface_id
+        self.base_id = surface_id if surface_id in self.seriko else \
+            window.surface_id
         if self.base_id in self.seriko:
             surface_ids = []
             for actor in self.seriko[self.base_id]:
@@ -241,7 +235,8 @@ class Controler:
     def start(self, window):
         self.invoke_runonce(window)
         self.invoke_always(window)
-        self.timeout_id = gobject.timeout_add(int(self.frame['step'] * 1000), self.update, window)
+        self.timeout_id = gobject.timeout_add(int(self.frame['step'] * 1000),
+                                              self.update, window)
 
     def terminate(self, window):
         self.terminate_flag = 1
@@ -301,7 +296,7 @@ class Actor:
     def add_pattern(self, surface, interval, method, args):
         self.patterns.append((surface, interval, method, args))
 
-    def invoke(self, base_frame):
+    def invoke(self, window, base_frame):
         self.terminate_flag = 0
 
     def update(self, window, base_frame):
@@ -680,14 +675,14 @@ def test():
         print 'Usage:', sys.argv[0], '[surface??a.txt ...]'
     for filename in sys.argv[1:]:
         print 'Reading', filename, '...'
-        for actor in get_actors(ninix.config.open(filename)):
+        for actor in get_actors(ninix.config.create_from_file(filename)):
             print '#%d' % actor.get_id(),
             print actor.__class__.__name__,
             print '(%s)' % actor.get_interval()
             print 'number of patterns =', len(actor.get_patterns())
             for pattern in actor.get_patterns():
                 print 'surface=%s, interval=%d, method=%s, args=%s' % pattern
-        for actor in get_mayuna(ninix.config.open(filename)):
+        for actor in get_mayuna(ninix.config.create_from_file(filename)):
             print '#%d' % actor.get_id(),
             print actor.__class__.__name__,
             print '(%s)' % actor.get_interval()
