@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -15,6 +15,7 @@
 
 import os
 import re
+import urllib
 import urlparse
 import random
 
@@ -36,9 +37,7 @@ class Surface:
         keymap_old = {}
         keymap_new = {}
 
-    def __init__(self, sakura, prefs, debug=0):
-        self.__sakura = sakura
-        self.prefs = prefs
+    def __init__(self, callback, debug=0):
         self.debug = debug
         self.window = []
         self.__menu = None
@@ -46,6 +45,7 @@ class Surface:
         self.desc = None
         self.__use_pna = False
         self.__alpha_channel = None
+        self.callback = callback
 
     def set_debug(self, debug):
         self.debug = debug
@@ -54,10 +54,7 @@ class Surface:
             surface_window.reset_surface()
 
     def set_use_pna(self, flag):
-        if flag:
-            self.__use_pna = True
-        else:
-            self.__use_pna = False
+        self.__use_pna = bool(flag)
         for surface_window in self.window:
             surface_window.set_use_pna(flag)
             surface_window.reset_surface()
@@ -79,6 +76,7 @@ class Surface:
         for surface_window in self.window:
             surface_window.set_scale(scale)
             surface_window.reset_surface()
+        self.callback['notify_observer']('set scale')
 
     def set_animation_quality(self, quality):
         for surface_window in self.window:
@@ -131,12 +129,12 @@ class Surface:
                 gtk_window.deiconify()
 
     def window_state(self, window, event):
-        if not self.__sakura.is_running():
+        if not self.callback['is_running']():
             return
         if event.new_window_state & gtk.gdk.WINDOW_STATE_ICONIFIED:
             if window == self.window[0].window:
-                self.__sakura.notify_iconified()
-                self.__sakura.notify_observer('iconified')
+                self.callback['notify_iconified']()
+                self.callback['notify_observer']('iconified')
             for surface_window in self.window:
                 gtk_window = surface_window.window
                 if gtk_window != window and \
@@ -151,18 +149,17 @@ class Surface:
                    gtk.gdk.WINDOW_STATE_ICONIFIED:
                     gtk_window.deiconify()
             if window == self.window[0].window:
-                self.__sakura.notify_deiconified()
+                self.callback['notify_deiconified']()
         return
 
     def delete(self, window, event):
-        self.__sakura.quit()
-        return False
+        return True
 
     def key_press(self, window, event):
         name = self.keymap_old.get(event.keyval, event.string)
         keycode = self.keymap_new.get(event.keyval, event.string)
         if name or keycode:
-            self.__sakura.notify_event('OnKeyPress', name, keycode)
+            self.callback['notify_event']('OnKeyPress', name, keycode)
         return True
 
     def window_stick(self, action):
@@ -314,37 +311,32 @@ class Surface:
         for surface_window in self.window:
             surface_window.destroy()
         self.window = []
-        for name, side, default, alias in [('sakura', 0, '0', alias0),
-                                           ('kero', 1, '10', alias1)]:
-            if name == 'sakura':
-                title = self.__sakura.get_selfname() or \
-                        ''.join(('surface.', name)) ## FIXME
-            else:
-                title = self.__sakura.get_keroname() or \
-                        ''.join(('surface.', name)) ## FIXME
-            if side >= 1:
-                skip_taskbar = True
-            else:
-                skip_taskbar = False
-            gtk_window = self.create_gtk_window(title, skip_taskbar)
-            seriko = self.get_seriko(surface)
-            tooltips = {}
-            if name in self.__tooltips:
-                tooltips = self.__tooltips[name]
-            surface_window = SurfaceWindow(
-                gtk_window, side, self.__sakura, self.prefs, desc, alias,
-                surface, tooltips, pixbufs, seriko, region, mayuna, bind[name],
-                default, self.__use_pna, self.__alpha_channel, self.debug)
-            self.window.append(surface_window)
         self.__surface = surface
-        self.__menu = ninix.menu.Menu(self.__sakura, self)
+        self.add_window(0, '0', alias0, mayuna, bind['sakura'])
+        self.add_window(1, '10', alias1, mayuna, bind['kero'])
+        menu_callback = {
+            'stick_window': self.window_stick,
+            'toggle_bind': self.toggle_bind,
+            }
+        for key in ['network_update', 'vanish_sakura', 'edit_preferences',
+                    'open_ghost_manager', 'show_usage', 'about',
+                    'close_sakura', 'close_all', 'getstring',
+                    'notify_site_selection', 'get_ghost_list', 'select_sakura',
+                    'start_sakura', 'get_shell_list', 'get_balloon_list',
+                    'get_own_balloon_name', 'select_balloon',
+                    'get_current_balloon_name', 'get_current_item',
+                    'get_plugin_list', 'select_plugin',
+                    'get_nekodorif_list', 'select_nekodorif',
+                    'get_kinoko_list', 'select_kinoko']:
+            menu_callback[key] = self.callback[key]
+        self.__menu = ninix.menu.Menu(menu_callback)
         top_dir = self.prefix
         name = self.desc.get('menu.background.bitmap.filename',
                              'menu_background.png')
         name = name.replace('\\', '/')
         path = os.path.join(top_dir, name)
         if not os.path.exists(path):
-            top_dir = os.path.join(self.__sakura.get_prefix(),
+            top_dir = os.path.join(self.callback['get_prefix'](),
                                    'ghost', 'master')
             path = os.path.join(top_dir, 'menu_background.png')
             if os.path.exists(path):
@@ -382,30 +374,47 @@ class Surface:
         if path_background:
             self.__menu.set_pixmap(
                 path_background, path_sidebar, path_foreground)
-        fontcolor_r = self.desc.getint('menu.background.font.color.r', 0)
-        fontcolor_g = self.desc.getint('menu.background.font.color.g', 0)
-        fontcolor_b = self.desc.getint('menu.background.font.color.b', 0)
+        fontcolor_r = self.desc.get_with_type('menu.background.font.color.r', int, 0)
+        fontcolor_g = self.desc.get_with_type('menu.background.font.color.g', int, 0)
+        fontcolor_b = self.desc.get_with_type('menu.background.font.color.b', int, 0)
         background = (fontcolor_r, fontcolor_g, fontcolor_b)
-        fontcolor_r = self.desc.getint('menu.foreground.font.color.r', 0)
-        fontcolor_g = self.desc.getint('menu.foreground.font.color.g', 0)
-        fontcolor_b = self.desc.getint('menu.foreground.font.color.b', 0)
+        fontcolor_r = self.desc.get_with_type('menu.foreground.font.color.r', int, 0)
+        fontcolor_g = self.desc.get_with_type('menu.foreground.font.color.g', int, 0)
+        fontcolor_b = self.desc.get_with_type('menu.foreground.font.color.b', int, 0)
         foreground = (fontcolor_r, fontcolor_g, fontcolor_b)
         self.__menu.set_fontcolor(background, foreground)
         self.__menu.create_mayuna_menu(self.get_mayuna_menu())
 
-    def add_window(self, side, default):
-        assert side >= 2 and len(self.window) == side ## FIXME
-        name = 'char%d' % side
-        title = ''.join(('surface.', name))
-        gtk_window = self.create_gtk_window(title, 1)
+    def add_window(self, side, default, alias=None, mayuna={}, bind={}):
+        assert len(self.window) == side
+        if side == 0:
+            name = 'sakura'
+            title = self.callback['get_selfname']() or \
+                ''.join(('surface.', name))
+        elif side == 1:
+            name = 'kero'
+            title = self.callback['get_keroname']() or \
+                ''.join(('surface.', name))
+        else:
+            name = 'char%d' % side
+            title = ''.join(('surface.', name))
+        skip_taskbar = bool(side >= 1)
+        gtk_window = self.create_gtk_window(title, skip_taskbar)
         seriko = self.get_seriko(self.__surface)
         tooltips = {}
         if name in self.__tooltips:
             tooltips = self.__tooltips[name]
+        surface_window_callback = {}
+        for key in ['enqueue_event', 'notify_observer', 'get_preference',
+                    'set_balloon_direction', 'reset_idle_time',
+                    'notify_surface_click', 'notify_event', 'busy',
+                    'notify_surface_mouse_motion']:
+            surface_window_callback[key] = self.callback[key]
         surface_window = SurfaceWindow(
-            gtk_window, side, self.__sakura, self.prefs, self.desc, None,
+            gtk_window, side, surface_window_callback, self.desc, alias,
             self.__surface, tooltips, self.__pixbufs, seriko, self.__region,
-            {}, {}, default, self.__use_pna, self.__alpha_channel, self.debug)
+            mayuna, bind, default, self.__use_pna, self.__alpha_channel,
+            self.debug)
         self.window.append(surface_window)
         surface_window.set_scale(self.__scale)
         ##surface_window.set_surface(default)
@@ -529,7 +538,7 @@ class Surface:
             x, y = self.get_position(side)
             sox, soy = self.window[side].get_surface_offset()
             w, h = self.get_surface_size(side)
-            bw, bh = self.__sakura.get_balloon_size(side)
+            bw, bh = self.callback['get_balloon_size'](side)
             ox, oy = self.get_balloon_offset(side)
             direction = self.get_direction(side)
             align = self.get_alignment(side)
@@ -548,7 +557,7 @@ class Surface:
                     by = min(y + soy + oy, top + scrn_h - bh)
             self.set_position(side, x, y)
             self.set_direction(side, direction)
-            self.__sakura.set_balloon_position(side, bx, by)
+            self.callback['set_balloon_position'](side, bx, by)
 
     def reset_position(self):
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
@@ -561,13 +570,13 @@ class Surface:
                     y = top
                 else:
                     y = top + scrn_h - h
-                bw, bh = self.__sakura.get_balloon_size(side)
+                bw, bh = self.callback['get_balloon_size'](side)
                 ox, oy = self.get_balloon_offset(side)
                 direction = 0 # left
                 bx = max(x - bw + ox, left)
             else:
-                b0w, b0h = self.__sakura.get_balloon_size(side - 1)
-                b1w, b1h = self.__sakura.get_balloon_size(side)
+                b0w, b0h = self.callback['get_balloon_size'](side - 1)
+                b1w, b1h = self.callback['get_balloon_size'](side)
                 o0x, o0y = self.get_balloon_offset(side - 1)
                 o1x, o1y = self.get_balloon_offset(side)
                 w, h = self.get_surface_size(side)
@@ -593,14 +602,14 @@ class Surface:
                     by = min(y + oy, top + scrn_h - bh)
             self.set_position(side, x, y)
             self.set_direction(side, direction)
-            self.__sakura.set_balloon_position(side, bx, by)
-            s0x, s0y, s0w, s0h = x, y, w, h
-        ##self.__sakura.notify_observer('set position')
+            self.callback['set_balloon_position'](side, bx, by)
+            s0x, s0y, s0w, s0h = x, y, w, h ## FIXME
+        ##self.callback['notify_observer']('set position')
 
     def set_position(self, side, x, y):
         if len(self.window) > side:
             self.window[side].set_position(x, y)
-        self.__sakura.notify_observer('set position')
+        self.callback['notify_observer']('set position')
 
     def get_position(self, side):
         if len(self.window) > side:
@@ -639,38 +648,38 @@ class Surface:
     def show(self, side):
         if len(self.window) > side:
             self.window[side].show()
-            self.__sakura.notify_observer('show')
-            self.__sakura.notify_observer('raise', (side)) ## FIXME
+            self.callback['notify_observer']('show')
+            self.callback['notify_observer']('raise', (side)) # XXX
 
     def hide_all(self):
         for side in range(len(self.window)):
             self.window[side].hide()
-        self.__sakura.notify_observer('hide')
+        self.callback['notify_observer']('hide')
 
     def hide(self, side):
         if len(self.window) > side:
             self.window[side].hide()
-        self.__sakura.notify_observer('hide')
+        self.callback['notify_observer']('hide')
 
     def raise_all(self):
         for side in range(len(self.window)):
             self.window[side].raise_()
-            self.__sakura.notify_observer('raise', (side))
+            self.callback['notify_observer']('raise', (side))
 
     def raise_(self, side):
         if len(self.window) > side:
             self.window[side].raise_()
-            self.__sakura.notify_observer('raise', (side))
+            self.callback['notify_observer']('raise', (side))
 
     def lower_all(self):
         for side in range(len(self.window)):
             self.window[side].lower()
-        self.__sakura.notify_observer('lower')
+        self.callback['notify_observer']('lower')
 
     def lower(self, side):
         if len(self.window) > side:
             self.window[side].lower()
-        self.__sakura.notify_observer('lower')
+        self.callback['notify_observer']('lower')
 
     def invoke(self, side, actor_id):
         if len(self.window) > side:
@@ -690,12 +699,11 @@ class Surface:
         pixbuf = None
         if path is not None:
             try:
-                pixbuf = ninix.pix.create_pixbuf_from_file(path, is_pnr=0)
+                pixbuf = ninix.pix.create_pixbuf_from_file(path, is_pnr=False)
             except:
                 pixbuf = None
         for window in self.window:
             window.window.set_icon(pixbuf)
-        del pixbuf
 
     def get_mikire_kasanari(self):
         if not self.is_shown(0):
@@ -775,13 +783,12 @@ class SurfaceWindow:
         ('text/plain', 0, 0),
         ]
 
-    def __init__(self, window, side, sakura, prefs, desc, alias, surface, tooltips,
+    def __init__(self, window, side, callback, desc, alias, surface, tooltips,
                  pixbuf, seriko, region, mayuna, bind,
                  default_id, use_pna, alpha, debug):
         self.window = window
         self.side = side
-        self.__sakura = sakura
-        self.prefs = prefs
+        self.callback = callback
         self.desc = desc
         self.alias = alias
         self.tooltips = tooltips
@@ -789,7 +796,6 @@ class SurfaceWindow:
         self.__scale = 100 # %
         self.__use_pna = use_pna
         self.__alpha_channel = alpha
-        self.__position_dirty = False
         self.__current_part = ''
         if self.alias is not None:
             default_id = self.alias.get(default_id, [default_id])[0]
@@ -810,7 +816,7 @@ class SurfaceWindow:
         self.set_direction(0)
         self.window.connect('leave_notify_event', self.window_leave_notify) # XXX
         self.window.connect('enter_notify_event', self.window_enter_notify) # XXX
-        self.window.connect('focus_in_event', self.focus_in)
+        self.window.connect('configure_event', self.configure)
         # create drawing area
         self.darea = gtk.DrawingArea()
         self.darea.show()
@@ -820,7 +826,6 @@ class SurfaceWindow:
                               gtk.gdk.POINTER_MOTION_MASK|
                               gtk.gdk.POINTER_MOTION_HINT_MASK|
                               gtk.gdk.SCROLL_MASK)
-        self.callbacks = []
         for signal, func in [('expose_event',         self.redraw),
                              ('button_press_event',   self.button_press),
                              ('button_release_event', self.button_release),
@@ -828,7 +833,7 @@ class SurfaceWindow:
                              ('drag_data_received',   self.drag_data_received),
                              ('scroll_event',         self.scroll),
                              ]:
-            self.callbacks.append(self.darea.connect(signal, func))
+            self.darea.connect(signal, func)
         self.darea.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.dnd_targets,
                                  gtk.gdk.ACTION_COPY)
         self.window.add(self.darea)
@@ -840,10 +845,7 @@ class SurfaceWindow:
         self.debug = debug
 
     def set_use_pna(self, flag):
-        if flag:
-            self.__use_pna = True
-        else:
-            self.__use_pna = False
+        self.__use_pna = bool(flag)
         self.reset_pixbuf_cache()
 
     def set_alpha_channel(self, value):
@@ -867,17 +869,18 @@ class SurfaceWindow:
     def drag_data_received(self, widget, context, x, y, data, info, time):
         if self.debug & 8192:
             print 'Content-type:', data.type
-            print 'Content-length:', data.length
+            print 'Content-length:', data.get_length()
             print repr(data.data)
         if str(data.type) == 'text/plain':
             filelist = []
             for line in data.data.split('\r\n'):
                 scheme, host, path, params, query, fragment = \
                         urlparse.urlparse(line)
-                if scheme == 'file' and os.path.exists(path):
-                    filelist.append(path)
+                pathname = urllib.url2pathname(path)
+                if scheme == 'file' and os.path.exists(pathname):
+                    filelist.append(pathname)
             if filelist:
-                self.__sakura.enqueue_event(
+                self.callback['enqueue_event'](
                     'OnFileDrop2', chr(1).join(filelist), self.side)
         return True
 
@@ -937,7 +940,7 @@ class SurfaceWindow:
         # relocate window
         self.set_position(x, y)
         if self.side < 2:
-            self.__sakura.notify_observer('set surface')
+            self.callback['notify_observer']('set surface')
 
     def iter_mayuna(self, surface_width, surface_height, mayuna, done):
         for surface, interval, method, args in mayuna.patterns:
@@ -1042,7 +1045,6 @@ class SurfaceWindow:
             overlay.composite(pixbuf, dest_x, dest_y,
                               w, h, x, y, 1.0, 1.0,
                               gtk.gdk.INTERP_BILINEAR, 255)
-            del overlay
         return pixbuf
 
     def get_pixbuf(self, pixbuf_id):
@@ -1060,12 +1062,10 @@ class SurfaceWindow:
         surface_gc = self.darea.window.new_gc()
         surface_gc.function = gtk.gdk.INVERT
         for part, x1, y1, x2, y2 in self.collisions:
-            scale = self.__scale
-            if scale != 100:
-                x1 = x1 * scale / 100
-                x2 = x2 * scale / 100
-                y1 = y1 * scale / 100
-                y2 = y2 * scale / 100
+            x1 = x1 * self.__scale / 100
+            x2 = x2 * self.__scale / 100
+            y1 = y1 * self.__scale / 100
+            y2 = y2 * self.__scale / 100
             self.darea.window.draw_rectangle(
                 surface_gc, 0, x1, y1, x2 - x1, y2 - y1)
 
@@ -1107,7 +1107,6 @@ class SurfaceWindow:
                                                     surface_array[y][x][3] = 0 # alpha
                             else:
                                 raise RuntimeError, 'should not reach here'
-                            del pixbuf
                 self.mayuna_cache[surface_id] = surface_pixbuf.copy()
         else:
             surface_pixbuf =self.get_pixbuf(surface_id)
@@ -1148,7 +1147,6 @@ class SurfaceWindow:
                     pixbuf.composite(surface_pixbuf, dest_x, dest_y,
                                      w, h, x, y, 1.0, 1.0,
                                      gtk.gdk.INTERP_BILINEAR, 255)
-                    del pixbuf
         else:
             surface_pixbuf = None
         self.frame_buffer.append((frame, move, surface_pixbuf, dirty))
@@ -1163,14 +1161,12 @@ class SurfaceWindow:
                 return
         else:
             surface_pixbuf = self.create_surface_pixbuf()
-        scale = self.__scale
         w = surface_pixbuf.get_width()
         h = surface_pixbuf.get_height()
-        if scale != 100:
-            w = max(1, w * scale / 100)
-            h = max(1, h * scale / 100)
-            surface_pixbuf = surface_pixbuf.scale_simple(
-                w, h, gtk.gdk.INTERP_BILINEAR)
+        w = max(8, w * self.__scale / 100)
+        h = max(8, h * self.__scale / 100)
+        surface_pixbuf = surface_pixbuf.scale_simple(
+            w, h, gtk.gdk.INTERP_BILINEAR)
         mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
         surface_pixbuf.render_threshold_alpha(
             mask_pixmap, 0, 0, 0, 0, w, h, 1)
@@ -1178,7 +1174,6 @@ class SurfaceWindow:
             self.window.input_shape_combine_mask(mask_pixmap, 0, 0)
         else:
             self.window.shape_combine_mask(mask_pixmap, 0, 0)
-        del mask_pixmap
         self.current_surface_pixbuf = surface_pixbuf
         self.darea.queue_draw()
 
@@ -1189,7 +1184,7 @@ class SurfaceWindow:
         cr.paint()
         cr.set_source_pixbuf(self.current_surface_pixbuf, 0, 0)
         cr.paint_with_alpha(self.__alpha_channel)
-        if self.prefs.get('check_collision'):
+        if self.callback['get_preference']('check_collision'):
             self.draw_region()
 
     def remove_overlay(self, actor):
@@ -1207,41 +1202,37 @@ class SurfaceWindow:
         self.__move(xoffset, yoffset)
         if self.side < 2:
             args = (self.side, xoffset, yoffset)
-            self.__sakura.notify_observer('move surface', args) # animation
+            self.callback['notify_observer']('move surface', args) # animation
 
     def get_balloon_offset(self):
         path, config = self.surface[''.join(('surface', self.surface_id))]
         side = self.side
         if side == 0:
             name = 'sakura'
-            x = config.getint('%s.balloon.offsetx' % name)
-            y = config.getint('%s.balloon.offsety' % name)
+            x = config.get_with_type('%s.balloon.offsetx' % name, int)
+            y = config.get_with_type('%s.balloon.offsety' % name, int)
         elif side == 1:
             name = 'kero'
-            x = config.getint('%s.balloon.offsetx' % name)
-            y = config.getint('%s.balloon.offsety' % name)
+            x = config.get_with_type('%s.balloon.offsetx' % name, int)
+            y = config.get_with_type('%s.balloon.offsety' % name, int)
         else:
             name = 'char%d' % side
             x, y = None, None # XXX
         if x is None:
-            x = self.desc.getint('%s.balloon.offsetx' % name, 0)
+            x = self.desc.get_with_type('%s.balloon.offsetx' % name, int, 0)
         if y is None:
-            y = self.desc.getint('%s.balloon.offsety' % name, 0)
-        scale = self.__scale
-        if scale != 100:
-            x = x * scale / 100
-            y = y * scale / 100
+            y = self.desc.get_with_type('%s.balloon.offsety' % name, int, 0)
+        x *= self.__scale / 100
+        y *= self.__scale / 100
         return x, y
 
     def get_collision_area(self, part):
         for p, x1, y1, x2, y2 in self.collisions: ## FIXME
             if p == part:
-                scale = self.__scale
-                if scale != 100:
-                    x1 = x1 * scale / 100
-                    x2 = x2 * scale / 100
-                    y1 = y1 * scale / 100
-                    y2 = y2 * scale / 100
+                x1 = x1 * self.__scale / 100
+                x2 = x2 * self.__scale / 100
+                y1 = y1 * self.__scale / 100
+                y2 = y2 * self.__scale / 100
                 return x1, y1, x2, y2
         return None
 
@@ -1254,10 +1245,8 @@ class SurfaceWindow:
         if surface_id not in self.pixbuf_cache: # XXX
             self.get_pixbuf(surface_id)
         w, h = self.pixbuf_cache[surface_id][1]
-        scale = self.__scale
-        if scale != 100:
-            w = int(w * scale / 100)
-            h = int(h * scale / 100)
+        w = max(8, int(w * self.__scale / 100))
+        h = max(8, int(h * self.__scale / 100))
         return w, h
 
     def get_surface_offset(self):
@@ -1270,25 +1259,22 @@ class SurfaceWindow:
                 return part
         return ''
 
-    def get_config_int(self, name, scaling=False):
+    def __get_with_scaling(self, name, conv):
         basename = ''.join(('surface', self.surface_id))
         path, config = self.surface[basename]
-        value = config.getint(name)
+        value = config.get_with_type(name, conv)
         if value is not None:
-            if scaling:
-                scale = self.__scale
-                if scale != 100:
-                    value = int(value * scale / 100)
+            value = conv(value * self.__scale / 100)
         return value
 
     def get_center(self):
-        centerx = self.get_config_int('point.centerx', scaling=True)
-        centery = self.get_config_int('point.centery', scaling=True)
+        centerx = self.__get_with_scaling('point.centerx', int)
+        centery = self.__get_with_scaling('point.centery', int)
         return centerx, centery
 
     def get_kinoko_center(self):
-        centerx = self.get_config_int('point.kinoko.centerx', scaling=True)
-        centery = self.get_config_int('point.kinoko.centery', scaling=True)
+        centerx = self.__get_with_scaling('point.kinoko.centerx', int)
+        centery = self.__get_with_scaling('point.kinoko.centery', int)
         return centerx, centery
 
     def get_direction(self):
@@ -1296,7 +1282,7 @@ class SurfaceWindow:
 
     def set_direction(self, direction):
         self.direction = direction # 0: left, 1: right
-        self.__sakura.set_balloon_direction(self.side, direction)
+        self.callback['set_balloon_direction'](self.side, direction)
 
     def set_position(self, x, y):
         self.position = (x, y)
@@ -1310,9 +1296,6 @@ class SurfaceWindow:
         self.set_direction(direction)
 
     def update_position(self):
-        if not self.__position_dirty:
-            return
-        self.__position_dirty = False
         x, y = self.window.get_position()
         p, q = self.window_offset
         w, h = self.get_surface_size()
@@ -1328,7 +1311,7 @@ class SurfaceWindow:
         old_x, old_y = self.get_position()
         if new_x != old_x or new_y != old_y or new_y != y - q:
             self.set_position(new_x, new_y)
-        self.__sakura.notify_observer('set position')
+            self.callback['notify_observer']('set position')
 
     def get_position(self):
         return self.position
@@ -1358,8 +1341,6 @@ class SurfaceWindow:
 
     def destroy(self):
         self.reset_pixbuf_cache()
-        for tag in self.callbacks:
-            self.darea.disconnect(tag)
         self.seriko.terminate(self)
         self.window.remove(self.darea)
         self.darea.destroy()
@@ -1388,25 +1369,19 @@ class SurfaceWindow:
         self.window.window.lower()
 
     def button_press(self, window, event):
-        self.__sakura.reset_idle_time()
+        self.callback['reset_idle_time']()
         x = int(event.x)
         y = int(event.y)
-        scale = self.__scale
-        if scale != 100:
-            x = int(x * 100 / scale)
-            y = int(y * 100 / scale)
-        if event.type == gtk.gdk.BUTTON_PRESS:
-            click = 1
-            if event.button == 1:
-                self.__position_dirty = True
-                self.window.begin_move_drag(
-                    event.button, int(event.x_root), int(event.y_root),
-                    event.time)
-        else:
-            click = 2
-        self.__sakura.notify_observer('raise', (self.side)) ## FIXME: automagical raise 
-        self.__sakura.notify_surface_click(
+        x = int(x * 100 / self.__scale)
+        y = int(y * 100 / self.__scale)
+        click = 1 if event.type == gtk.gdk.BUTTON_PRESS else 2
+        # automagical raise
+        self.callback['notify_observer']('raise', (self.side))
+        self.callback['notify_surface_click'](
             event.button, click, self.side, x, y)
+        if click == 1 and event.button == 1:
+            self.window.begin_move_drag(
+                event.button, int(event.x_root), int(event.y_root), event.time)
         return True
 
     def button_release(self, window, event):
@@ -1417,16 +1392,14 @@ class SurfaceWindow:
             x, y, state = self.darea.window.get_pointer()
         else:
             x, y, state = event.x, event.y, event.state
-        scale = self.__scale
-        if scale != 100:
-            x = int(x * 100 / scale)
-            y = int(y * 100 / scale)
+        x = int(x * 100 / self.__scale)
+        y = int(y * 100 / self.__scale)
         part = self.get_touched_region(x, y)
         if part != self.__current_part:
             if part == '':
                 self.window.set_tooltip_text(None)
                 self.darea.window.set_cursor(None)
-                self.__sakura.notify_event(
+                self.callback['notify_event'](
                     'OnMouseLeave', x, y, '', self.side, self.__current_part)
             else:
                 if part in self.tooltips:
@@ -1436,26 +1409,24 @@ class SurfaceWindow:
                     self.window.set_tooltip_text(None)
                 cursor = gtk.gdk.Cursor(gtk.gdk.HAND1)
                 self.darea.window.set_cursor(cursor)
-                self.__sakura.notify_event(
+                self.callback['notify_event'](
                     'OnMouseEnter', x, y, '', self.side, part)
         self.__current_part = part
-        if not self.__sakura.busy():
+        if not self.callback['busy']():
             if state & gtk.gdk.BUTTON1_MASK or \
                state & gtk.gdk.BUTTON2_MASK or \
                state & gtk.gdk.BUTTON3_MASK:
                 pass
             else:
-                self.__sakura.notify_surface_mouse_motion(
+                self.callback['notify_surface_mouse_motion'](
                     self.side, x, y, part)
         return True
 
     def scroll(self, darea, event):
         x = int(event.x)
         y = int(event.y)
-        scale = self.__scale
-        if scale != 100:
-            x = int(x * 100 / scale)
-            y = int(y * 100 / scale)
+        x = int(x * 100 / self.__scale)
+        y = int(y * 100 / self.__scale)
         if event.direction == gtk.gdk.SCROLL_UP:
             count = 1
         elif event.direction == gtk.gdk.SCROLL_DOWN:
@@ -1464,8 +1435,8 @@ class SurfaceWindow:
             count = 0
         if count != 0:
             part = self.get_touched_region(x, y)
-            self.__sakura.notify_event('OnMouseWheel',
-                                       x, y, count, self.side, part)
+            self.callback['notify_event'](
+                'OnMouseWheel', x, y, count, self.side, part)
         return True
 
     def toggle_bind(self, bind_id):
@@ -1477,28 +1448,26 @@ class SurfaceWindow:
 
     def window_enter_notify(self, window, event):
         x, y, state = event.x, event.y, event.state
-        scale = self.__scale
-        if scale != 100:
-            x = int(x * 100 / scale)
-            y = int(y * 100 / scale)
-        self.__sakura.notify_event('OnMouseEnterAll', x, y, '', self.side, '')
+        x = int(x * 100 / self.__scale)
+        y = int(y * 100 / self.__scale)
+        self.callback['notify_event'](
+            'OnMouseEnterAll', x, y, '', self.side, '')
 
     def window_leave_notify(self, window, event):
         x, y, state = event.x, event.y, event.state
-        scale = self.__scale
-        if scale != 100:
-            x = int(x * 100 / scale)
-            y = int(y * 100 / scale)
+        x = int(x * 100 / self.__scale)
+        y = int(y * 100 / self.__scale)
         if self.__current_part != '': # XXX
-            self.__sakura.notify_event(
+            self.callback['notify_event'](
                 'OnMouseLeave', x, y, '', self.side, self.__current_part)
             self.__current_part = ''
-        self.__sakura.notify_event('OnMouseLeaveAll', x, y, '', self.side, '')
+        self.callback['notify_event'](
+            'OnMouseLeaveAll', x, y, '', self.side, '')
         return True
 
-    def focus_in(self, window, event): # This gets called when a drag ends.
-        self.update_position()
-        return True
+    def configure(self, window, event):
+        self.update_position() # See ICCCM 4.1.5.
+        return False
 
 
 def test():
