@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -15,7 +15,7 @@
 import urlparse
 import StringIO
 import os
-import md5
+import hashlib
 import time
 import httplib
 
@@ -24,8 +24,8 @@ class NetworkUpdate:
 
     __BACKUP_SUFFIX = '.BACKUP'
 
-    def __init__(self, sakura):
-        self.__sakura = sakura
+    def __init__(self, callback):
+        self.callback = callback
         self.event_queue = []
         self.state = None
         self.backups = []
@@ -71,8 +71,7 @@ class NetworkUpdate:
 
     def interrupt(self):
         self.event_queue = []
-        if self.__sakura:
-            self.__sakura.enqueue_event('OnUpdateFailure', 'artificial')
+        self.callback['enqueue_event']('OnUpdateFailure', 'artificial')
         self.state = None
         self.stop(revert=1)
 
@@ -103,7 +102,7 @@ class NetworkUpdate:
         return time.time() - self.timestamp > self.timeout
 
     def run(self):
-        if self.state is None or (self.__sakura and self.__sakura.event_queue):
+        if self.state is None or self.callback['check_event_queue']():
             return 0
         elif self.state == 0:
             self.start_updates()
@@ -296,7 +295,7 @@ class NetworkUpdate:
             except IOError:
                 pass
             else:
-                m = md5.new()
+                m = hashlib.md5()
                 m.update(data)
                 if checksum == self.make_checksum(m.digest()):
                     continue
@@ -306,7 +305,7 @@ class NetworkUpdate:
 
     def update_file(self, filename, checksum):
         data = ''.join(self.buffer)
-        m = md5.new()
+        m = hashlib.md5()
         m.update(data)
         digest = self.make_checksum(m.digest())
         if digest == checksum:
@@ -399,7 +398,8 @@ def test():
     import sys
     if len(sys.argv) != 3:
         raise SystemExit, 'Usage: update.py homeurl ghostdir\n'
-    update = NetworkUpdate(None)
+    update = NetworkUpdate({'enqueu_event': lambda *a: None,
+                            'check_event_queue': lambda *a: None,})
     update.start(sys.argv[1], sys.argv[2], timeout=60)
     while 1:
         state = update.state
